# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../mw4/gui/widgets/hemisphere.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_HemisphereDialog(object):
    def setupUi(self, HemisphereDialog):
        HemisphereDialog.setObjectName("HemisphereDialog")
        HemisphereDialog.resize(800, 600)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(HemisphereDialog.sizePolicy().hasHeightForWidth())
        HemisphereDialog.setSizePolicy(sizePolicy)
        HemisphereDialog.setMinimumSize(QtCore.QSize(800, 400))
        HemisphereDialog.setMaximumSize(QtCore.QSize(1600, 1200))
        HemisphereDialog.setSizeIncrement(QtCore.QSize(10, 10))
        HemisphereDialog.setBaseSize(QtCore.QSize(10, 10))
        font = QtGui.QFont()
        font.setFamily("Arial")
        HemisphereDialog.setFont(font)
        self.verticalLayout = QtWidgets.QVBoxLayout(HemisphereDialog)
        self.verticalLayout.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout.setSpacing(4)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(4, 4, 4, 4)
        self.horizontalLayout.setSpacing(4)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.groupBox_3 = QtWidgets.QGroupBox(HemisphereDialog)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout.setContentsMargins(10, 10, 10, 10)
        self.gridLayout.setSpacing(10)
        self.gridLayout.setObjectName("gridLayout")
        self.checkShowMeridian = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkShowMeridian.setFont(font)
        self.checkShowMeridian.setObjectName("checkShowMeridian")
        self.gridLayout.addWidget(self.checkShowMeridian, 1, 0, 1, 1)
        self.checkShowCelestial = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkShowCelestial.setFont(font)
        self.checkShowCelestial.setObjectName("checkShowCelestial")
        self.gridLayout.addWidget(self.checkShowCelestial, 2, 0, 1, 1)
        self.checkUseHorizon = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkUseHorizon.setFont(font)
        self.checkUseHorizon.setChecked(False)
        self.checkUseHorizon.setObjectName("checkUseHorizon")
        self.gridLayout.addWidget(self.checkUseHorizon, 0, 1, 1, 1)
        self.checkShowAlignStar = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkShowAlignStar.setFont(font)
        self.checkShowAlignStar.setObjectName("checkShowAlignStar")
        self.gridLayout.addWidget(self.checkShowAlignStar, 0, 0, 1, 1)
        self.checkShowSlewPath = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkShowSlewPath.setFont(font)
        self.checkShowSlewPath.setObjectName("checkShowSlewPath")
        self.gridLayout.addWidget(self.checkShowSlewPath, 3, 0, 1, 1)
        self.horizontalLayout.addWidget(self.groupBox_3)
        self.line_3 = QtWidgets.QFrame(HemisphereDialog)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_3.setMidLineWidth(1)
        self.line_3.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_3.setObjectName("line_3")
        self.horizontalLayout.addWidget(self.line_3)
        self.groupBox_2 = QtWidgets.QGroupBox(HemisphereDialog)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_2.setContentsMargins(10, 10, 10, 10)
        self.gridLayout_2.setSpacing(10)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.checkEditNone = QtWidgets.QRadioButton(self.groupBox_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkEditNone.setFont(font)
        self.checkEditNone.setChecked(True)
        self.checkEditNone.setObjectName("checkEditNone")
        self.gridLayout_2.addWidget(self.checkEditNone, 0, 0, 1, 1)
        self.checkEditBuildPoints = QtWidgets.QRadioButton(self.groupBox_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkEditBuildPoints.setFont(font)
        self.checkEditBuildPoints.setObjectName("checkEditBuildPoints")
        self.gridLayout_2.addWidget(self.checkEditBuildPoints, 1, 0, 1, 1)
        self.checkEditHorizonMask = QtWidgets.QRadioButton(self.groupBox_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkEditHorizonMask.setFont(font)
        self.checkEditHorizonMask.setObjectName("checkEditHorizonMask")
        self.gridLayout_2.addWidget(self.checkEditHorizonMask, 2, 0, 1, 1)
        self.checkPolarAlignment = QtWidgets.QRadioButton(self.groupBox_2)
        self.checkPolarAlignment.setEnabled(True)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkPolarAlignment.setFont(font)
        self.checkPolarAlignment.setObjectName("checkPolarAlignment")
        self.gridLayout_2.addWidget(self.checkPolarAlignment, 3, 0, 1, 1)
        self.horizontalLayout.addWidget(self.groupBox_2)
        self.line_4 = QtWidgets.QFrame(HemisphereDialog)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_4.setMidLineWidth(1)
        self.line_4.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_4.setObjectName("line_4")
        self.horizontalLayout.addWidget(self.line_4)
        self.groupBox = QtWidgets.QGroupBox(HemisphereDialog)
        self.groupBox.setEnabled(True)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_4.setContentsMargins(10, 10, 10, 10)
        self.gridLayout_4.setSpacing(10)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.showAltAz = QtWidgets.QRadioButton(self.groupBox)
        self.showAltAz.setEnabled(True)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.showAltAz.setFont(font)
        self.showAltAz.setObjectName("showAltAz")
        self.gridLayout_4.addWidget(self.showAltAz, 0, 0, 1, 1)
        self.showPolar = QtWidgets.QRadioButton(self.groupBox)
        self.showPolar.setEnabled(False)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.showPolar.setFont(font)
        self.showPolar.setObjectName("showPolar")
        self.gridLayout_4.addWidget(self.showPolar, 1, 0, 1, 1)
        self.showSphere = QtWidgets.QRadioButton(self.groupBox)
        self.showSphere.setEnabled(False)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.showSphere.setFont(font)
        self.showSphere.setObjectName("showSphere")
        self.gridLayout_4.addWidget(self.showSphere, 2, 0, 1, 1)
        self.horizontalLayout.addWidget(self.groupBox)
        self.line_5 = QtWidgets.QFrame(HemisphereDialog)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_5.setMidLineWidth(1)
        self.line_5.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_5.setObjectName("line_5")
        self.horizontalLayout.addWidget(self.line_5)
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.clearBuildP = QtWidgets.QPushButton(HemisphereDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.clearBuildP.sizePolicy().hasHeightForWidth())
        self.clearBuildP.setSizePolicy(sizePolicy)
        self.clearBuildP.setMinimumSize(QtCore.QSize(100, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.clearBuildP.setFont(font)
        self.clearBuildP.setObjectName("clearBuildP")
        self.gridLayout_3.addWidget(self.clearBuildP, 0, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem, 1, 0, 1, 1)
        self.horizontalLayout.addLayout(self.gridLayout_3)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.line = QtWidgets.QFrame(HemisphereDialog)
        self.line.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line.setLineWidth(2)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.hemisphere = QtWidgets.QWidget(HemisphereDialog)
        self.hemisphere.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hemisphere.sizePolicy().hasHeightForWidth())
        self.hemisphere.setSizePolicy(sizePolicy)
        self.hemisphere.setMinimumSize(QtCore.QSize(790, 270))
        self.hemisphere.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.hemisphere.setSizeIncrement(QtCore.QSize(10, 10))
        self.hemisphere.setBaseSize(QtCore.QSize(10, 10))
        self.hemisphere.setAutoFillBackground(True)
        self.hemisphere.setStyleSheet("")
        self.hemisphere.setObjectName("hemisphere")
        self.verticalLayout.addWidget(self.hemisphere)
        self.verticalLayout.setStretch(2, 1)

        self.retranslateUi(HemisphereDialog)
        QtCore.QMetaObject.connectSlotsByName(HemisphereDialog)

    def retranslateUi(self, HemisphereDialog):
        _translate = QtCore.QCoreApplication.translate
        HemisphereDialog.setWindowTitle(_translate("HemisphereDialog", "Hemisphere"))
        self.groupBox_3.setTitle(_translate("HemisphereDialog", "Show Elements"))
        self.checkShowMeridian.setText(_translate("HemisphereDialog", "Meridian limits "))
        self.checkShowCelestial.setText(_translate("HemisphereDialog", "Celestial paths "))
        self.checkUseHorizon.setToolTip(_translate("HemisphereDialog", "<html><head/><body><p>Use horizon mask file in hemisphere window.</p></body></html>"))
        self.checkUseHorizon.setText(_translate("HemisphereDialog", "Horizon mask "))
        self.checkShowAlignStar.setText(_translate("HemisphereDialog", "Align stars "))
        self.checkShowSlewPath.setText(_translate("HemisphereDialog", "Slew path "))
        self.groupBox_2.setTitle(_translate("HemisphereDialog", "Operation Mode"))
        self.checkEditNone.setText(_translate("HemisphereDialog", "Normal mode"))
        self.checkEditBuildPoints.setText(_translate("HemisphereDialog", "Edit Build Points"))
        self.checkEditHorizonMask.setText(_translate("HemisphereDialog", "Edit Horizon Mask "))
        self.checkPolarAlignment.setText(_translate("HemisphereDialog", "Polar/Ortho Align "))
        self.groupBox.setTitle(_translate("HemisphereDialog", "Views / Coords"))
        self.showAltAz.setText(_translate("HemisphereDialog", "Alt/Az Plane"))
        self.showPolar.setText(_translate("HemisphereDialog", "Polar Diagram "))
        self.showSphere.setText(_translate("HemisphereDialog", "Sphere"))
        self.clearBuildP.setToolTip(_translate("HemisphereDialog", "<html><head/><body><p>Delete all points on coordinate window including Base / Model</p></body></html>"))
        self.clearBuildP.setText(_translate("HemisphereDialog", "Clear Points"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    HemisphereDialog = QtWidgets.QWidget()
    ui = Ui_HemisphereDialog()
    ui.setupUi(HemisphereDialog)
    HemisphereDialog.show()
    sys.exit(app.exec_())
