# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../mw4/gui/widgets/image.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ImageDialog(object):
    def setupUi(self, ImageDialog):
        ImageDialog.setObjectName("ImageDialog")
        ImageDialog.resize(800, 600)
        ImageDialog.setMinimumSize(QtCore.QSize(800, 600))
        ImageDialog.setMaximumSize(QtCore.QSize(1600, 1200))
        ImageDialog.setSizeIncrement(QtCore.QSize(10, 10))
        font = QtGui.QFont()
        font.setFamily("Arial")
        ImageDialog.setFont(font)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(ImageDialog)
        self.verticalLayout_2.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout_2.setSpacing(4)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setContentsMargins(4, 4, 4, 4)
        self.horizontalLayout_5.setSpacing(4)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.exposesettings = QtWidgets.QGridLayout()
        self.exposesettings.setContentsMargins(4, 4, 4, 4)
        self.exposesettings.setHorizontalSpacing(8)
        self.exposesettings.setVerticalSpacing(4)
        self.exposesettings.setObjectName("exposesettings")
        self.exposeN = QtWidgets.QPushButton(ImageDialog)
        self.exposeN.setMinimumSize(QtCore.QSize(80, 25))
        self.exposeN.setObjectName("exposeN")
        self.exposesettings.addWidget(self.exposeN, 1, 0, 1, 1)
        self.solve = QtWidgets.QPushButton(ImageDialog)
        self.solve.setMinimumSize(QtCore.QSize(80, 25))
        self.solve.setObjectName("solve")
        self.exposesettings.addWidget(self.solve, 2, 0, 1, 1)
        self.abortSolve = QtWidgets.QPushButton(ImageDialog)
        self.abortSolve.setEnabled(False)
        self.abortSolve.setMinimumSize(QtCore.QSize(80, 25))
        self.abortSolve.setObjectName("abortSolve")
        self.exposesettings.addWidget(self.abortSolve, 2, 1, 1, 1)
        self.abortImage = QtWidgets.QPushButton(ImageDialog)
        self.abortImage.setEnabled(False)
        self.abortImage.setMinimumSize(QtCore.QSize(80, 25))
        self.abortImage.setObjectName("abortImage")
        self.exposesettings.addWidget(self.abortImage, 1, 1, 1, 1)
        self.checkAutoSolve = QtWidgets.QCheckBox(ImageDialog)
        self.checkAutoSolve.setMinimumSize(QtCore.QSize(0, 25))
        self.checkAutoSolve.setObjectName("checkAutoSolve")
        self.exposesettings.addWidget(self.checkAutoSolve, 2, 5, 1, 1)
        self.load = QtWidgets.QPushButton(ImageDialog)
        self.load.setMinimumSize(QtCore.QSize(80, 25))
        self.load.setObjectName("load")
        self.exposesettings.addWidget(self.load, 0, 5, 1, 1)
        self.imageFileName = QtWidgets.QLineEdit(ImageDialog)
        self.imageFileName.setEnabled(False)
        self.imageFileName.setMinimumSize(QtCore.QSize(200, 25))
        self.imageFileName.setObjectName("imageFileName")
        self.exposesettings.addWidget(self.imageFileName, 0, 1, 1, 4)
        self.expose = QtWidgets.QPushButton(ImageDialog)
        self.expose.setMinimumSize(QtCore.QSize(80, 25))
        self.expose.setObjectName("expose")
        self.exposesettings.addWidget(self.expose, 0, 0, 1, 1)
        self.checkStackImages = QtWidgets.QCheckBox(ImageDialog)
        self.checkStackImages.setEnabled(True)
        self.checkStackImages.setObjectName("checkStackImages")
        self.exposesettings.addWidget(self.checkStackImages, 1, 5, 1, 1)
        self.horizontalLayout_5.addLayout(self.exposesettings)
        self.line_2 = QtWidgets.QFrame(ImageDialog)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_2.setMidLineWidth(1)
        self.line_2.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_2.setObjectName("line_2")
        self.horizontalLayout_5.addWidget(self.line_2)
        self.groupBox_3 = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox_3.setObjectName("groupBox_3")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_5.setContentsMargins(10, 4, 10, 4)
        self.verticalLayout_5.setSpacing(4)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.checkUseWCS = QtWidgets.QCheckBox(self.groupBox_3)
        self.checkUseWCS.setMinimumSize(QtCore.QSize(80, 25))
        self.checkUseWCS.setObjectName("checkUseWCS")
        self.verticalLayout_5.addWidget(self.checkUseWCS)
        self.line_5 = QtWidgets.QFrame(self.groupBox_3)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_5.setMidLineWidth(1)
        self.line_5.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_5.setObjectName("line_5")
        self.verticalLayout_5.addWidget(self.line_5)
        self.checkShowGrid = QtWidgets.QCheckBox(self.groupBox_3)
        self.checkShowGrid.setObjectName("checkShowGrid")
        self.verticalLayout_5.addWidget(self.checkShowGrid)
        self.checkShowCrosshair = QtWidgets.QCheckBox(self.groupBox_3)
        self.checkShowCrosshair.setObjectName("checkShowCrosshair")
        self.verticalLayout_5.addWidget(self.checkShowCrosshair)
        self.horizontalLayout_5.addWidget(self.groupBox_3)
        self.line_4 = QtWidgets.QFrame(ImageDialog)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_4.setMidLineWidth(1)
        self.line_4.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_4.setObjectName("line_4")
        self.horizontalLayout_5.addWidget(self.line_4)
        self.attributsgroup = QtWidgets.QGroupBox(ImageDialog)
        self.attributsgroup.setObjectName("attributsgroup")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.attributsgroup)
        self.verticalLayout_6.setContentsMargins(10, 4, 15, 4)
        self.verticalLayout_6.setSpacing(4)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        spacerItem = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_6.addItem(spacerItem)
        self.hasWCS = QtWidgets.QLabel(self.attributsgroup)
        self.hasWCS.setMinimumSize(QtCore.QSize(0, 16))
        self.hasWCS.setObjectName("hasWCS")
        self.verticalLayout_6.addWidget(self.hasWCS)
        self.hasDistortion = QtWidgets.QLabel(self.attributsgroup)
        self.hasDistortion.setEnabled(True)
        self.hasDistortion.setObjectName("hasDistortion")
        self.verticalLayout_6.addWidget(self.hasDistortion)
        self.isFlipped = QtWidgets.QLabel(self.attributsgroup)
        self.isFlipped.setObjectName("isFlipped")
        self.verticalLayout_6.addWidget(self.isFlipped)
        spacerItem1 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_6.addItem(spacerItem1)
        self.verticalLayout_6.setStretch(0, 1)
        self.verticalLayout_6.setStretch(4, 1)
        self.horizontalLayout_5.addWidget(self.attributsgroup)
        self.line_3 = QtWidgets.QFrame(ImageDialog)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_3.setMidLineWidth(1)
        self.line_3.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_3.setObjectName("line_3")
        self.horizontalLayout_5.addWidget(self.line_3)
        self.imagesettings = QtWidgets.QGroupBox(ImageDialog)
        self.imagesettings.setObjectName("imagesettings")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.imagesettings)
        self.verticalLayout.setContentsMargins(4, 6, 4, 4)
        self.verticalLayout.setSpacing(4)
        self.verticalLayout.setObjectName("verticalLayout")
        self.stretch = QtWidgets.QComboBox(self.imagesettings)
        self.stretch.setMinimumSize(QtCore.QSize(0, 25))
        self.stretch.setObjectName("stretch")
        self.verticalLayout.addWidget(self.stretch)
        self.color = QtWidgets.QComboBox(self.imagesettings)
        self.color.setMinimumSize(QtCore.QSize(0, 25))
        self.color.setObjectName("color")
        self.verticalLayout.addWidget(self.color)
        self.zoom = QtWidgets.QComboBox(self.imagesettings)
        self.zoom.setMinimumSize(QtCore.QSize(0, 25))
        self.zoom.setObjectName("zoom")
        self.verticalLayout.addWidget(self.zoom)
        self.horizontalLayout_5.addWidget(self.imagesettings)
        spacerItem2 = QtWidgets.QSpacerItem(5, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem2)
        self.horizontalLayout_5.setStretch(7, 1)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.line = QtWidgets.QFrame(ImageDialog)
        self.line.setAutoFillBackground(True)
        self.line.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line.setLineWidth(2)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setObjectName("line")
        self.verticalLayout_2.addWidget(self.line)
        self.imagelayout = QtWidgets.QHBoxLayout()
        self.imagelayout.setContentsMargins(2, 2, 2, 2)
        self.imagelayout.setSpacing(2)
        self.imagelayout.setObjectName("imagelayout")
        self.groupBox = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox.setMinimumSize(QtCore.QSize(120, 0))
        self.groupBox.setMaximumSize(QtCore.QSize(100, 16777215))
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setContentsMargins(4, 6, 4, 2)
        self.verticalLayout_3.setSpacing(2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.verticalLayout_3.addWidget(self.label)
        self.object = QtWidgets.QLineEdit(self.groupBox)
        self.object.setObjectName("object")
        self.verticalLayout_3.addWidget(self.object)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_3.addWidget(self.label_2)
        self.ra = QtWidgets.QLineEdit(self.groupBox)
        self.ra.setObjectName("ra")
        self.verticalLayout_3.addWidget(self.ra)
        self.label_3 = QtWidgets.QLabel(self.groupBox)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_3.addWidget(self.label_3)
        self.dec = QtWidgets.QLineEdit(self.groupBox)
        self.dec.setObjectName("dec")
        self.verticalLayout_3.addWidget(self.dec)
        self.label_6 = QtWidgets.QLabel(self.groupBox)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_3.addWidget(self.label_6)
        self.rotation = QtWidgets.QLineEdit(self.groupBox)
        self.rotation.setObjectName("rotation")
        self.verticalLayout_3.addWidget(self.rotation)
        self.label_7 = QtWidgets.QLabel(self.groupBox)
        self.label_7.setObjectName("label_7")
        self.verticalLayout_3.addWidget(self.label_7)
        self.scale = QtWidgets.QLineEdit(self.groupBox)
        self.scale.setObjectName("scale")
        self.verticalLayout_3.addWidget(self.scale)
        self.label_8 = QtWidgets.QLabel(self.groupBox)
        self.label_8.setObjectName("label_8")
        self.verticalLayout_3.addWidget(self.label_8)
        self.expTime = QtWidgets.QLineEdit(self.groupBox)
        self.expTime.setObjectName("expTime")
        self.verticalLayout_3.addWidget(self.expTime)
        self.label_10 = QtWidgets.QLabel(self.groupBox)
        self.label_10.setObjectName("label_10")
        self.verticalLayout_3.addWidget(self.label_10)
        self.filter = QtWidgets.QLineEdit(self.groupBox)
        self.filter.setObjectName("filter")
        self.verticalLayout_3.addWidget(self.filter)
        self.label_11 = QtWidgets.QLabel(self.groupBox)
        self.label_11.setObjectName("label_11")
        self.verticalLayout_3.addWidget(self.label_11)
        self.binX = QtWidgets.QLineEdit(self.groupBox)
        self.binX.setObjectName("binX")
        self.verticalLayout_3.addWidget(self.binX)
        self.label_12 = QtWidgets.QLabel(self.groupBox)
        self.label_12.setObjectName("label_12")
        self.verticalLayout_3.addWidget(self.label_12)
        self.binY = QtWidgets.QLineEdit(self.groupBox)
        self.binY.setObjectName("binY")
        self.verticalLayout_3.addWidget(self.binY)
        self.label_9 = QtWidgets.QLabel(self.groupBox)
        self.label_9.setObjectName("label_9")
        self.verticalLayout_3.addWidget(self.label_9)
        self.ccdTemp = QtWidgets.QLineEdit(self.groupBox)
        self.ccdTemp.setObjectName("ccdTemp")
        self.verticalLayout_3.addWidget(self.ccdTemp)
        self.label_4 = QtWidgets.QLabel(self.groupBox)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_3.addWidget(self.label_4)
        self.sqm = QtWidgets.QLineEdit(self.groupBox)
        self.sqm.setObjectName("sqm")
        self.verticalLayout_3.addWidget(self.sqm)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem3)
        self.imagelayout.addWidget(self.groupBox)
        self.image = QtWidgets.QWidget(ImageDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.image.sizePolicy().hasHeightForWidth())
        self.image.setSizePolicy(sizePolicy)
        self.image.setMinimumSize(QtCore.QSize(0, 0))
        self.image.setObjectName("image")
        self.imagelayout.addWidget(self.image)
        self.verticalLayout_2.addLayout(self.imagelayout)
        self.verticalLayout_2.setStretch(2, 1)

        self.retranslateUi(ImageDialog)
        QtCore.QMetaObject.connectSlotsByName(ImageDialog)

    def retranslateUi(self, ImageDialog):
        _translate = QtCore.QCoreApplication.translate
        ImageDialog.setWindowTitle(_translate("ImageDialog", "Imaging"))
        self.exposeN.setText(_translate("ImageDialog", "Expose N"))
        self.solve.setText(_translate("ImageDialog", "Solve"))
        self.abortSolve.setText(_translate("ImageDialog", "Abort"))
        self.abortImage.setText(_translate("ImageDialog", "Abort"))
        self.checkAutoSolve.setText(_translate("ImageDialog", "Auto Solve"))
        self.load.setText(_translate("ImageDialog", "Load FITS"))
        self.expose.setText(_translate("ImageDialog", "Expose 1"))
        self.checkStackImages.setText(_translate("ImageDialog", "Stack Images"))
        self.groupBox_3.setTitle(_translate("ImageDialog", "Views / Coords"))
        self.checkUseWCS.setText(_translate("ImageDialog", "Show RA / DEC"))
        self.checkShowGrid.setText(_translate("ImageDialog", "Show grid"))
        self.checkShowCrosshair.setText(_translate("ImageDialog", "Show crosshair  "))
        self.attributsgroup.setTitle(_translate("ImageDialog", "Image attributes"))
        self.hasWCS.setText(_translate("ImageDialog", "WCS Header"))
        self.hasDistortion.setText(_translate("ImageDialog", "Distortion Param"))
        self.isFlipped.setText(_translate("ImageDialog", "Is Flipped"))
        self.imagesettings.setTitle(_translate("ImageDialog", "Image settings"))
        self.groupBox.setTitle(_translate("ImageDialog", "Fits Header"))
        self.label.setText(_translate("ImageDialog", "Object Name"))
        self.label_2.setText(_translate("ImageDialog", "RA [hours]"))
        self.label_3.setText(_translate("ImageDialog", "DEC [°]"))
        self.label_6.setText(_translate("ImageDialog", "Rotation [°]"))
        self.label_7.setText(_translate("ImageDialog", "Scale [arcs/pixel]"))
        self.label_8.setText(_translate("ImageDialog", "Exposure Time [s]"))
        self.label_10.setText(_translate("ImageDialog", "Filter"))
        self.label_11.setText(_translate("ImageDialog", "Binning X"))
        self.label_12.setText(_translate("ImageDialog", "Binning Y"))
        self.label_9.setText(_translate("ImageDialog", "CCD Temp [°]"))
        self.label_4.setText(_translate("ImageDialog", "SQM [mpas]"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ImageDialog = QtWidgets.QWidget()
    ui = Ui_ImageDialog()
    ui.setupUi(ImageDialog)
    ImageDialog.show()
    sys.exit(app.exec_())
