# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../mw4/gui/widgets/main.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.setWindowModality(QtCore.Qt.NonModal)
        MainWindow.setEnabled(True)
        MainWindow.resize(800, 600)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        MainWindow.setMinimumSize(QtCore.QSize(800, 600))
        MainWindow.setMaximumSize(QtCore.QSize(800, 1200))
        MainWindow.setSizeIncrement(QtCore.QSize(9, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        MainWindow.setFont(font)
        MainWindow.setCursor(QtGui.QCursor(QtCore.Qt.ArrowCursor))
        MainWindow.setFocusPolicy(QtCore.Qt.NoFocus)
        MainWindow.setContextMenuPolicy(QtCore.Qt.PreventContextMenu)
        MainWindow.setToolTipDuration(6)
        MainWindow.setAutoFillBackground(True)
        self.verticalLayout = QtWidgets.QVBoxLayout(MainWindow)
        self.verticalLayout.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout.setSpacing(4)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(4, 4, 4, 4)
        self.horizontalLayout.setSpacing(4)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.status = QtWidgets.QGridLayout()
        self.status.setContentsMargins(0, -1, -1, -1)
        self.status.setSpacing(4)
        self.status.setObjectName("status")
        self.domeConnected = QtWidgets.QPushButton(MainWindow)
        self.domeConnected.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.domeConnected.sizePolicy().hasHeightForWidth())
        self.domeConnected.setSizePolicy(sizePolicy)
        self.domeConnected.setMinimumSize(QtCore.QSize(0, 25))
        self.domeConnected.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.domeConnected.setFont(font)
        self.domeConnected.setObjectName("domeConnected")
        self.status.addWidget(self.domeConnected, 3, 2, 1, 1)
        self.imagingText = QtWidgets.QLineEdit(MainWindow)
        self.imagingText.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.imagingText.sizePolicy().hasHeightForWidth())
        self.imagingText.setSizePolicy(sizePolicy)
        self.imagingText.setMinimumSize(QtCore.QSize(0, 25))
        self.imagingText.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.imagingText.setFont(font)
        self.imagingText.setMouseTracking(False)
        self.imagingText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.imagingText.setAcceptDrops(False)
        self.imagingText.setText("")
        self.imagingText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.imagingText.setReadOnly(True)
        self.imagingText.setObjectName("imagingText")
        self.status.addWidget(self.imagingText, 2, 3, 1, 1)
        self.mountConnected = QtWidgets.QPushButton(MainWindow)
        self.mountConnected.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mountConnected.sizePolicy().hasHeightForWidth())
        self.mountConnected.setSizePolicy(sizePolicy)
        self.mountConnected.setMinimumSize(QtCore.QSize(0, 25))
        self.mountConnected.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.mountConnected.setFont(font)
        self.mountConnected.setFlat(False)
        self.mountConnected.setObjectName("mountConnected")
        self.status.addWidget(self.mountConnected, 3, 0, 1, 1)
        self.astrometryText = QtWidgets.QLineEdit(MainWindow)
        self.astrometryText.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.astrometryText.sizePolicy().hasHeightForWidth())
        self.astrometryText.setSizePolicy(sizePolicy)
        self.astrometryText.setMinimumSize(QtCore.QSize(0, 25))
        self.astrometryText.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.astrometryText.setFont(font)
        self.astrometryText.setMouseTracking(False)
        self.astrometryText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.astrometryText.setAcceptDrops(False)
        self.astrometryText.setText("")
        self.astrometryText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.astrometryText.setReadOnly(True)
        self.astrometryText.setObjectName("astrometryText")
        self.status.addWidget(self.astrometryText, 2, 4, 1, 1)
        self.imagingConnected = QtWidgets.QPushButton(MainWindow)
        self.imagingConnected.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.imagingConnected.sizePolicy().hasHeightForWidth())
        self.imagingConnected.setSizePolicy(sizePolicy)
        self.imagingConnected.setMinimumSize(QtCore.QSize(0, 25))
        self.imagingConnected.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.imagingConnected.setFont(font)
        self.imagingConnected.setObjectName("imagingConnected")
        self.status.addWidget(self.imagingConnected, 3, 3, 1, 1)
        self.domeText = QtWidgets.QLineEdit(MainWindow)
        self.domeText.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.domeText.sizePolicy().hasHeightForWidth())
        self.domeText.setSizePolicy(sizePolicy)
        self.domeText.setMinimumSize(QtCore.QSize(0, 25))
        self.domeText.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.domeText.setFont(font)
        self.domeText.setMouseTracking(False)
        self.domeText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.domeText.setAcceptDrops(False)
        self.domeText.setText("")
        self.domeText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.domeText.setReadOnly(True)
        self.domeText.setObjectName("domeText")
        self.status.addWidget(self.domeText, 2, 2, 1, 1)
        self.astrometryConnected = QtWidgets.QPushButton(MainWindow)
        self.astrometryConnected.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.astrometryConnected.sizePolicy().hasHeightForWidth())
        self.astrometryConnected.setSizePolicy(sizePolicy)
        self.astrometryConnected.setMinimumSize(QtCore.QSize(0, 25))
        self.astrometryConnected.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.astrometryConnected.setFont(font)
        self.astrometryConnected.setObjectName("astrometryConnected")
        self.status.addWidget(self.astrometryConnected, 3, 4, 1, 1)
        self.environConnected = QtWidgets.QPushButton(MainWindow)
        self.environConnected.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.environConnected.sizePolicy().hasHeightForWidth())
        self.environConnected.setSizePolicy(sizePolicy)
        self.environConnected.setMinimumSize(QtCore.QSize(0, 25))
        self.environConnected.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.environConnected.setFont(font)
        self.environConnected.setObjectName("environConnected")
        self.status.addWidget(self.environConnected, 3, 1, 1, 1)
        self.label_38 = QtWidgets.QLabel(MainWindow)
        self.label_38.setMaximumSize(QtCore.QSize(16777215, 10))
        self.label_38.setObjectName("label_38")
        self.status.addWidget(self.label_38, 0, 0, 1, 5)
        self.openHemisphereW = QtWidgets.QPushButton(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.openHemisphereW.sizePolicy().hasHeightForWidth())
        self.openHemisphereW.setSizePolicy(sizePolicy)
        self.openHemisphereW.setMinimumSize(QtCore.QSize(0, 25))
        self.openHemisphereW.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.openHemisphereW.setFont(font)
        self.openHemisphereW.setStyleSheet("")
        self.openHemisphereW.setObjectName("openHemisphereW")
        self.status.addWidget(self.openHemisphereW, 4, 2, 3, 1)
        self.openSatelliteW = QtWidgets.QPushButton(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.openSatelliteW.sizePolicy().hasHeightForWidth())
        self.openSatelliteW.setSizePolicy(sizePolicy)
        self.openSatelliteW.setMinimumSize(QtCore.QSize(0, 25))
        self.openSatelliteW.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.openSatelliteW.setFont(font)
        self.openSatelliteW.setStyleSheet("")
        self.openSatelliteW.setObjectName("openSatelliteW")
        self.status.addWidget(self.openSatelliteW, 4, 4, 3, 1)
        self.line_3 = QtWidgets.QFrame(MainWindow)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_3.setMidLineWidth(1)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setObjectName("line_3")
        self.status.addWidget(self.line_3, 1, 0, 1, 5)
        self.openImageW = QtWidgets.QPushButton(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.openImageW.sizePolicy().hasHeightForWidth())
        self.openImageW.setSizePolicy(sizePolicy)
        self.openImageW.setMinimumSize(QtCore.QSize(0, 25))
        self.openImageW.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.openImageW.setFont(font)
        self.openImageW.setStyleSheet("")
        self.openImageW.setObjectName("openImageW")
        self.status.addWidget(self.openImageW, 4, 3, 3, 1)
        self.statusText = QtWidgets.QLineEdit(MainWindow)
        self.statusText.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.statusText.sizePolicy().hasHeightForWidth())
        self.statusText.setSizePolicy(sizePolicy)
        self.statusText.setMinimumSize(QtCore.QSize(0, 25))
        self.statusText.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.statusText.setFont(font)
        self.statusText.setMouseTracking(False)
        self.statusText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.statusText.setAcceptDrops(False)
        self.statusText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.statusText.setReadOnly(True)
        self.statusText.setObjectName("statusText")
        self.status.addWidget(self.statusText, 2, 0, 1, 2)
        self.openMeasureW = QtWidgets.QPushButton(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.openMeasureW.sizePolicy().hasHeightForWidth())
        self.openMeasureW.setSizePolicy(sizePolicy)
        self.openMeasureW.setMinimumSize(QtCore.QSize(0, 25))
        self.openMeasureW.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.openMeasureW.setFont(font)
        self.openMeasureW.setStyleSheet("")
        self.openMeasureW.setObjectName("openMeasureW")
        self.status.addWidget(self.openMeasureW, 4, 1, 3, 1)
        self.openMessageW = QtWidgets.QPushButton(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.openMessageW.sizePolicy().hasHeightForWidth())
        self.openMessageW.setSizePolicy(sizePolicy)
        self.openMessageW.setMinimumSize(QtCore.QSize(0, 25))
        self.openMessageW.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.openMessageW.setFont(font)
        self.openMessageW.setStyleSheet("")
        self.openMessageW.setObjectName("openMessageW")
        self.status.addWidget(self.openMessageW, 4, 0, 3, 1)
        self.status.setColumnStretch(0, 1)
        self.horizontalLayout.addLayout(self.status)
        self.line_2 = QtWidgets.QFrame(MainWindow)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_2.setMidLineWidth(1)
        self.line_2.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_2.setObjectName("line_2")
        self.horizontalLayout.addWidget(self.line_2)
        self.time = QtWidgets.QGridLayout()
        self.time.setSpacing(0)
        self.time.setObjectName("time")
        self.timeUTC = QtWidgets.QLineEdit(MainWindow)
        self.timeUTC.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.timeUTC.sizePolicy().hasHeightForWidth())
        self.timeUTC.setSizePolicy(sizePolicy)
        self.timeUTC.setMinimumSize(QtCore.QSize(100, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.timeUTC.setFont(font)
        self.timeUTC.setMouseTracking(False)
        self.timeUTC.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeUTC.setAcceptDrops(False)
        self.timeUTC.setAlignment(QtCore.Qt.AlignCenter)
        self.timeUTC.setReadOnly(True)
        self.timeUTC.setObjectName("timeUTC")
        self.time.addWidget(self.timeUTC, 5, 0, 1, 2)
        self.timeComputer = QtWidgets.QLineEdit(MainWindow)
        self.timeComputer.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.timeComputer.sizePolicy().hasHeightForWidth())
        self.timeComputer.setSizePolicy(sizePolicy)
        self.timeComputer.setMinimumSize(QtCore.QSize(150, 40))
        self.timeComputer.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(28)
        font.setBold(False)
        font.setWeight(50)
        self.timeComputer.setFont(font)
        self.timeComputer.setMouseTracking(False)
        self.timeComputer.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeComputer.setAcceptDrops(False)
        self.timeComputer.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.timeComputer.setStyleSheet("")
        self.timeComputer.setAlignment(QtCore.Qt.AlignCenter)
        self.timeComputer.setReadOnly(True)
        self.timeComputer.setProperty("large", True)
        self.timeComputer.setObjectName("timeComputer")
        self.time.addWidget(self.timeComputer, 1, 0, 3, 3)
        self.line_6 = QtWidgets.QFrame(MainWindow)
        self.line_6.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_6.setMidLineWidth(1)
        self.line_6.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_6.setObjectName("line_6")
        self.time.addWidget(self.line_6, 6, 0, 1, 2)
        self.profile = QtWidgets.QLineEdit(MainWindow)
        self.profile.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.profile.sizePolicy().hasHeightForWidth())
        self.profile.setSizePolicy(sizePolicy)
        self.profile.setMinimumSize(QtCore.QSize(100, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.profile.setFont(font)
        self.profile.setMouseTracking(False)
        self.profile.setFocusPolicy(QtCore.Qt.NoFocus)
        self.profile.setAcceptDrops(False)
        self.profile.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.profile.setReadOnly(True)
        self.profile.setObjectName("profile")
        self.time.addWidget(self.profile, 7, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(MainWindow)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.time.addWidget(self.label_3, 7, 0, 1, 1)
        self.line_10 = QtWidgets.QFrame(MainWindow)
        self.line_10.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_10.setMidLineWidth(1)
        self.line_10.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_10.setObjectName("line_10")
        self.time.addWidget(self.line_10, 4, 0, 1, 2)
        self.line_8 = QtWidgets.QFrame(MainWindow)
        self.line_8.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_8.setMidLineWidth(1)
        self.line_8.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_8.setObjectName("line_8")
        self.time.addWidget(self.line_8, 8, 0, 1, 2)
        self.horizontalLayout.addLayout(self.time)
        self.line_5 = QtWidgets.QFrame(MainWindow)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_5.setMidLineWidth(1)
        self.line_5.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_5.setObjectName("line_5")
        self.horizontalLayout.addWidget(self.line_5)
        self.loadsave = QtWidgets.QGridLayout()
        self.loadsave.setSpacing(4)
        self.loadsave.setObjectName("loadsave")
        self.saveConfigAs = QtWidgets.QPushButton(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.saveConfigAs.sizePolicy().hasHeightForWidth())
        self.saveConfigAs.setSizePolicy(sizePolicy)
        self.saveConfigAs.setMinimumSize(QtCore.QSize(90, 25))
        self.saveConfigAs.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.saveConfigAs.setFont(font)
        self.saveConfigAs.setStyleSheet("")
        self.saveConfigAs.setObjectName("saveConfigAs")
        self.loadsave.addWidget(self.saveConfigAs, 5, 2, 3, 1)
        self.mountOn = QtWidgets.QPushButton(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mountOn.sizePolicy().hasHeightForWidth())
        self.mountOn.setSizePolicy(sizePolicy)
        self.mountOn.setMinimumSize(QtCore.QSize(90, 25))
        self.mountOn.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.mountOn.setFont(font)
        self.mountOn.setObjectName("mountOn")
        self.loadsave.addWidget(self.mountOn, 8, 1, 1, 1)
        self.loadFrom = QtWidgets.QPushButton(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.loadFrom.sizePolicy().hasHeightForWidth())
        self.loadFrom.setSizePolicy(sizePolicy)
        self.loadFrom.setMinimumSize(QtCore.QSize(90, 25))
        self.loadFrom.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.loadFrom.setFont(font)
        self.loadFrom.setStyleSheet("")
        self.loadFrom.setObjectName("loadFrom")
        self.loadsave.addWidget(self.loadFrom, 4, 1, 1, 1)
        self.saveConfig = QtWidgets.QPushButton(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.saveConfig.sizePolicy().hasHeightForWidth())
        self.saveConfig.setSizePolicy(sizePolicy)
        self.saveConfig.setMinimumSize(QtCore.QSize(90, 25))
        self.saveConfig.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.saveConfig.setFont(font)
        self.saveConfig.setStyleSheet("")
        self.saveConfig.setObjectName("saveConfig")
        self.loadsave.addWidget(self.saveConfig, 5, 1, 3, 1)
        self.saveConfigQuit = QtWidgets.QPushButton(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.saveConfigQuit.sizePolicy().hasHeightForWidth())
        self.saveConfigQuit.setSizePolicy(sizePolicy)
        self.saveConfigQuit.setMinimumSize(QtCore.QSize(90, 25))
        self.saveConfigQuit.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.saveConfigQuit.setFont(font)
        self.saveConfigQuit.setStyleSheet("")
        self.saveConfigQuit.setObjectName("saveConfigQuit")
        self.loadsave.addWidget(self.saveConfigQuit, 4, 2, 1, 1)
        self.mountOff = QtWidgets.QPushButton(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mountOff.sizePolicy().hasHeightForWidth())
        self.mountOff.setSizePolicy(sizePolicy)
        self.mountOff.setMinimumSize(QtCore.QSize(90, 25))
        self.mountOff.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.mountOff.setFont(font)
        self.mountOff.setObjectName("mountOff")
        self.loadsave.addWidget(self.mountOff, 8, 2, 1, 1)
        self.line_7 = QtWidgets.QFrame(MainWindow)
        self.line_7.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_7.setMidLineWidth(1)
        self.line_7.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_7.setObjectName("line_7")
        self.loadsave.addWidget(self.line_7, 3, 1, 1, 2)
        self.statusOnline = QtWidgets.QLabel(MainWindow)
        self.statusOnline.setMaximumSize(QtCore.QSize(16777215, 10))
        self.statusOnline.setObjectName("statusOnline")
        self.loadsave.addWidget(self.statusOnline, 0, 1, 1, 2)
        self.horizontalLayout.addLayout(self.loadsave)
        self.horizontalLayout.setStretch(2, 1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.line = QtWidgets.QFrame(MainWindow)
        self.line.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line.setLineWidth(2)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        spacerItem = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.mainTabWidget = QtWidgets.QTabWidget(MainWindow)
        self.mainTabWidget.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mainTabWidget.sizePolicy().hasHeightForWidth())
        self.mainTabWidget.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mainTabWidget.setFont(font)
        self.mainTabWidget.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.mainTabWidget.setTabPosition(QtWidgets.QTabWidget.North)
        self.mainTabWidget.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.mainTabWidget.setElideMode(QtCore.Qt.ElideRight)
        self.mainTabWidget.setTabsClosable(False)
        self.mainTabWidget.setMovable(False)
        self.mainTabWidget.setTabBarAutoHide(False)
        self.mainTabWidget.setObjectName("mainTabWidget")
        self.Mount = QtWidgets.QWidget()
        self.Mount.setObjectName("Mount")
        self.RA = QtWidgets.QLineEdit(self.Mount)
        self.RA.setEnabled(False)
        self.RA.setGeometry(QtCore.QRect(75, 100, 141, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.RA.setFont(font)
        self.RA.setMouseTracking(False)
        self.RA.setFocusPolicy(QtCore.Qt.NoFocus)
        self.RA.setAcceptDrops(False)
        self.RA.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.RA.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.RA.setReadOnly(True)
        self.RA.setProperty("large", True)
        self.RA.setObjectName("RA")
        self.AZ = QtWidgets.QLineEdit(self.Mount)
        self.AZ.setEnabled(False)
        self.AZ.setGeometry(QtCore.QRect(330, 160, 146, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.AZ.setFont(font)
        self.AZ.setMouseTracking(False)
        self.AZ.setFocusPolicy(QtCore.Qt.NoFocus)
        self.AZ.setAcceptDrops(False)
        self.AZ.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.AZ.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.AZ.setReadOnly(True)
        self.AZ.setProperty("large", True)
        self.AZ.setObjectName("AZ")
        self.DEC = QtWidgets.QLineEdit(self.Mount)
        self.DEC.setEnabled(False)
        self.DEC.setGeometry(QtCore.QRect(330, 100, 146, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.DEC.setFont(font)
        self.DEC.setMouseTracking(False)
        self.DEC.setFocusPolicy(QtCore.Qt.NoFocus)
        self.DEC.setAcceptDrops(False)
        self.DEC.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.DEC.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.DEC.setReadOnly(True)
        self.DEC.setProperty("large", True)
        self.DEC.setObjectName("DEC")
        self.ALT = QtWidgets.QLineEdit(self.Mount)
        self.ALT.setEnabled(False)
        self.ALT.setGeometry(QtCore.QRect(75, 160, 141, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.ALT.setFont(font)
        self.ALT.setMouseTracking(False)
        self.ALT.setFocusPolicy(QtCore.Qt.NoFocus)
        self.ALT.setAcceptDrops(False)
        self.ALT.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.ALT.setStyleSheet("")
        self.ALT.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.ALT.setReadOnly(True)
        self.ALT.setProperty("large", True)
        self.ALT.setObjectName("ALT")
        self.pierside = QtWidgets.QLineEdit(self.Mount)
        self.pierside.setEnabled(False)
        self.pierside.setGeometry(QtCore.QRect(135, 315, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.pierside.setFont(font)
        self.pierside.setMouseTracking(False)
        self.pierside.setFocusPolicy(QtCore.Qt.NoFocus)
        self.pierside.setAcceptDrops(False)
        self.pierside.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.pierside.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.pierside.setReadOnly(True)
        self.pierside.setObjectName("pierside")
        self.label_18 = QtWidgets.QLabel(self.Mount)
        self.label_18.setGeometry(QtCore.QRect(10, 315, 71, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_18.setFont(font)
        self.label_18.setObjectName("label_18")
        self.label_52 = QtWidgets.QLabel(self.Mount)
        self.label_52.setGeometry(QtCore.QRect(285, 255, 91, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_52.setFont(font)
        self.label_52.setWordWrap(False)
        self.label_52.setObjectName("label_52")
        self.timeToFlip = QtWidgets.QLineEdit(self.Mount)
        self.timeToFlip.setEnabled(False)
        self.timeToFlip.setGeometry(QtCore.QRect(405, 255, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.timeToFlip.setFont(font)
        self.timeToFlip.setMouseTracking(False)
        self.timeToFlip.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeToFlip.setAcceptDrops(False)
        self.timeToFlip.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.timeToFlip.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.timeToFlip.setReadOnly(True)
        self.timeToFlip.setObjectName("timeToFlip")
        self.label_61 = QtWidgets.QLabel(self.Mount)
        self.label_61.setGeometry(QtCore.QRect(455, 225, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_61.setFont(font)
        self.label_61.setAlignment(QtCore.Qt.AlignCenter)
        self.label_61.setWordWrap(False)
        self.label_61.setObjectName("label_61")
        self.label_80 = QtWidgets.QLabel(self.Mount)
        self.label_80.setGeometry(QtCore.QRect(10, 200, 231, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_80.setFont(font)
        self.label_80.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_80.setObjectName("label_80")
        self.line_12 = QtWidgets.QFrame(self.Mount)
        self.line_12.setGeometry(QtCore.QRect(10, 220, 766, 1))
        self.line_12.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_12.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_12.setObjectName("line_12")
        self.line_13 = QtWidgets.QFrame(self.Mount)
        self.line_13.setGeometry(QtCore.QRect(10, 40, 251, 1))
        self.line_13.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_13.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_13.setObjectName("line_13")
        self.tracking = QtWidgets.QPushButton(self.Mount)
        self.tracking.setGeometry(QtCore.QRect(505, 50, 136, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.tracking.setFont(font)
        self.tracking.setObjectName("tracking")
        self.park = QtWidgets.QPushButton(self.Mount)
        self.park.setGeometry(QtCore.QRect(505, 160, 136, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.park.setFont(font)
        self.park.setObjectName("park")
        self.line_16 = QtWidgets.QFrame(self.Mount)
        self.line_16.setGeometry(QtCore.QRect(505, 150, 271, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_16.setFont(font)
        self.line_16.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_16.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_16.setObjectName("line_16")
        self.label_84 = QtWidgets.QLabel(self.Mount)
        self.label_84.setGeometry(QtCore.QRect(505, 20, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_84.setFont(font)
        self.label_84.setObjectName("label_84")
        self.line_15 = QtWidgets.QFrame(self.Mount)
        self.line_15.setGeometry(QtCore.QRect(505, 40, 271, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_15.setFont(font)
        self.line_15.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_15.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_15.setObjectName("line_15")
        self.label_85 = QtWidgets.QLabel(self.Mount)
        self.label_85.setGeometry(QtCore.QRect(505, 130, 61, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_85.setFont(font)
        self.label_85.setObjectName("label_85")
        self.stop = QtWidgets.QPushButton(self.Mount)
        self.stop.setGeometry(QtCore.QRect(645, 160, 131, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.stop.setFont(font)
        self.stop.setObjectName("stop")
        self.flipMount = QtWidgets.QPushButton(self.Mount)
        self.flipMount.setGeometry(QtCore.QRect(650, 50, 126, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.flipMount.setFont(font)
        self.flipMount.setObjectName("flipMount")
        self.setLunarTracking = QtWidgets.QPushButton(self.Mount)
        self.setLunarTracking.setGeometry(QtCore.QRect(505, 85, 91, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.setLunarTracking.setFont(font)
        self.setLunarTracking.setObjectName("setLunarTracking")
        self.setSolarTracking = QtWidgets.QPushButton(self.Mount)
        self.setSolarTracking.setGeometry(QtCore.QRect(600, 85, 91, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.setSolarTracking.setFont(font)
        self.setSolarTracking.setObjectName("setSolarTracking")
        self.setSiderealTracking = QtWidgets.QPushButton(self.Mount)
        self.setSiderealTracking.setGeometry(QtCore.QRect(695, 85, 81, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.setSiderealTracking.setFont(font)
        self.setSiderealTracking.setObjectName("setSiderealTracking")
        self.label_90 = QtWidgets.QLabel(self.Mount)
        self.label_90.setGeometry(QtCore.QRect(285, 225, 101, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_90.setFont(font)
        self.label_90.setWordWrap(False)
        self.label_90.setObjectName("label_90")
        self.timeToMeridian = QtWidgets.QLineEdit(self.Mount)
        self.timeToMeridian.setEnabled(False)
        self.timeToMeridian.setGeometry(QtCore.QRect(405, 225, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.timeToMeridian.setFont(font)
        self.timeToMeridian.setMouseTracking(False)
        self.timeToMeridian.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeToMeridian.setAcceptDrops(False)
        self.timeToMeridian.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.timeToMeridian.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.timeToMeridian.setReadOnly(True)
        self.timeToMeridian.setObjectName("timeToMeridian")
        self.label_91 = QtWidgets.QLabel(self.Mount)
        self.label_91.setGeometry(QtCore.QRect(455, 255, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_91.setFont(font)
        self.label_91.setAlignment(QtCore.Qt.AlignCenter)
        self.label_91.setWordWrap(False)
        self.label_91.setObjectName("label_91")
        self.line_51 = QtWidgets.QFrame(self.Mount)
        self.line_51.setGeometry(QtCore.QRect(485, 20, 10, 171))
        self.line_51.setStyleSheet("")
        self.line_51.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_51.setLineWidth(2)
        self.line_51.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_51.setObjectName("line_51")
        self.label_98 = QtWidgets.QLabel(self.Mount)
        self.label_98.setGeometry(QtCore.QRect(10, 20, 211, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_98.setFont(font)
        self.label_98.setObjectName("label_98")
        self.groupBoxCoordinateSystems = QtWidgets.QGroupBox(self.Mount)
        self.groupBoxCoordinateSystems.setGeometry(QtCore.QRect(260, 15, 216, 41))
        self.groupBoxCoordinateSystems.setObjectName("groupBoxCoordinateSystems")
        self.checkJNow = QtWidgets.QRadioButton(self.groupBoxCoordinateSystems)
        self.checkJNow.setGeometry(QtCore.QRect(105, 15, 66, 20))
        self.checkJNow.setSizeIncrement(QtCore.QSize(0, 18))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkJNow.setFont(font)
        self.checkJNow.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkJNow.setChecked(False)
        self.checkJNow.setObjectName("checkJNow")
        self.checkJ2000 = QtWidgets.QRadioButton(self.groupBoxCoordinateSystems)
        self.checkJ2000.setGeometry(QtCore.QRect(15, 15, 66, 20))
        self.checkJ2000.setSizeIncrement(QtCore.QSize(0, 18))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkJ2000.setFont(font)
        self.checkJ2000.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkJ2000.setChecked(True)
        self.checkJ2000.setObjectName("checkJ2000")
        self.UTCExpire = QtWidgets.QLineEdit(self.Mount)
        self.UTCExpire.setEnabled(False)
        self.UTCExpire.setGeometry(QtCore.QRect(105, 285, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.UTCExpire.setFont(font)
        self.UTCExpire.setMouseTracking(False)
        self.UTCExpire.setFocusPolicy(QtCore.Qt.NoFocus)
        self.UTCExpire.setAcceptDrops(False)
        self.UTCExpire.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.UTCExpire.setInputMask("")
        self.UTCExpire.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.UTCExpire.setReadOnly(True)
        self.UTCExpire.setObjectName("UTCExpire")
        self.label_152 = QtWidgets.QLabel(self.Mount)
        self.label_152.setGeometry(QtCore.QRect(10, 285, 96, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_152.setFont(font)
        self.label_152.setObjectName("label_152")
        self.timeSidereal = QtWidgets.QLineEdit(self.Mount)
        self.timeSidereal.setEnabled(False)
        self.timeSidereal.setGeometry(QtCore.QRect(105, 225, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.timeSidereal.setFont(font)
        self.timeSidereal.setMouseTracking(False)
        self.timeSidereal.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeSidereal.setAcceptDrops(False)
        self.timeSidereal.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.timeSidereal.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.timeSidereal.setReadOnly(True)
        self.timeSidereal.setObjectName("timeSidereal")
        self.label_7 = QtWidgets.QLabel(self.Mount)
        self.label_7.setGeometry(QtCore.QRect(10, 225, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_7.setFont(font)
        self.label_7.setObjectName("label_7")
        self.timeJD = QtWidgets.QLineEdit(self.Mount)
        self.timeJD.setEnabled(False)
        self.timeJD.setGeometry(QtCore.QRect(105, 255, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.timeJD.setFont(font)
        self.timeJD.setMouseTracking(False)
        self.timeJD.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeJD.setAcceptDrops(False)
        self.timeJD.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.timeJD.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.timeJD.setReadOnly(True)
        self.timeJD.setObjectName("timeJD")
        self.label_63 = QtWidgets.QLabel(self.Mount)
        self.label_63.setGeometry(QtCore.QRect(10, 255, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_63.setFont(font)
        self.label_63.setObjectName("label_63")
        self.label_64 = QtWidgets.QLabel(self.Mount)
        self.label_64.setGeometry(QtCore.QRect(190, 345, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_64.setFont(font)
        self.label_64.setObjectName("label_64")
        self.label_74 = QtWidgets.QLabel(self.Mount)
        self.label_74.setGeometry(QtCore.QRect(285, 375, 116, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_74.setFont(font)
        self.label_74.setObjectName("label_74")
        self.statusUnattendedFlip = QtWidgets.QLineEdit(self.Mount)
        self.statusUnattendedFlip.setEnabled(False)
        self.statusUnattendedFlip.setGeometry(QtCore.QRect(405, 345, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.statusUnattendedFlip.setFont(font)
        self.statusUnattendedFlip.setMouseTracking(False)
        self.statusUnattendedFlip.setFocusPolicy(QtCore.Qt.NoFocus)
        self.statusUnattendedFlip.setAcceptDrops(False)
        self.statusUnattendedFlip.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.statusUnattendedFlip.setInputMask("")
        self.statusUnattendedFlip.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.statusUnattendedFlip.setReadOnly(True)
        self.statusUnattendedFlip.setProperty("input", True)
        self.statusUnattendedFlip.setObjectName("statusUnattendedFlip")
        self.label_55 = QtWidgets.QLabel(self.Mount)
        self.label_55.setGeometry(QtCore.QRect(285, 345, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_55.setFont(font)
        self.label_55.setObjectName("label_55")
        self.statusDualTracking = QtWidgets.QLineEdit(self.Mount)
        self.statusDualTracking.setEnabled(False)
        self.statusDualTracking.setGeometry(QtCore.QRect(405, 375, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.statusDualTracking.setFont(font)
        self.statusDualTracking.setMouseTracking(False)
        self.statusDualTracking.setFocusPolicy(QtCore.Qt.NoFocus)
        self.statusDualTracking.setAcceptDrops(False)
        self.statusDualTracking.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.statusDualTracking.setInputMask("")
        self.statusDualTracking.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.statusDualTracking.setReadOnly(True)
        self.statusDualTracking.setProperty("input", True)
        self.statusDualTracking.setObjectName("statusDualTracking")
        self.label_232 = QtWidgets.QLabel(self.Mount)
        self.label_232.setGeometry(QtCore.QRect(10, 345, 91, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_232.setFont(font)
        self.label_232.setObjectName("label_232")
        self.label_242 = QtWidgets.QLabel(self.Mount)
        self.label_242.setGeometry(QtCore.QRect(10, 375, 91, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_242.setFont(font)
        self.label_242.setObjectName("label_242")
        self.refractionTemp = QtWidgets.QLineEdit(self.Mount)
        self.refractionTemp.setEnabled(False)
        self.refractionTemp.setGeometry(QtCore.QRect(135, 345, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.refractionTemp.setFont(font)
        self.refractionTemp.setMouseTracking(False)
        self.refractionTemp.setFocusPolicy(QtCore.Qt.NoFocus)
        self.refractionTemp.setAcceptDrops(False)
        self.refractionTemp.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.refractionTemp.setMaxLength(8)
        self.refractionTemp.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.refractionTemp.setReadOnly(True)
        self.refractionTemp.setObjectName("refractionTemp")
        self.refractionPress = QtWidgets.QLineEdit(self.Mount)
        self.refractionPress.setEnabled(False)
        self.refractionPress.setGeometry(QtCore.QRect(135, 375, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.refractionPress.setFont(font)
        self.refractionPress.setMouseTracking(False)
        self.refractionPress.setFocusPolicy(QtCore.Qt.NoFocus)
        self.refractionPress.setAcceptDrops(False)
        self.refractionPress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.refractionPress.setMaxLength(8)
        self.refractionPress.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.refractionPress.setReadOnly(True)
        self.refractionPress.setObjectName("refractionPress")
        self.label_81 = QtWidgets.QLabel(self.Mount)
        self.label_81.setGeometry(QtCore.QRect(190, 375, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_81.setFont(font)
        self.label_81.setObjectName("label_81")
        self.label_130 = QtWidgets.QLabel(self.Mount)
        self.label_130.setGeometry(QtCore.QRect(190, 225, 41, 21))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.label_130.setFont(font)
        self.label_130.setObjectName("label_130")
        self.label_222 = QtWidgets.QLabel(self.Mount)
        self.label_222.setGeometry(QtCore.QRect(190, 285, 36, 21))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.label_222.setFont(font)
        self.label_222.setObjectName("label_222")
        self.statusRefraction = QtWidgets.QLineEdit(self.Mount)
        self.statusRefraction.setEnabled(False)
        self.statusRefraction.setGeometry(QtCore.QRect(135, 405, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.statusRefraction.setFont(font)
        self.statusRefraction.setMouseTracking(False)
        self.statusRefraction.setFocusPolicy(QtCore.Qt.NoFocus)
        self.statusRefraction.setAcceptDrops(False)
        self.statusRefraction.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.statusRefraction.setInputMask("")
        self.statusRefraction.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.statusRefraction.setReadOnly(True)
        self.statusRefraction.setProperty("input", True)
        self.statusRefraction.setObjectName("statusRefraction")
        self.label_86 = QtWidgets.QLabel(self.Mount)
        self.label_86.setGeometry(QtCore.QRect(10, 405, 126, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_86.setFont(font)
        self.label_86.setObjectName("label_86")
        self.label_139 = QtWidgets.QLabel(self.Mount)
        self.label_139.setGeometry(QtCore.QRect(190, 255, 36, 21))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.label_139.setFont(font)
        self.label_139.setObjectName("label_139")
        self.label_148 = QtWidgets.QLabel(self.Mount)
        self.label_148.setGeometry(QtCore.QRect(545, 315, 126, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_148.setFont(font)
        self.label_148.setObjectName("label_148")
        self.statusGPSSynced = QtWidgets.QLineEdit(self.Mount)
        self.statusGPSSynced.setEnabled(False)
        self.statusGPSSynced.setGeometry(QtCore.QRect(690, 315, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.statusGPSSynced.setFont(font)
        self.statusGPSSynced.setMouseTracking(False)
        self.statusGPSSynced.setFocusPolicy(QtCore.Qt.NoFocus)
        self.statusGPSSynced.setAcceptDrops(False)
        self.statusGPSSynced.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.statusGPSSynced.setInputMask("")
        self.statusGPSSynced.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.statusGPSSynced.setReadOnly(True)
        self.statusGPSSynced.setObjectName("statusGPSSynced")
        self.meridianLimitSlew = QtWidgets.QLineEdit(self.Mount)
        self.meridianLimitSlew.setEnabled(False)
        self.meridianLimitSlew.setGeometry(QtCore.QRect(405, 315, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.meridianLimitSlew.setFont(font)
        self.meridianLimitSlew.setMouseTracking(False)
        self.meridianLimitSlew.setFocusPolicy(QtCore.Qt.NoFocus)
        self.meridianLimitSlew.setAcceptDrops(False)
        self.meridianLimitSlew.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.meridianLimitSlew.setInputMask("")
        self.meridianLimitSlew.setCursorPosition(1)
        self.meridianLimitSlew.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.meridianLimitSlew.setReadOnly(True)
        self.meridianLimitSlew.setProperty("input", True)
        self.meridianLimitSlew.setObjectName("meridianLimitSlew")
        self.label_28 = QtWidgets.QLabel(self.Mount)
        self.label_28.setGeometry(QtCore.QRect(285, 405, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_28.setFont(font)
        self.label_28.setObjectName("label_28")
        self.label_20 = QtWidgets.QLabel(self.Mount)
        self.label_20.setGeometry(QtCore.QRect(545, 225, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_20.setFont(font)
        self.label_20.setObjectName("label_20")
        self.label_4 = QtWidgets.QLabel(self.Mount)
        self.label_4.setGeometry(QtCore.QRect(545, 255, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.label_133 = QtWidgets.QLabel(self.Mount)
        self.label_133.setGeometry(QtCore.QRect(460, 405, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_133.setFont(font)
        self.label_133.setObjectName("label_133")
        self.meridianLimitTrack = QtWidgets.QLineEdit(self.Mount)
        self.meridianLimitTrack.setEnabled(False)
        self.meridianLimitTrack.setGeometry(QtCore.QRect(405, 285, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.meridianLimitTrack.setFont(font)
        self.meridianLimitTrack.setMouseTracking(False)
        self.meridianLimitTrack.setFocusPolicy(QtCore.Qt.NoFocus)
        self.meridianLimitTrack.setAcceptDrops(False)
        self.meridianLimitTrack.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.meridianLimitTrack.setInputMask("")
        self.meridianLimitTrack.setCursorPosition(1)
        self.meridianLimitTrack.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.meridianLimitTrack.setReadOnly(True)
        self.meridianLimitTrack.setProperty("input", True)
        self.meridianLimitTrack.setObjectName("meridianLimitTrack")
        self.label_109 = QtWidgets.QLabel(self.Mount)
        self.label_109.setGeometry(QtCore.QRect(740, 255, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_109.setFont(font)
        self.label_109.setAlignment(QtCore.Qt.AlignCenter)
        self.label_109.setWordWrap(False)
        self.label_109.setObjectName("label_109")
        self.label_65 = QtWidgets.QLabel(self.Mount)
        self.label_65.setGeometry(QtCore.QRect(285, 315, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_65.setFont(font)
        self.label_65.setObjectName("label_65")
        self.label_89 = QtWidgets.QLabel(self.Mount)
        self.label_89.setGeometry(QtCore.QRect(455, 315, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_89.setFont(font)
        self.label_89.setAlignment(QtCore.Qt.AlignCenter)
        self.label_89.setWordWrap(False)
        self.label_89.setObjectName("label_89")
        self.horizonLimitLow = QtWidgets.QLineEdit(self.Mount)
        self.horizonLimitLow.setEnabled(False)
        self.horizonLimitLow.setGeometry(QtCore.QRect(690, 255, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.horizonLimitLow.setFont(font)
        self.horizonLimitLow.setMouseTracking(False)
        self.horizonLimitLow.setFocusPolicy(QtCore.Qt.NoFocus)
        self.horizonLimitLow.setAcceptDrops(False)
        self.horizonLimitLow.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.horizonLimitLow.setInputMask("")
        self.horizonLimitLow.setMaxLength(32767)
        self.horizonLimitLow.setFrame(True)
        self.horizonLimitLow.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.horizonLimitLow.setReadOnly(True)
        self.horizonLimitLow.setClearButtonEnabled(False)
        self.horizonLimitLow.setProperty("input", True)
        self.horizonLimitLow.setObjectName("horizonLimitLow")
        self.horizonLimitHigh = QtWidgets.QLineEdit(self.Mount)
        self.horizonLimitHigh.setEnabled(False)
        self.horizonLimitHigh.setGeometry(QtCore.QRect(690, 225, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.horizonLimitHigh.setFont(font)
        self.horizonLimitHigh.setMouseTracking(False)
        self.horizonLimitHigh.setFocusPolicy(QtCore.Qt.NoFocus)
        self.horizonLimitHigh.setAcceptDrops(False)
        self.horizonLimitHigh.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.horizonLimitHigh.setInputMask("")
        self.horizonLimitHigh.setCursorPosition(1)
        self.horizonLimitHigh.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.horizonLimitHigh.setReadOnly(True)
        self.horizonLimitHigh.setProperty("input", True)
        self.horizonLimitHigh.setObjectName("horizonLimitHigh")
        self.label_88 = QtWidgets.QLabel(self.Mount)
        self.label_88.setGeometry(QtCore.QRect(455, 285, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_88.setFont(font)
        self.label_88.setAlignment(QtCore.Qt.AlignCenter)
        self.label_88.setWordWrap(False)
        self.label_88.setObjectName("label_88")
        self.label_104 = QtWidgets.QLabel(self.Mount)
        self.label_104.setGeometry(QtCore.QRect(740, 225, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_104.setFont(font)
        self.label_104.setAlignment(QtCore.Qt.AlignCenter)
        self.label_104.setWordWrap(False)
        self.label_104.setObjectName("label_104")
        self.label_34 = QtWidgets.QLabel(self.Mount)
        self.label_34.setGeometry(QtCore.QRect(285, 285, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_34.setFont(font)
        self.label_34.setObjectName("label_34")
        self.slewRate = QtWidgets.QLineEdit(self.Mount)
        self.slewRate.setEnabled(False)
        self.slewRate.setGeometry(QtCore.QRect(405, 405, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.slewRate.setFont(font)
        self.slewRate.setMouseTracking(False)
        self.slewRate.setFocusPolicy(QtCore.Qt.NoFocus)
        self.slewRate.setAcceptDrops(False)
        self.slewRate.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.slewRate.setInputMask("")
        self.slewRate.setCursorPosition(1)
        self.slewRate.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.slewRate.setReadOnly(True)
        self.slewRate.setProperty("input", True)
        self.slewRate.setObjectName("slewRate")
        self.label_6 = QtWidgets.QLabel(self.Mount)
        self.label_6.setGeometry(QtCore.QRect(545, 345, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.label_5 = QtWidgets.QLabel(self.Mount)
        self.label_5.setGeometry(QtCore.QRect(545, 375, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.label_12 = QtWidgets.QLabel(self.Mount)
        self.label_12.setGeometry(QtCore.QRect(545, 405, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_12.setFont(font)
        self.label_12.setObjectName("label_12")
        self.siteLongitude = QtWidgets.QLineEdit(self.Mount)
        self.siteLongitude.setEnabled(False)
        self.siteLongitude.setGeometry(QtCore.QRect(645, 375, 96, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.siteLongitude.setFont(font)
        self.siteLongitude.setMouseTracking(False)
        self.siteLongitude.setFocusPolicy(QtCore.Qt.NoFocus)
        self.siteLongitude.setAcceptDrops(False)
        self.siteLongitude.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.siteLongitude.setMaxLength(32634)
        self.siteLongitude.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.siteLongitude.setReadOnly(True)
        self.siteLongitude.setCursorMoveStyle(QtCore.Qt.VisualMoveStyle)
        self.siteLongitude.setProperty("input", True)
        self.siteLongitude.setObjectName("siteLongitude")
        self.siteLatitude = QtWidgets.QLineEdit(self.Mount)
        self.siteLatitude.setEnabled(False)
        self.siteLatitude.setGeometry(QtCore.QRect(645, 345, 96, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.siteLatitude.setFont(font)
        self.siteLatitude.setMouseTracking(False)
        self.siteLatitude.setFocusPolicy(QtCore.Qt.NoFocus)
        self.siteLatitude.setAcceptDrops(False)
        self.siteLatitude.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.siteLatitude.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.siteLatitude.setReadOnly(True)
        self.siteLatitude.setProperty("input", True)
        self.siteLatitude.setObjectName("siteLatitude")
        self.label_105 = QtWidgets.QLabel(self.Mount)
        self.label_105.setGeometry(QtCore.QRect(745, 375, 31, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_105.setFont(font)
        self.label_105.setAlignment(QtCore.Qt.AlignCenter)
        self.label_105.setWordWrap(False)
        self.label_105.setObjectName("label_105")
        self.label_82 = QtWidgets.QLabel(self.Mount)
        self.label_82.setGeometry(QtCore.QRect(740, 405, 21, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_82.setFont(font)
        self.label_82.setAlignment(QtCore.Qt.AlignCenter)
        self.label_82.setWordWrap(False)
        self.label_82.setObjectName("label_82")
        self.label_106 = QtWidgets.QLabel(self.Mount)
        self.label_106.setGeometry(QtCore.QRect(745, 345, 31, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_106.setFont(font)
        self.label_106.setAlignment(QtCore.Qt.AlignCenter)
        self.label_106.setWordWrap(False)
        self.label_106.setObjectName("label_106")
        self.siteElevation = QtWidgets.QLineEdit(self.Mount)
        self.siteElevation.setEnabled(False)
        self.siteElevation.setGeometry(QtCore.QRect(645, 405, 96, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.siteElevation.setFont(font)
        self.siteElevation.setMouseTracking(False)
        self.siteElevation.setFocusPolicy(QtCore.Qt.NoFocus)
        self.siteElevation.setAcceptDrops(False)
        self.siteElevation.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.siteElevation.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.siteElevation.setReadOnly(True)
        self.siteElevation.setProperty("input", True)
        self.siteElevation.setObjectName("siteElevation")
        self.label_151 = QtWidgets.QLabel(self.Mount)
        self.label_151.setGeometry(QtCore.QRect(545, 285, 126, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_151.setFont(font)
        self.label_151.setObjectName("label_151")
        self.statusWOL = QtWidgets.QLineEdit(self.Mount)
        self.statusWOL.setEnabled(False)
        self.statusWOL.setGeometry(QtCore.QRect(690, 285, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.statusWOL.setFont(font)
        self.statusWOL.setMouseTracking(False)
        self.statusWOL.setFocusPolicy(QtCore.Qt.NoFocus)
        self.statusWOL.setAcceptDrops(False)
        self.statusWOL.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.statusWOL.setInputMask("")
        self.statusWOL.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.statusWOL.setReadOnly(True)
        self.statusWOL.setObjectName("statusWOL")
        self.HA = QtWidgets.QLineEdit(self.Mount)
        self.HA.setEnabled(False)
        self.HA.setGeometry(QtCore.QRect(75, 50, 141, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.HA.setFont(font)
        self.HA.setMouseTracking(False)
        self.HA.setFocusPolicy(QtCore.Qt.NoFocus)
        self.HA.setAcceptDrops(False)
        self.HA.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.HA.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.HA.setReadOnly(True)
        self.HA.setProperty("large", True)
        self.HA.setObjectName("HA")
        self.label_58 = QtWidgets.QLabel(self.Mount)
        self.label_58.setGeometry(QtCore.QRect(10, 60, 66, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.label_58.setFont(font)
        self.label_58.setProperty("large", True)
        self.label_58.setObjectName("label_58")
        self.label_77 = QtWidgets.QLabel(self.Mount)
        self.label_77.setGeometry(QtCore.QRect(10, 110, 66, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.label_77.setFont(font)
        self.label_77.setProperty("large", True)
        self.label_77.setObjectName("label_77")
        self.label_78 = QtWidgets.QLabel(self.Mount)
        self.label_78.setGeometry(QtCore.QRect(10, 160, 66, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.label_78.setFont(font)
        self.label_78.setProperty("large", True)
        self.label_78.setObjectName("label_78")
        self.label_79 = QtWidgets.QLabel(self.Mount)
        self.label_79.setGeometry(QtCore.QRect(260, 160, 66, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.label_79.setFont(font)
        self.label_79.setProperty("large", True)
        self.label_79.setObjectName("label_79")
        self.label_102 = QtWidgets.QLabel(self.Mount)
        self.label_102.setGeometry(QtCore.QRect(260, 110, 66, 31))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.label_102.setFont(font)
        self.label_102.setProperty("large", True)
        self.label_102.setObjectName("label_102")
        self.HAfloat = QtWidgets.QLineEdit(self.Mount)
        self.HAfloat.setEnabled(False)
        self.HAfloat.setGeometry(QtCore.QRect(75, 80, 141, 16))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.HAfloat.sizePolicy().hasHeightForWidth())
        self.HAfloat.setSizePolicy(sizePolicy)
        self.HAfloat.setMinimumSize(QtCore.QSize(0, 15))
        self.HAfloat.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.HAfloat.setFont(font)
        self.HAfloat.setMouseTracking(False)
        self.HAfloat.setFocusPolicy(QtCore.Qt.NoFocus)
        self.HAfloat.setAcceptDrops(False)
        self.HAfloat.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.HAfloat.setReadOnly(True)
        self.HAfloat.setObjectName("HAfloat")
        self.RAfloat = QtWidgets.QLineEdit(self.Mount)
        self.RAfloat.setEnabled(False)
        self.RAfloat.setGeometry(QtCore.QRect(75, 130, 141, 16))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.RAfloat.sizePolicy().hasHeightForWidth())
        self.RAfloat.setSizePolicy(sizePolicy)
        self.RAfloat.setMinimumSize(QtCore.QSize(0, 15))
        self.RAfloat.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.RAfloat.setFont(font)
        self.RAfloat.setMouseTracking(False)
        self.RAfloat.setFocusPolicy(QtCore.Qt.NoFocus)
        self.RAfloat.setAcceptDrops(False)
        self.RAfloat.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.RAfloat.setReadOnly(True)
        self.RAfloat.setObjectName("RAfloat")
        self.DECfloat = QtWidgets.QLineEdit(self.Mount)
        self.DECfloat.setEnabled(False)
        self.DECfloat.setGeometry(QtCore.QRect(330, 130, 146, 16))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.DECfloat.sizePolicy().hasHeightForWidth())
        self.DECfloat.setSizePolicy(sizePolicy)
        self.DECfloat.setMinimumSize(QtCore.QSize(0, 15))
        self.DECfloat.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.DECfloat.setFont(font)
        self.DECfloat.setMouseTracking(False)
        self.DECfloat.setFocusPolicy(QtCore.Qt.NoFocus)
        self.DECfloat.setAcceptDrops(False)
        self.DECfloat.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.DECfloat.setReadOnly(True)
        self.DECfloat.setObjectName("DECfloat")
        self.line_15.raise_()
        self.line_16.raise_()
        self.line_12.raise_()
        self.line_13.raise_()
        self.line_51.raise_()
        self.RA.raise_()
        self.AZ.raise_()
        self.DEC.raise_()
        self.ALT.raise_()
        self.pierside.raise_()
        self.label_18.raise_()
        self.label_52.raise_()
        self.timeToFlip.raise_()
        self.label_61.raise_()
        self.label_80.raise_()
        self.tracking.raise_()
        self.park.raise_()
        self.label_84.raise_()
        self.label_85.raise_()
        self.stop.raise_()
        self.flipMount.raise_()
        self.setLunarTracking.raise_()
        self.setSolarTracking.raise_()
        self.setSiderealTracking.raise_()
        self.label_90.raise_()
        self.timeToMeridian.raise_()
        self.label_91.raise_()
        self.label_98.raise_()
        self.groupBoxCoordinateSystems.raise_()
        self.UTCExpire.raise_()
        self.label_152.raise_()
        self.timeSidereal.raise_()
        self.label_7.raise_()
        self.timeJD.raise_()
        self.label_63.raise_()
        self.label_64.raise_()
        self.label_74.raise_()
        self.statusUnattendedFlip.raise_()
        self.label_55.raise_()
        self.statusDualTracking.raise_()
        self.label_232.raise_()
        self.label_242.raise_()
        self.refractionTemp.raise_()
        self.refractionPress.raise_()
        self.label_81.raise_()
        self.label_130.raise_()
        self.label_222.raise_()
        self.statusRefraction.raise_()
        self.label_86.raise_()
        self.label_139.raise_()
        self.label_148.raise_()
        self.statusGPSSynced.raise_()
        self.meridianLimitSlew.raise_()
        self.label_28.raise_()
        self.label_20.raise_()
        self.label_4.raise_()
        self.label_133.raise_()
        self.meridianLimitTrack.raise_()
        self.label_109.raise_()
        self.label_65.raise_()
        self.label_89.raise_()
        self.horizonLimitLow.raise_()
        self.horizonLimitHigh.raise_()
        self.label_88.raise_()
        self.label_104.raise_()
        self.label_34.raise_()
        self.slewRate.raise_()
        self.label_6.raise_()
        self.label_5.raise_()
        self.label_12.raise_()
        self.siteLongitude.raise_()
        self.siteLatitude.raise_()
        self.label_105.raise_()
        self.label_82.raise_()
        self.label_106.raise_()
        self.siteElevation.raise_()
        self.label_151.raise_()
        self.statusWOL.raise_()
        self.HA.raise_()
        self.label_58.raise_()
        self.label_77.raise_()
        self.label_78.raise_()
        self.label_79.raise_()
        self.label_102.raise_()
        self.HAfloat.raise_()
        self.RAfloat.raise_()
        self.DECfloat.raise_()
        self.mainTabWidget.addTab(self.Mount, "")
        self.Environ = QtWidgets.QWidget()
        self.Environ.setObjectName("Environ")
        self.label_32 = QtWidgets.QLabel(self.Environ)
        self.label_32.setGeometry(QtCore.QRect(15, 205, 71, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_32.setFont(font)
        self.label_32.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_32.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_32.setObjectName("label_32")
        self.label_25 = QtWidgets.QLabel(self.Environ)
        self.label_25.setGeometry(QtCore.QRect(15, 105, 66, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_25.setFont(font)
        self.label_25.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_25.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_25.setObjectName("label_25")
        self.label_31 = QtWidgets.QLabel(self.Environ)
        self.label_31.setGeometry(QtCore.QRect(15, 180, 66, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_31.setFont(font)
        self.label_31.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_31.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_31.setObjectName("label_31")
        self.label_26 = QtWidgets.QLabel(self.Environ)
        self.label_26.setGeometry(QtCore.QRect(15, 130, 66, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_26.setFont(font)
        self.label_26.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_26.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_26.setObjectName("label_26")
        self.setRefractionManual = QtWidgets.QPushButton(self.Environ)
        self.setRefractionManual.setGeometry(QtCore.QRect(595, 125, 181, 46))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.setRefractionManual.setFont(font)
        self.setRefractionManual.setFlat(False)
        self.setRefractionManual.setObjectName("setRefractionManual")
        self.label_30 = QtWidgets.QLabel(self.Environ)
        self.label_30.setGeometry(QtCore.QRect(15, 155, 71, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_30.setFont(font)
        self.label_30.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_30.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_30.setObjectName("label_30")
        self.label_24 = QtWidgets.QLabel(self.Environ)
        self.label_24.setGeometry(QtCore.QRect(15, 55, 61, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_24.setFont(font)
        self.label_24.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_24.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_24.setObjectName("label_24")
        self.label_27 = QtWidgets.QLabel(self.Environ)
        self.label_27.setGeometry(QtCore.QRect(15, 80, 66, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_27.setFont(font)
        self.label_27.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_27.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_27.setObjectName("label_27")
        self.label_142 = QtWidgets.QLabel(self.Environ)
        self.label_142.setGeometry(QtCore.QRect(15, 255, 76, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_142.setFont(font)
        self.label_142.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_142.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_142.setObjectName("label_142")
        self.refractionGroup = QtWidgets.QGroupBox(self.Environ)
        self.refractionGroup.setGeometry(QtCore.QRect(595, 35, 181, 86))
        self.refractionGroup.setObjectName("refractionGroup")
        self.checkRefracCont = QtWidgets.QRadioButton(self.refractionGroup)
        self.checkRefracCont.setGeometry(QtCore.QRect(15, 40, 136, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkRefracCont.setFont(font)
        self.checkRefracCont.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkRefracCont.setChecked(False)
        self.checkRefracCont.setObjectName("checkRefracCont")
        self.checkRefracNoTrack = QtWidgets.QRadioButton(self.refractionGroup)
        self.checkRefracNoTrack.setGeometry(QtCore.QRect(15, 60, 151, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkRefracNoTrack.setFont(font)
        self.checkRefracNoTrack.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkRefracNoTrack.setChecked(False)
        self.checkRefracNoTrack.setObjectName("checkRefracNoTrack")
        self.checkRefracNone = QtWidgets.QRadioButton(self.refractionGroup)
        self.checkRefracNone.setGeometry(QtCore.QRect(15, 20, 111, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkRefracNone.setFont(font)
        self.checkRefracNone.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkRefracNone.setChecked(True)
        self.checkRefracNone.setObjectName("checkRefracNone")
        self.mountGroup = QtWidgets.QGroupBox(self.Environ)
        self.mountGroup.setGeometry(QtCore.QRect(505, 35, 76, 246))
        self.mountGroup.setObjectName("mountGroup")
        self.refractionTemp1 = QtWidgets.QLineEdit(self.mountGroup)
        self.refractionTemp1.setEnabled(False)
        self.refractionTemp1.setGeometry(QtCore.QRect(10, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.refractionTemp1.setFont(font)
        self.refractionTemp1.setMouseTracking(False)
        self.refractionTemp1.setFocusPolicy(QtCore.Qt.NoFocus)
        self.refractionTemp1.setAcceptDrops(False)
        self.refractionTemp1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.refractionTemp1.setMaxLength(8)
        self.refractionTemp1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.refractionTemp1.setReadOnly(True)
        self.refractionTemp1.setObjectName("refractionTemp1")
        self.refractionPress1 = QtWidgets.QLineEdit(self.mountGroup)
        self.refractionPress1.setEnabled(False)
        self.refractionPress1.setGeometry(QtCore.QRect(10, 45, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.refractionPress1.setFont(font)
        self.refractionPress1.setMouseTracking(False)
        self.refractionPress1.setFocusPolicy(QtCore.Qt.NoFocus)
        self.refractionPress1.setAcceptDrops(False)
        self.refractionPress1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.refractionPress1.setMaxLength(8)
        self.refractionPress1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.refractionPress1.setReadOnly(True)
        self.refractionPress1.setObjectName("refractionPress1")
        self.environGroup = QtWidgets.QGroupBox(self.Environ)
        self.environGroup.setEnabled(True)
        self.environGroup.setGeometry(QtCore.QRect(415, 35, 76, 246))
        self.environGroup.setObjectName("environGroup")
        self.environTemp = QtWidgets.QLineEdit(self.environGroup)
        self.environTemp.setEnabled(False)
        self.environTemp.setGeometry(QtCore.QRect(10, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.environTemp.setFont(font)
        self.environTemp.setMouseTracking(False)
        self.environTemp.setFocusPolicy(QtCore.Qt.NoFocus)
        self.environTemp.setAcceptDrops(False)
        self.environTemp.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.environTemp.setMaxLength(8)
        self.environTemp.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.environTemp.setReadOnly(True)
        self.environTemp.setObjectName("environTemp")
        self.environPress = QtWidgets.QLineEdit(self.environGroup)
        self.environPress.setEnabled(False)
        self.environPress.setGeometry(QtCore.QRect(10, 45, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.environPress.setFont(font)
        self.environPress.setMouseTracking(False)
        self.environPress.setFocusPolicy(QtCore.Qt.NoFocus)
        self.environPress.setAcceptDrops(False)
        self.environPress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.environPress.setMaxLength(8)
        self.environPress.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.environPress.setReadOnly(True)
        self.environPress.setObjectName("environPress")
        self.environDewPoint = QtWidgets.QLineEdit(self.environGroup)
        self.environDewPoint.setEnabled(False)
        self.environDewPoint.setGeometry(QtCore.QRect(10, 70, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.environDewPoint.setFont(font)
        self.environDewPoint.setMouseTracking(False)
        self.environDewPoint.setFocusPolicy(QtCore.Qt.NoFocus)
        self.environDewPoint.setAcceptDrops(False)
        self.environDewPoint.setToolTip("")
        self.environDewPoint.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.environDewPoint.setMaxLength(8)
        self.environDewPoint.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.environDewPoint.setReadOnly(True)
        self.environDewPoint.setObjectName("environDewPoint")
        self.environHumidity = QtWidgets.QLineEdit(self.environGroup)
        self.environHumidity.setEnabled(False)
        self.environHumidity.setGeometry(QtCore.QRect(10, 95, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.environHumidity.setFont(font)
        self.environHumidity.setMouseTracking(False)
        self.environHumidity.setFocusPolicy(QtCore.Qt.NoFocus)
        self.environHumidity.setAcceptDrops(False)
        self.environHumidity.setToolTip("")
        self.environHumidity.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.environHumidity.setMaxLength(8)
        self.environHumidity.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.environHumidity.setReadOnly(True)
        self.environHumidity.setObjectName("environHumidity")
        self.label_33 = QtWidgets.QLabel(self.Environ)
        self.label_33.setGeometry(QtCore.QRect(15, 230, 66, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_33.setFont(font)
        self.label_33.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_33.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_33.setObjectName("label_33")
        self.weatherGroup = QtWidgets.QGroupBox(self.Environ)
        self.weatherGroup.setGeometry(QtCore.QRect(145, 35, 76, 246))
        self.weatherGroup.setObjectName("weatherGroup")
        self.weatherRainVol = QtWidgets.QLineEdit(self.weatherGroup)
        self.weatherRainVol.setEnabled(False)
        self.weatherRainVol.setGeometry(QtCore.QRect(10, 145, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.weatherRainVol.setFont(font)
        self.weatherRainVol.setMouseTracking(False)
        self.weatherRainVol.setFocusPolicy(QtCore.Qt.NoFocus)
        self.weatherRainVol.setAcceptDrops(False)
        self.weatherRainVol.setToolTip("")
        self.weatherRainVol.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.weatherRainVol.setMaxLength(8)
        self.weatherRainVol.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.weatherRainVol.setReadOnly(True)
        self.weatherRainVol.setObjectName("weatherRainVol")
        self.weatherWindSpeed = QtWidgets.QLineEdit(self.weatherGroup)
        self.weatherWindSpeed.setEnabled(False)
        self.weatherWindSpeed.setGeometry(QtCore.QRect(10, 170, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.weatherWindSpeed.setFont(font)
        self.weatherWindSpeed.setMouseTracking(False)
        self.weatherWindSpeed.setFocusPolicy(QtCore.Qt.NoFocus)
        self.weatherWindSpeed.setAcceptDrops(False)
        self.weatherWindSpeed.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.weatherWindSpeed.setMaxLength(8)
        self.weatherWindSpeed.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.weatherWindSpeed.setReadOnly(True)
        self.weatherWindSpeed.setObjectName("weatherWindSpeed")
        self.weatherCloudCover = QtWidgets.QLineEdit(self.weatherGroup)
        self.weatherCloudCover.setEnabled(False)
        self.weatherCloudCover.setGeometry(QtCore.QRect(10, 120, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.weatherCloudCover.setFont(font)
        self.weatherCloudCover.setMouseTracking(False)
        self.weatherCloudCover.setFocusPolicy(QtCore.Qt.NoFocus)
        self.weatherCloudCover.setAcceptDrops(False)
        self.weatherCloudCover.setToolTip("")
        self.weatherCloudCover.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.weatherCloudCover.setMaxLength(8)
        self.weatherCloudCover.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.weatherCloudCover.setReadOnly(True)
        self.weatherCloudCover.setObjectName("weatherCloudCover")
        self.weatherTemp = QtWidgets.QLineEdit(self.weatherGroup)
        self.weatherTemp.setEnabled(False)
        self.weatherTemp.setGeometry(QtCore.QRect(10, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.weatherTemp.setFont(font)
        self.weatherTemp.setMouseTracking(False)
        self.weatherTemp.setFocusPolicy(QtCore.Qt.NoFocus)
        self.weatherTemp.setAcceptDrops(False)
        self.weatherTemp.setToolTip("")
        self.weatherTemp.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.weatherTemp.setMaxLength(8)
        self.weatherTemp.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.weatherTemp.setReadOnly(True)
        self.weatherTemp.setObjectName("weatherTemp")
        self.weatherPress = QtWidgets.QLineEdit(self.weatherGroup)
        self.weatherPress.setEnabled(False)
        self.weatherPress.setGeometry(QtCore.QRect(10, 45, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.weatherPress.setFont(font)
        self.weatherPress.setMouseTracking(False)
        self.weatherPress.setFocusPolicy(QtCore.Qt.NoFocus)
        self.weatherPress.setAcceptDrops(False)
        self.weatherPress.setToolTip("")
        self.weatherPress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.weatherPress.setMaxLength(8)
        self.weatherPress.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.weatherPress.setReadOnly(True)
        self.weatherPress.setObjectName("weatherPress")
        self.weatherHumidity = QtWidgets.QLineEdit(self.weatherGroup)
        self.weatherHumidity.setEnabled(False)
        self.weatherHumidity.setGeometry(QtCore.QRect(10, 95, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.weatherHumidity.setFont(font)
        self.weatherHumidity.setMouseTracking(False)
        self.weatherHumidity.setFocusPolicy(QtCore.Qt.NoFocus)
        self.weatherHumidity.setAcceptDrops(False)
        self.weatherHumidity.setToolTip("")
        self.weatherHumidity.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.weatherHumidity.setMaxLength(8)
        self.weatherHumidity.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.weatherHumidity.setReadOnly(True)
        self.weatherHumidity.setObjectName("weatherHumidity")
        self.weatherWindDir = QtWidgets.QLineEdit(self.weatherGroup)
        self.weatherWindDir.setEnabled(False)
        self.weatherWindDir.setGeometry(QtCore.QRect(10, 195, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.weatherWindDir.setFont(font)
        self.weatherWindDir.setMouseTracking(False)
        self.weatherWindDir.setFocusPolicy(QtCore.Qt.NoFocus)
        self.weatherWindDir.setAcceptDrops(False)
        self.weatherWindDir.setToolTip("")
        self.weatherWindDir.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.weatherWindDir.setMaxLength(8)
        self.weatherWindDir.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.weatherWindDir.setReadOnly(True)
        self.weatherWindDir.setObjectName("weatherWindDir")
        self.skymeterGroup = QtWidgets.QGroupBox(self.Environ)
        self.skymeterGroup.setGeometry(QtCore.QRect(235, 35, 76, 246))
        self.skymeterGroup.setObjectName("skymeterGroup")
        self.skymeterSQR = QtWidgets.QLineEdit(self.skymeterGroup)
        self.skymeterSQR.setEnabled(False)
        self.skymeterSQR.setGeometry(QtCore.QRect(10, 220, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.skymeterSQR.setFont(font)
        self.skymeterSQR.setMouseTracking(False)
        self.skymeterSQR.setFocusPolicy(QtCore.Qt.NoFocus)
        self.skymeterSQR.setAcceptDrops(False)
        self.skymeterSQR.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.skymeterSQR.setMaxLength(8)
        self.skymeterSQR.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.skymeterSQR.setReadOnly(True)
        self.skymeterSQR.setObjectName("skymeterSQR")
        self.skymeterTemp = QtWidgets.QLineEdit(self.skymeterGroup)
        self.skymeterTemp.setEnabled(False)
        self.skymeterTemp.setGeometry(QtCore.QRect(10, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.skymeterTemp.setFont(font)
        self.skymeterTemp.setMouseTracking(False)
        self.skymeterTemp.setFocusPolicy(QtCore.Qt.NoFocus)
        self.skymeterTemp.setAcceptDrops(False)
        self.skymeterTemp.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.skymeterTemp.setMaxLength(8)
        self.skymeterTemp.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.skymeterTemp.setReadOnly(True)
        self.skymeterTemp.setObjectName("skymeterTemp")
        self.label_70 = QtWidgets.QLabel(self.Environ)
        self.label_70.setGeometry(QtCore.QRect(90, 55, 36, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_70.setFont(font)
        self.label_70.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_70.setWordWrap(False)
        self.label_70.setObjectName("label_70")
        self.label_73 = QtWidgets.QLabel(self.Environ)
        self.label_73.setGeometry(QtCore.QRect(90, 80, 36, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_73.setFont(font)
        self.label_73.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_73.setWordWrap(False)
        self.label_73.setObjectName("label_73")
        self.label_87 = QtWidgets.QLabel(self.Environ)
        self.label_87.setGeometry(QtCore.QRect(90, 255, 41, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_87.setFont(font)
        self.label_87.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_87.setWordWrap(False)
        self.label_87.setObjectName("label_87")
        self.label_71 = QtWidgets.QLabel(self.Environ)
        self.label_71.setGeometry(QtCore.QRect(90, 105, 36, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_71.setFont(font)
        self.label_71.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_71.setWordWrap(False)
        self.label_71.setObjectName("label_71")
        self.label_72 = QtWidgets.QLabel(self.Environ)
        self.label_72.setGeometry(QtCore.QRect(90, 130, 36, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_72.setFont(font)
        self.label_72.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_72.setWordWrap(False)
        self.label_72.setObjectName("label_72")
        self.label_143 = QtWidgets.QLabel(self.Environ)
        self.label_143.setGeometry(QtCore.QRect(90, 155, 36, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_143.setFont(font)
        self.label_143.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_143.setWordWrap(False)
        self.label_143.setObjectName("label_143")
        self.label_68 = QtWidgets.QLabel(self.Environ)
        self.label_68.setGeometry(QtCore.QRect(90, 180, 36, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_68.setFont(font)
        self.label_68.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_68.setWordWrap(False)
        self.label_68.setObjectName("label_68")
        self.label_69 = QtWidgets.QLabel(self.Environ)
        self.label_69.setGeometry(QtCore.QRect(90, 230, 36, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_69.setFont(font)
        self.label_69.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_69.setWordWrap(False)
        self.label_69.setObjectName("label_69")
        self.label_67 = QtWidgets.QLabel(self.Environ)
        self.label_67.setGeometry(QtCore.QRect(90, 205, 41, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_67.setFont(font)
        self.label_67.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_67.setWordWrap(False)
        self.label_67.setObjectName("label_67")
        self.powerGroup = QtWidgets.QGroupBox(self.Environ)
        self.powerGroup.setGeometry(QtCore.QRect(325, 35, 76, 246))
        self.powerGroup.setObjectName("powerGroup")
        self.powerTemp = QtWidgets.QLineEdit(self.powerGroup)
        self.powerTemp.setEnabled(False)
        self.powerTemp.setGeometry(QtCore.QRect(10, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.powerTemp.setFont(font)
        self.powerTemp.setMouseTracking(False)
        self.powerTemp.setFocusPolicy(QtCore.Qt.NoFocus)
        self.powerTemp.setAcceptDrops(False)
        self.powerTemp.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.powerTemp.setMaxLength(8)
        self.powerTemp.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerTemp.setReadOnly(True)
        self.powerTemp.setObjectName("powerTemp")
        self.powerDewPoint = QtWidgets.QLineEdit(self.powerGroup)
        self.powerDewPoint.setEnabled(False)
        self.powerDewPoint.setGeometry(QtCore.QRect(10, 70, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.powerDewPoint.setFont(font)
        self.powerDewPoint.setMouseTracking(False)
        self.powerDewPoint.setFocusPolicy(QtCore.Qt.NoFocus)
        self.powerDewPoint.setAcceptDrops(False)
        self.powerDewPoint.setToolTip("")
        self.powerDewPoint.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.powerDewPoint.setMaxLength(8)
        self.powerDewPoint.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerDewPoint.setReadOnly(True)
        self.powerDewPoint.setObjectName("powerDewPoint")
        self.powerHumidity = QtWidgets.QLineEdit(self.powerGroup)
        self.powerHumidity.setEnabled(False)
        self.powerHumidity.setGeometry(QtCore.QRect(10, 95, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.powerHumidity.setFont(font)
        self.powerHumidity.setMouseTracking(False)
        self.powerHumidity.setFocusPolicy(QtCore.Qt.NoFocus)
        self.powerHumidity.setAcceptDrops(False)
        self.powerHumidity.setToolTip("")
        self.powerHumidity.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.powerHumidity.setMaxLength(8)
        self.powerHumidity.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerHumidity.setReadOnly(True)
        self.powerHumidity.setObjectName("powerHumidity")
        self.label_228 = QtWidgets.QLabel(self.Environ)
        self.label_228.setGeometry(QtCore.QRect(15, 10, 281, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_228.setFont(font)
        self.label_228.setObjectName("label_228")
        self.line_53 = QtWidgets.QFrame(self.Environ)
        self.line_53.setGeometry(QtCore.QRect(15, 30, 761, 1))
        self.line_53.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_53.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_53.setObjectName("line_53")
        self.picClearOutside = QtWidgets.QLabel(self.Environ)
        self.picClearOutside.setGeometry(QtCore.QRect(145, 285, 629, 145))
        self.picClearOutside.setFrameShape(QtWidgets.QFrame.Box)
        self.picClearOutside.setFrameShadow(QtWidgets.QFrame.Raised)
        self.picClearOutside.setMidLineWidth(0)
        self.picClearOutside.setProperty("iconpicture", True)
        self.picClearOutside.setObjectName("picClearOutside")
        self.picClearOutsideHeader = QtWidgets.QLabel(self.Environ)
        self.picClearOutsideHeader.setGeometry(QtCore.QRect(10, 285, 131, 145))
        self.picClearOutsideHeader.setFrameShape(QtWidgets.QFrame.Box)
        self.picClearOutsideHeader.setFrameShadow(QtWidgets.QFrame.Raised)
        self.picClearOutsideHeader.setMidLineWidth(0)
        self.picClearOutsideHeader.setProperty("iconpicture", True)
        self.picClearOutsideHeader.setObjectName("picClearOutsideHeader")
        self.mainTabWidget.addTab(self.Environ, "")
        self.AlignMount = QtWidgets.QWidget()
        self.AlignMount.setObjectName("AlignMount")
        self.label = QtWidgets.QLabel(self.AlignMount)
        self.label.setGeometry(QtCore.QRect(410, 60, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.azimuthError = QtWidgets.QLineEdit(self.AlignMount)
        self.azimuthError.setEnabled(False)
        self.azimuthError.setGeometry(QtCore.QRect(680, 90, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.azimuthError.setFont(font)
        self.azimuthError.setMouseTracking(False)
        self.azimuthError.setFocusPolicy(QtCore.Qt.NoFocus)
        self.azimuthError.setAcceptDrops(False)
        self.azimuthError.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.azimuthError.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azimuthError.setReadOnly(True)
        self.azimuthError.setObjectName("azimuthError")
        self.positionAngle = QtWidgets.QLineEdit(self.AlignMount)
        self.positionAngle.setEnabled(False)
        self.positionAngle.setGeometry(QtCore.QRect(500, 150, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.positionAngle.setFont(font)
        self.positionAngle.setMouseTracking(False)
        self.positionAngle.setFocusPolicy(QtCore.Qt.NoFocus)
        self.positionAngle.setAcceptDrops(False)
        self.positionAngle.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.positionAngle.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.positionAngle.setReadOnly(True)
        self.positionAngle.setObjectName("positionAngle")
        self.azimuthTurns = QtWidgets.QLineEdit(self.AlignMount)
        self.azimuthTurns.setEnabled(False)
        self.azimuthTurns.setGeometry(QtCore.QRect(700, 260, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.azimuthTurns.setFont(font)
        self.azimuthTurns.setMouseTracking(False)
        self.azimuthTurns.setFocusPolicy(QtCore.Qt.NoFocus)
        self.azimuthTurns.setAcceptDrops(False)
        self.azimuthTurns.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.azimuthTurns.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azimuthTurns.setReadOnly(True)
        self.azimuthTurns.setObjectName("azimuthTurns")
        self.polarError = QtWidgets.QLineEdit(self.AlignMount)
        self.polarError.setEnabled(False)
        self.polarError.setGeometry(QtCore.QRect(680, 150, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.polarError.setFont(font)
        self.polarError.setMouseTracking(False)
        self.polarError.setFocusPolicy(QtCore.Qt.NoFocus)
        self.polarError.setAcceptDrops(False)
        self.polarError.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.polarError.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.polarError.setReadOnly(True)
        self.polarError.setObjectName("polarError")
        self.label_14 = QtWidgets.QLabel(self.AlignMount)
        self.label_14.setGeometry(QtCore.QRect(410, 90, 91, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_14.setFont(font)
        self.label_14.setObjectName("label_14")
        self.label_153 = QtWidgets.QLabel(self.AlignMount)
        self.label_153.setGeometry(QtCore.QRect(575, 120, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_153.setFont(font)
        self.label_153.setObjectName("label_153")
        self.terms = QtWidgets.QLineEdit(self.AlignMount)
        self.terms.setEnabled(False)
        self.terms.setGeometry(QtCore.QRect(500, 120, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.terms.setFont(font)
        self.terms.setMouseTracking(False)
        self.terms.setFocusPolicy(QtCore.Qt.NoFocus)
        self.terms.setAcceptDrops(False)
        self.terms.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.terms.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.terms.setReadOnly(True)
        self.terms.setObjectName("terms")
        self.altitudeTurns = QtWidgets.QLineEdit(self.AlignMount)
        self.altitudeTurns.setEnabled(False)
        self.altitudeTurns.setGeometry(QtCore.QRect(500, 260, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altitudeTurns.setFont(font)
        self.altitudeTurns.setMouseTracking(False)
        self.altitudeTurns.setFocusPolicy(QtCore.Qt.NoFocus)
        self.altitudeTurns.setAcceptDrops(False)
        self.altitudeTurns.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.altitudeTurns.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeTurns.setReadOnly(True)
        self.altitudeTurns.setObjectName("altitudeTurns")
        self.label_156 = QtWidgets.QLabel(self.AlignMount)
        self.label_156.setGeometry(QtCore.QRect(615, 260, 91, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_156.setFont(font)
        self.label_156.setObjectName("label_156")
        self.label_158 = QtWidgets.QLabel(self.AlignMount)
        self.label_158.setGeometry(QtCore.QRect(575, 90, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_158.setFont(font)
        self.label_158.setObjectName("label_158")
        self.numberStars = QtWidgets.QLineEdit(self.AlignMount)
        self.numberStars.setEnabled(False)
        self.numberStars.setGeometry(QtCore.QRect(500, 60, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.numberStars.setFont(font)
        self.numberStars.setMouseTracking(False)
        self.numberStars.setFocusPolicy(QtCore.Qt.NoFocus)
        self.numberStars.setAcceptDrops(False)
        self.numberStars.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.numberStars.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberStars.setReadOnly(True)
        self.numberStars.setObjectName("numberStars")
        self.errorRMS = QtWidgets.QLineEdit(self.AlignMount)
        self.errorRMS.setEnabled(False)
        self.errorRMS.setGeometry(QtCore.QRect(500, 90, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.errorRMS.setFont(font)
        self.errorRMS.setMouseTracking(False)
        self.errorRMS.setFocusPolicy(QtCore.Qt.NoFocus)
        self.errorRMS.setAcceptDrops(False)
        self.errorRMS.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.errorRMS.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.errorRMS.setReadOnly(True)
        self.errorRMS.setObjectName("errorRMS")
        self.label_155 = QtWidgets.QLabel(self.AlignMount)
        self.label_155.setGeometry(QtCore.QRect(410, 15, 291, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_155.setFont(font)
        self.label_155.setObjectName("label_155")
        self.orthoError = QtWidgets.QLineEdit(self.AlignMount)
        self.orthoError.setEnabled(False)
        self.orthoError.setGeometry(QtCore.QRect(680, 120, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.orthoError.setFont(font)
        self.orthoError.setMouseTracking(False)
        self.orthoError.setFocusPolicy(QtCore.Qt.NoFocus)
        self.orthoError.setAcceptDrops(False)
        self.orthoError.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.orthoError.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.orthoError.setReadOnly(True)
        self.orthoError.setObjectName("orthoError")
        self.label_159 = QtWidgets.QLabel(self.AlignMount)
        self.label_159.setGeometry(QtCore.QRect(575, 60, 106, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_159.setFont(font)
        self.label_159.setObjectName("label_159")
        self.label_15 = QtWidgets.QLabel(self.AlignMount)
        self.label_15.setGeometry(QtCore.QRect(575, 150, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_15.setFont(font)
        self.label_15.setObjectName("label_15")
        self.altitudeError = QtWidgets.QLineEdit(self.AlignMount)
        self.altitudeError.setEnabled(False)
        self.altitudeError.setGeometry(QtCore.QRect(680, 60, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altitudeError.setFont(font)
        self.altitudeError.setMouseTracking(False)
        self.altitudeError.setFocusPolicy(QtCore.Qt.NoFocus)
        self.altitudeError.setAcceptDrops(False)
        self.altitudeError.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.altitudeError.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeError.setReadOnly(True)
        self.altitudeError.setObjectName("altitudeError")
        self.label_154 = QtWidgets.QLabel(self.AlignMount)
        self.label_154.setGeometry(QtCore.QRect(410, 120, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_154.setFont(font)
        self.label_154.setObjectName("label_154")
        self.label_157 = QtWidgets.QLabel(self.AlignMount)
        self.label_157.setGeometry(QtCore.QRect(410, 260, 91, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_157.setFont(font)
        self.label_157.setObjectName("label_157")
        self.line_54 = QtWidgets.QFrame(self.AlignMount)
        self.line_54.setGeometry(QtCore.QRect(410, 35, 366, 1))
        self.line_54.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_54.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_54.setObjectName("line_54")
        self.label_160 = QtWidgets.QLabel(self.AlignMount)
        self.label_160.setGeometry(QtCore.QRect(410, 150, 71, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_160.setFont(font)
        self.label_160.setObjectName("label_160")
        self.line_64 = QtWidgets.QFrame(self.AlignMount)
        self.line_64.setGeometry(QtCore.QRect(390, 10, 11, 416))
        self.line_64.setStyleSheet("")
        self.line_64.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_64.setLineWidth(2)
        self.line_64.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_64.setObjectName("line_64")
        self.picAZ = QtWidgets.QLabel(self.AlignMount)
        self.picAZ.setGeometry(QtCore.QRect(630, 280, 144, 144))
        self.picAZ.setFrameShape(QtWidgets.QFrame.Box)
        self.picAZ.setFrameShadow(QtWidgets.QFrame.Raised)
        self.picAZ.setMidLineWidth(0)
        self.picAZ.setProperty("iconpicture", True)
        self.picAZ.setObjectName("picAZ")
        self.picALT = QtWidgets.QLabel(self.AlignMount)
        self.picALT.setGeometry(QtCore.QRect(425, 280, 144, 144))
        self.picALT.setFrameShape(QtWidgets.QFrame.Box)
        self.picALT.setFrameShadow(QtWidgets.QFrame.Raised)
        self.picALT.setMidLineWidth(0)
        self.picALT.setProperty("iconpicture", True)
        self.picALT.setObjectName("picALT")
        self.label_167 = QtWidgets.QLabel(self.AlignMount)
        self.label_167.setGeometry(QtCore.QRect(735, 60, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_167.setFont(font)
        self.label_167.setAlignment(QtCore.Qt.AlignCenter)
        self.label_167.setWordWrap(False)
        self.label_167.setObjectName("label_167")
        self.label_168 = QtWidgets.QLabel(self.AlignMount)
        self.label_168.setGeometry(QtCore.QRect(735, 90, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_168.setFont(font)
        self.label_168.setAlignment(QtCore.Qt.AlignCenter)
        self.label_168.setWordWrap(False)
        self.label_168.setObjectName("label_168")
        self.label_169 = QtWidgets.QLabel(self.AlignMount)
        self.label_169.setGeometry(QtCore.QRect(735, 120, 46, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_169.setFont(font)
        self.label_169.setAlignment(QtCore.Qt.AlignCenter)
        self.label_169.setWordWrap(False)
        self.label_169.setObjectName("label_169")
        self.label_218 = QtWidgets.QLabel(self.AlignMount)
        self.label_218.setGeometry(QtCore.QRect(545, 150, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_218.setFont(font)
        self.label_218.setAlignment(QtCore.Qt.AlignCenter)
        self.label_218.setWordWrap(False)
        self.label_218.setObjectName("label_218")
        self.label_201 = QtWidgets.QLabel(self.AlignMount)
        self.label_201.setGeometry(QtCore.QRect(735, 150, 46, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_201.setFont(font)
        self.label_201.setAlignment(QtCore.Qt.AlignCenter)
        self.label_201.setWordWrap(False)
        self.label_201.setObjectName("label_201")
        self.label_219 = QtWidgets.QLabel(self.AlignMount)
        self.label_219.setGeometry(QtCore.QRect(410, 235, 156, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_219.setFont(font)
        self.label_219.setObjectName("label_219")
        self.line_75 = QtWidgets.QFrame(self.AlignMount)
        self.line_75.setGeometry(QtCore.QRect(410, 255, 371, 1))
        self.line_75.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_75.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_75.setObjectName("line_75")
        self.groupBox_10 = QtWidgets.QGroupBox(self.AlignMount)
        self.groupBox_10.setGeometry(QtCore.QRect(15, 300, 366, 51))
        self.groupBox_10.setProperty("large", True)
        self.groupBox_10.setObjectName("groupBox_10")
        self.cancelAlignModel = QtWidgets.QPushButton(self.groupBox_10)
        self.cancelAlignModel.setEnabled(False)
        self.cancelAlignModel.setGeometry(QtCore.QRect(190, 20, 171, 26))
        self.cancelAlignModel.setStyleSheet("")
        self.cancelAlignModel.setObjectName("cancelAlignModel")
        self.runAlignModel = QtWidgets.QPushButton(self.groupBox_10)
        self.runAlignModel.setGeometry(QtCore.QRect(5, 20, 171, 26))
        self.runAlignModel.setObjectName("runAlignModel")
        self.groupBox_11 = QtWidgets.QGroupBox(self.AlignMount)
        self.groupBox_11.setGeometry(QtCore.QRect(15, 125, 366, 76))
        self.groupBox_11.setProperty("large", True)
        self.groupBox_11.setObjectName("groupBox_11")
        self.genAlignBuild = QtWidgets.QPushButton(self.groupBox_11)
        self.genAlignBuild.setGeometry(QtCore.QRect(5, 45, 356, 26))
        self.genAlignBuild.setObjectName("genAlignBuild")
        self.numberBase = QtWidgets.QDoubleSpinBox(self.groupBox_11)
        self.numberBase.setGeometry(QtCore.QRect(320, 20, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.numberBase.setFont(font)
        self.numberBase.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberBase.setDecimals(0)
        self.numberBase.setMinimum(3.0)
        self.numberBase.setMaximum(10.0)
        self.numberBase.setSingleStep(1.0)
        self.numberBase.setProperty("value", 3.0)
        self.numberBase.setObjectName("numberBase")
        self.label_127 = QtWidgets.QLabel(self.groupBox_11)
        self.label_127.setGeometry(QtCore.QRect(110, 20, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_127.setFont(font)
        self.label_127.setAlignment(QtCore.Qt.AlignCenter)
        self.label_127.setWordWrap(False)
        self.label_127.setObjectName("label_127")
        self.label_206 = QtWidgets.QLabel(self.groupBox_11)
        self.label_206.setGeometry(QtCore.QRect(265, 20, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_206.setFont(font)
        self.label_206.setObjectName("label_206")
        self.label_171 = QtWidgets.QLabel(self.groupBox_11)
        self.label_171.setGeometry(QtCore.QRect(130, 20, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_171.setFont(font)
        self.label_171.setObjectName("label_171")
        self.label_170 = QtWidgets.QLabel(self.groupBox_11)
        self.label_170.setGeometry(QtCore.QRect(10, 20, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_170.setFont(font)
        self.label_170.setObjectName("label_170")
        self.label_128 = QtWidgets.QLabel(self.groupBox_11)
        self.label_128.setGeometry(QtCore.QRect(235, 20, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_128.setFont(font)
        self.label_128.setAlignment(QtCore.Qt.AlignCenter)
        self.label_128.setWordWrap(False)
        self.label_128.setObjectName("label_128")
        self.azBase = QtWidgets.QDoubleSpinBox(self.groupBox_11)
        self.azBase.setGeometry(QtCore.QRect(185, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.azBase.setFont(font)
        self.azBase.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azBase.setDecimals(0)
        self.azBase.setMaximum(120.0)
        self.azBase.setSingleStep(1.0)
        self.azBase.setProperty("value", 50.0)
        self.azBase.setObjectName("azBase")
        self.altBase = QtWidgets.QDoubleSpinBox(self.groupBox_11)
        self.altBase.setGeometry(QtCore.QRect(60, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altBase.setFont(font)
        self.altBase.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altBase.setDecimals(0)
        self.altBase.setMinimum(20.0)
        self.altBase.setMaximum(70.0)
        self.altBase.setSingleStep(1.0)
        self.altBase.setProperty("value", 50.0)
        self.altBase.setObjectName("altBase")
        self.groupBox_12 = QtWidgets.QGroupBox(self.AlignMount)
        self.groupBox_12.setGeometry(QtCore.QRect(15, 25, 366, 76))
        self.groupBox_12.setProperty("large", True)
        self.groupBox_12.setObjectName("groupBox_12")
        self.alignBuildPFileName = QtWidgets.QLineEdit(self.groupBox_12)
        self.alignBuildPFileName.setEnabled(False)
        self.alignBuildPFileName.setGeometry(QtCore.QRect(5, 20, 356, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.alignBuildPFileName.setFont(font)
        self.alignBuildPFileName.setMouseTracking(False)
        self.alignBuildPFileName.setFocusPolicy(QtCore.Qt.NoFocus)
        self.alignBuildPFileName.setAcceptDrops(False)
        self.alignBuildPFileName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.alignBuildPFileName.setText("")
        self.alignBuildPFileName.setReadOnly(True)
        self.alignBuildPFileName.setObjectName("alignBuildPFileName")
        self.saveAlignBuildPointsAs = QtWidgets.QPushButton(self.groupBox_12)
        self.saveAlignBuildPointsAs.setEnabled(True)
        self.saveAlignBuildPointsAs.setGeometry(QtCore.QRect(280, 45, 81, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.saveAlignBuildPointsAs.setFont(font)
        self.saveAlignBuildPointsAs.setStyleSheet("")
        self.saveAlignBuildPointsAs.setObjectName("saveAlignBuildPointsAs")
        self.saveAlignBuildPoints = QtWidgets.QPushButton(self.groupBox_12)
        self.saveAlignBuildPoints.setEnabled(True)
        self.saveAlignBuildPoints.setGeometry(QtCore.QRect(195, 45, 81, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.saveAlignBuildPoints.setFont(font)
        self.saveAlignBuildPoints.setStyleSheet("")
        self.saveAlignBuildPoints.setObjectName("saveAlignBuildPoints")
        self.genAlignBuildFile = QtWidgets.QPushButton(self.groupBox_12)
        self.genAlignBuildFile.setGeometry(QtCore.QRect(5, 45, 101, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.genAlignBuildFile.setFont(font)
        self.genAlignBuildFile.setStyleSheet("")
        self.genAlignBuildFile.setObjectName("genAlignBuildFile")
        self.loadAlignBuildPoints = QtWidgets.QPushButton(self.groupBox_12)
        self.loadAlignBuildPoints.setGeometry(QtCore.QRect(110, 45, 81, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.loadAlignBuildPoints.setFont(font)
        self.loadAlignBuildPoints.setStyleSheet("")
        self.loadAlignBuildPoints.setObjectName("loadAlignBuildPoints")
        self.groupBox_9 = QtWidgets.QGroupBox(self.AlignMount)
        self.groupBox_9.setGeometry(QtCore.QRect(15, 225, 366, 51))
        self.groupBox_9.setProperty("large", True)
        self.groupBox_9.setObjectName("groupBox_9")
        self.plateSolveSync = QtWidgets.QPushButton(self.groupBox_9)
        self.plateSolveSync.setEnabled(True)
        self.plateSolveSync.setGeometry(QtCore.QRect(5, 20, 356, 26))
        self.plateSolveSync.setObjectName("plateSolveSync")
        self.line_54.raise_()
        self.label.raise_()
        self.azimuthError.raise_()
        self.positionAngle.raise_()
        self.azimuthTurns.raise_()
        self.polarError.raise_()
        self.label_14.raise_()
        self.label_153.raise_()
        self.terms.raise_()
        self.altitudeTurns.raise_()
        self.label_156.raise_()
        self.label_158.raise_()
        self.numberStars.raise_()
        self.errorRMS.raise_()
        self.label_155.raise_()
        self.orthoError.raise_()
        self.label_159.raise_()
        self.label_15.raise_()
        self.altitudeError.raise_()
        self.label_154.raise_()
        self.label_157.raise_()
        self.label_160.raise_()
        self.line_64.raise_()
        self.picAZ.raise_()
        self.picALT.raise_()
        self.label_167.raise_()
        self.label_168.raise_()
        self.label_169.raise_()
        self.label_218.raise_()
        self.label_201.raise_()
        self.label_219.raise_()
        self.line_75.raise_()
        self.groupBox_10.raise_()
        self.groupBox_11.raise_()
        self.groupBox_12.raise_()
        self.groupBox_9.raise_()
        self.mainTabWidget.addTab(self.AlignMount, "")
        self.BuildModel = QtWidgets.QWidget()
        self.BuildModel.setObjectName("BuildModel")
        self.line_40 = QtWidgets.QFrame(self.BuildModel)
        self.line_40.setGeometry(QtCore.QRect(390, 10, 11, 416))
        self.line_40.setStyleSheet("")
        self.line_40.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_40.setLineWidth(2)
        self.line_40.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_40.setObjectName("line_40")
        self.groupBox_2 = QtWidgets.QGroupBox(self.BuildModel)
        self.groupBox_2.setGeometry(QtCore.QRect(15, 25, 366, 76))
        self.groupBox_2.setProperty("large", True)
        self.groupBox_2.setObjectName("groupBox_2")
        self.buildPFileName = QtWidgets.QLineEdit(self.groupBox_2)
        self.buildPFileName.setEnabled(False)
        self.buildPFileName.setGeometry(QtCore.QRect(5, 20, 356, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.buildPFileName.setFont(font)
        self.buildPFileName.setMouseTracking(False)
        self.buildPFileName.setAcceptDrops(False)
        self.buildPFileName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.buildPFileName.setText("")
        self.buildPFileName.setReadOnly(True)
        self.buildPFileName.setObjectName("buildPFileName")
        self.saveBuildPointsAs = QtWidgets.QPushButton(self.groupBox_2)
        self.saveBuildPointsAs.setEnabled(True)
        self.saveBuildPointsAs.setGeometry(QtCore.QRect(280, 45, 81, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.saveBuildPointsAs.setFont(font)
        self.saveBuildPointsAs.setStyleSheet("")
        self.saveBuildPointsAs.setObjectName("saveBuildPointsAs")
        self.saveBuildPoints = QtWidgets.QPushButton(self.groupBox_2)
        self.saveBuildPoints.setEnabled(True)
        self.saveBuildPoints.setGeometry(QtCore.QRect(195, 45, 81, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.saveBuildPoints.setFont(font)
        self.saveBuildPoints.setStyleSheet("")
        self.saveBuildPoints.setObjectName("saveBuildPoints")
        self.genBuildFile = QtWidgets.QPushButton(self.groupBox_2)
        self.genBuildFile.setGeometry(QtCore.QRect(5, 45, 101, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.genBuildFile.setFont(font)
        self.genBuildFile.setStyleSheet("")
        self.genBuildFile.setObjectName("genBuildFile")
        self.loadBuildPoints = QtWidgets.QPushButton(self.groupBox_2)
        self.loadBuildPoints.setGeometry(QtCore.QRect(110, 45, 81, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.loadBuildPoints.setFont(font)
        self.loadBuildPoints.setStyleSheet("")
        self.loadBuildPoints.setObjectName("loadBuildPoints")
        self.groupBox_3 = QtWidgets.QGroupBox(self.BuildModel)
        self.groupBox_3.setGeometry(QtCore.QRect(15, 115, 366, 81))
        self.groupBox_3.setProperty("large", True)
        self.groupBox_3.setObjectName("groupBox_3")
        self.label_178 = QtWidgets.QLabel(self.groupBox_3)
        self.label_178.setGeometry(QtCore.QRect(5, 25, 36, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_178.setFont(font)
        self.label_178.setObjectName("label_178")
        self.numberGridPointsRow = QtWidgets.QDoubleSpinBox(self.groupBox_3)
        self.numberGridPointsRow.setGeometry(QtCore.QRect(40, 25, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.numberGridPointsRow.setFont(font)
        self.numberGridPointsRow.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberGridPointsRow.setDecimals(0)
        self.numberGridPointsRow.setMinimum(2.0)
        self.numberGridPointsRow.setMaximum(8.0)
        self.numberGridPointsRow.setSingleStep(1.0)
        self.numberGridPointsRow.setProperty("value", 5.0)
        self.numberGridPointsRow.setObjectName("numberGridPointsRow")
        self.label_179 = QtWidgets.QLabel(self.groupBox_3)
        self.label_179.setGeometry(QtCore.QRect(90, 25, 26, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_179.setFont(font)
        self.label_179.setObjectName("label_179")
        self.label_190 = QtWidgets.QLabel(self.groupBox_3)
        self.label_190.setGeometry(QtCore.QRect(170, 25, 41, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_190.setFont(font)
        self.label_190.setObjectName("label_190")
        self.label_203 = QtWidgets.QLabel(self.groupBox_3)
        self.label_203.setGeometry(QtCore.QRect(250, 25, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_203.setFont(font)
        self.label_203.setAlignment(QtCore.Qt.AlignCenter)
        self.label_203.setWordWrap(False)
        self.label_203.setObjectName("label_203")
        self.altitudeMax = QtWidgets.QDoubleSpinBox(self.groupBox_3)
        self.altitudeMax.setGeometry(QtCore.QRect(310, 25, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.altitudeMax.setFont(font)
        self.altitudeMax.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeMax.setDecimals(0)
        self.altitudeMax.setMinimum(5.0)
        self.altitudeMax.setMaximum(85.0)
        self.altitudeMax.setSingleStep(5.0)
        self.altitudeMax.setProperty("value", 75.0)
        self.altitudeMax.setObjectName("altitudeMax")
        self.label_144 = QtWidgets.QLabel(self.groupBox_3)
        self.label_144.setGeometry(QtCore.QRect(350, 25, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_144.setFont(font)
        self.label_144.setAlignment(QtCore.Qt.AlignCenter)
        self.label_144.setWordWrap(False)
        self.label_144.setObjectName("label_144")
        self.numberGridPointsCol = QtWidgets.QDoubleSpinBox(self.groupBox_3)
        self.numberGridPointsCol.setGeometry(QtCore.QRect(120, 25, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.numberGridPointsCol.setFont(font)
        self.numberGridPointsCol.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberGridPointsCol.setDecimals(0)
        self.numberGridPointsCol.setMinimum(2.0)
        self.numberGridPointsCol.setMaximum(15.0)
        self.numberGridPointsCol.setSingleStep(2.0)
        self.numberGridPointsCol.setProperty("value", 6.0)
        self.numberGridPointsCol.setObjectName("numberGridPointsCol")
        self.label_180 = QtWidgets.QLabel(self.groupBox_3)
        self.label_180.setGeometry(QtCore.QRect(265, 25, 46, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_180.setFont(font)
        self.label_180.setObjectName("label_180")
        self.genBuildGrid = QtWidgets.QPushButton(self.groupBox_3)
        self.genBuildGrid.setGeometry(QtCore.QRect(5, 50, 356, 26))
        self.genBuildGrid.setObjectName("genBuildGrid")
        self.altitudeMin = QtWidgets.QDoubleSpinBox(self.groupBox_3)
        self.altitudeMin.setGeometry(QtCore.QRect(210, 25, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.altitudeMin.setFont(font)
        self.altitudeMin.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeMin.setDecimals(0)
        self.altitudeMin.setMinimum(5.0)
        self.altitudeMin.setMaximum(85.0)
        self.altitudeMin.setSingleStep(5.0)
        self.altitudeMin.setProperty("value", 30.0)
        self.altitudeMin.setObjectName("altitudeMin")
        self.groupBox_4 = QtWidgets.QGroupBox(self.BuildModel)
        self.groupBox_4.setGeometry(QtCore.QRect(15, 210, 366, 51))
        self.groupBox_4.setProperty("large", True)
        self.groupBox_4.setObjectName("groupBox_4")
        self.genBuildMax = QtWidgets.QPushButton(self.groupBox_4)
        self.genBuildMax.setGeometry(QtCore.QRect(5, 20, 86, 26))
        self.genBuildMax.setObjectName("genBuildMax")
        self.genBuildMed = QtWidgets.QPushButton(self.groupBox_4)
        self.genBuildMed.setGeometry(QtCore.QRect(95, 20, 86, 26))
        self.genBuildMed.setObjectName("genBuildMed")
        self.genBuildMin = QtWidgets.QPushButton(self.groupBox_4)
        self.genBuildMin.setGeometry(QtCore.QRect(275, 20, 86, 26))
        self.genBuildMin.setObjectName("genBuildMin")
        self.genBuildNorm = QtWidgets.QPushButton(self.groupBox_4)
        self.genBuildNorm.setGeometry(QtCore.QRect(185, 20, 86, 26))
        self.genBuildNorm.setObjectName("genBuildNorm")
        self.groupBox_5 = QtWidgets.QGroupBox(self.BuildModel)
        self.groupBox_5.setGeometry(QtCore.QRect(15, 280, 366, 76))
        self.groupBox_5.setProperty("large", True)
        self.groupBox_5.setObjectName("groupBox_5")
        self.label_188 = QtWidgets.QLabel(self.groupBox_5)
        self.label_188.setGeometry(QtCore.QRect(5, 20, 41, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_188.setFont(font)
        self.label_188.setObjectName("label_188")
        self.numberDSOPoints = QtWidgets.QDoubleSpinBox(self.groupBox_5)
        self.numberDSOPoints.setGeometry(QtCore.QRect(45, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.numberDSOPoints.setFont(font)
        self.numberDSOPoints.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberDSOPoints.setDecimals(0)
        self.numberDSOPoints.setMinimum(5.0)
        self.numberDSOPoints.setMaximum(50.0)
        self.numberDSOPoints.setSingleStep(1.0)
        self.numberDSOPoints.setProperty("value", 10.0)
        self.numberDSOPoints.setObjectName("numberDSOPoints")
        self.durationDSO = QtWidgets.QDoubleSpinBox(self.groupBox_5)
        self.durationDSO.setGeometry(QtCore.QRect(165, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.durationDSO.setFont(font)
        self.durationDSO.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.durationDSO.setDecimals(0)
        self.durationDSO.setMinimum(1.0)
        self.durationDSO.setMaximum(15.0)
        self.durationDSO.setSingleStep(1.0)
        self.durationDSO.setProperty("value", 10.0)
        self.durationDSO.setObjectName("durationDSO")
        self.timeShiftDSO = QtWidgets.QDoubleSpinBox(self.groupBox_5)
        self.timeShiftDSO.setGeometry(QtCore.QRect(305, 20, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.timeShiftDSO.setFont(font)
        self.timeShiftDSO.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.timeShiftDSO.setDecimals(0)
        self.timeShiftDSO.setMinimum(-12.0)
        self.timeShiftDSO.setMaximum(12.0)
        self.timeShiftDSO.setSingleStep(1.0)
        self.timeShiftDSO.setProperty("value", 0.0)
        self.timeShiftDSO.setObjectName("timeShiftDSO")
        self.label_204 = QtWidgets.QLabel(self.groupBox_5)
        self.label_204.setGeometry(QtCore.QRect(350, 20, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_204.setFont(font)
        self.label_204.setObjectName("label_204")
        self.label_189 = QtWidgets.QLabel(self.groupBox_5)
        self.label_189.setGeometry(QtCore.QRect(245, 20, 61, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_189.setFont(font)
        self.label_189.setObjectName("label_189")
        self.label_303 = QtWidgets.QLabel(self.groupBox_5)
        self.label_303.setGeometry(QtCore.QRect(220, 20, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_303.setFont(font)
        self.label_303.setObjectName("label_303")
        self.label_181 = QtWidgets.QLabel(self.groupBox_5)
        self.label_181.setGeometry(QtCore.QRect(110, 20, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_181.setFont(font)
        self.label_181.setObjectName("label_181")
        self.genBuildDSO = QtWidgets.QPushButton(self.groupBox_5)
        self.genBuildDSO.setGeometry(QtCore.QRect(5, 45, 356, 26))
        self.genBuildDSO.setObjectName("genBuildDSO")
        self.groupBox_6 = QtWidgets.QGroupBox(self.BuildModel)
        self.groupBox_6.setGeometry(QtCore.QRect(15, 370, 366, 51))
        self.groupBox_6.setProperty("large", True)
        self.groupBox_6.setObjectName("groupBox_6")
        self.genBuildGoldenSpiral = QtWidgets.QPushButton(self.groupBox_6)
        self.genBuildGoldenSpiral.setGeometry(QtCore.QRect(130, 20, 231, 26))
        self.genBuildGoldenSpiral.setObjectName("genBuildGoldenSpiral")
        self.numberSpiralPoints = QtWidgets.QDoubleSpinBox(self.groupBox_6)
        self.numberSpiralPoints.setGeometry(QtCore.QRect(45, 20, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.numberSpiralPoints.setFont(font)
        self.numberSpiralPoints.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberSpiralPoints.setDecimals(0)
        self.numberSpiralPoints.setMinimum(20.0)
        self.numberSpiralPoints.setMaximum(350.0)
        self.numberSpiralPoints.setSingleStep(5.0)
        self.numberSpiralPoints.setProperty("value", 20.0)
        self.numberSpiralPoints.setObjectName("numberSpiralPoints")
        self.label_309 = QtWidgets.QLabel(self.groupBox_6)
        self.label_309.setGeometry(QtCore.QRect(5, 20, 41, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_309.setFont(font)
        self.label_309.setObjectName("label_309")
        self.groupBox_7 = QtWidgets.QGroupBox(self.BuildModel)
        self.groupBox_7.setGeometry(QtCore.QRect(410, 370, 366, 51))
        self.groupBox_7.setProperty("large", True)
        self.groupBox_7.setObjectName("groupBox_7")
        self.batchModel = QtWidgets.QPushButton(self.groupBox_7)
        self.batchModel.setEnabled(True)
        self.batchModel.setGeometry(QtCore.QRect(5, 20, 356, 26))
        self.batchModel.setObjectName("batchModel")
        self.groupBox_8 = QtWidgets.QGroupBox(self.BuildModel)
        self.groupBox_8.setGeometry(QtCore.QRect(410, 25, 366, 171))
        self.groupBox_8.setProperty("large", True)
        self.groupBox_8.setObjectName("groupBox_8")
        self.cancelFullModel = QtWidgets.QPushButton(self.groupBox_8)
        self.cancelFullModel.setEnabled(False)
        self.cancelFullModel.setGeometry(QtCore.QRect(190, 20, 171, 26))
        self.cancelFullModel.setStyleSheet("")
        self.cancelFullModel.setObjectName("cancelFullModel")
        self.mSlew = QtWidgets.QLineEdit(self.groupBox_8)
        self.mSlew.setEnabled(False)
        self.mSlew.setGeometry(QtCore.QRect(145, 105, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mSlew.setFont(font)
        self.mSlew.setMouseTracking(False)
        self.mSlew.setFocusPolicy(QtCore.Qt.NoFocus)
        self.mSlew.setAcceptDrops(False)
        self.mSlew.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.mSlew.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.mSlew.setReadOnly(True)
        self.mSlew.setObjectName("mSlew")
        self.label_35 = QtWidgets.QLabel(self.groupBox_8)
        self.label_35.setGeometry(QtCore.QRect(195, 55, 91, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_35.setFont(font)
        self.label_35.setObjectName("label_35")
        self.runFullModel = QtWidgets.QPushButton(self.groupBox_8)
        self.runFullModel.setGeometry(QtCore.QRect(5, 20, 171, 26))
        self.runFullModel.setObjectName("runFullModel")
        self.mPoints = QtWidgets.QLineEdit(self.groupBox_8)
        self.mPoints.setEnabled(False)
        self.mPoints.setGeometry(QtCore.QRect(50, 105, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mPoints.setFont(font)
        self.mPoints.setMouseTracking(False)
        self.mPoints.setFocusPolicy(QtCore.Qt.NoFocus)
        self.mPoints.setAcceptDrops(False)
        self.mPoints.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.mPoints.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.mPoints.setReadOnly(True)
        self.mPoints.setObjectName("mPoints")
        self.label_29 = QtWidgets.QLabel(self.groupBox_8)
        self.label_29.setGeometry(QtCore.QRect(195, 80, 81, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_29.setFont(font)
        self.label_29.setObjectName("label_29")
        self.mSolve = QtWidgets.QLineEdit(self.groupBox_8)
        self.mSolve.setEnabled(False)
        self.mSolve.setGeometry(QtCore.QRect(330, 105, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mSolve.setFont(font)
        self.mSolve.setMouseTracking(False)
        self.mSolve.setFocusPolicy(QtCore.Qt.NoFocus)
        self.mSolve.setAcceptDrops(False)
        self.mSolve.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.mSolve.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.mSolve.setReadOnly(True)
        self.mSolve.setObjectName("mSolve")
        self.label_310 = QtWidgets.QLabel(self.groupBox_8)
        self.label_310.setGeometry(QtCore.QRect(100, 105, 51, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_310.setFont(font)
        self.label_310.setObjectName("label_310")
        self.label_311 = QtWidgets.QLabel(self.groupBox_8)
        self.label_311.setGeometry(QtCore.QRect(290, 105, 46, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_311.setFont(font)
        self.label_311.setObjectName("label_311")
        self.mImage = QtWidgets.QLineEdit(self.groupBox_8)
        self.mImage.setEnabled(False)
        self.mImage.setGeometry(QtCore.QRect(245, 105, 31, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mImage.setFont(font)
        self.mImage.setMouseTracking(False)
        self.mImage.setFocusPolicy(QtCore.Qt.NoFocus)
        self.mImage.setAcceptDrops(False)
        self.mImage.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.mImage.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.mImage.setReadOnly(True)
        self.mImage.setObjectName("mImage")
        self.timeElapsed = QtWidgets.QLineEdit(self.groupBox_8)
        self.timeElapsed.setEnabled(False)
        self.timeElapsed.setGeometry(QtCore.QRect(300, 80, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.timeElapsed.setFont(font)
        self.timeElapsed.setMouseTracking(False)
        self.timeElapsed.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeElapsed.setAcceptDrops(False)
        self.timeElapsed.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.timeElapsed.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.timeElapsed.setReadOnly(True)
        self.timeElapsed.setObjectName("timeElapsed")
        self.label_216 = QtWidgets.QLabel(self.groupBox_8)
        self.label_216.setGeometry(QtCore.QRect(5, 105, 51, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_216.setFont(font)
        self.label_216.setObjectName("label_216")
        self.timeFinished = QtWidgets.QLineEdit(self.groupBox_8)
        self.timeFinished.setEnabled(False)
        self.timeFinished.setGeometry(QtCore.QRect(115, 55, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.timeFinished.setFont(font)
        self.timeFinished.setMouseTracking(False)
        self.timeFinished.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeFinished.setAcceptDrops(False)
        self.timeFinished.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.timeFinished.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.timeFinished.setReadOnly(True)
        self.timeFinished.setObjectName("timeFinished")
        self.modelProgress = QtWidgets.QProgressBar(self.groupBox_8)
        self.modelProgress.setGeometry(QtCore.QRect(5, 135, 356, 26))
        self.modelProgress.setMaximum(100)
        self.modelProgress.setProperty("value", 0)
        self.modelProgress.setAlignment(QtCore.Qt.AlignCenter)
        self.modelProgress.setTextVisible(True)
        self.modelProgress.setOrientation(QtCore.Qt.Horizontal)
        self.modelProgress.setInvertedAppearance(False)
        self.modelProgress.setTextDirection(QtWidgets.QProgressBar.TopToBottom)
        self.modelProgress.setObjectName("modelProgress")
        self.label_312 = QtWidgets.QLabel(self.groupBox_8)
        self.label_312.setGeometry(QtCore.QRect(195, 105, 51, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_312.setFont(font)
        self.label_312.setObjectName("label_312")
        self.timeToFinish = QtWidgets.QLineEdit(self.groupBox_8)
        self.timeToFinish.setEnabled(False)
        self.timeToFinish.setGeometry(QtCore.QRect(300, 55, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.timeToFinish.setFont(font)
        self.timeToFinish.setMouseTracking(False)
        self.timeToFinish.setFocusPolicy(QtCore.Qt.NoFocus)
        self.timeToFinish.setAcceptDrops(False)
        self.timeToFinish.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.timeToFinish.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.timeToFinish.setReadOnly(True)
        self.timeToFinish.setObjectName("timeToFinish")
        self.label_42 = QtWidgets.QLabel(self.groupBox_8)
        self.label_42.setGeometry(QtCore.QRect(5, 55, 91, 21))
        font = QtGui.QFont()
        font.setFamily("MS Shell Dlg 2")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_42.setFont(font)
        self.label_42.setObjectName("label_42")
        self.mainTabWidget.addTab(self.BuildModel, "")
        self.ManageModel = QtWidgets.QWidget()
        self.ManageModel.setObjectName("ManageModel")
        self.label_37 = QtWidgets.QLabel(self.ManageModel)
        self.label_37.setGeometry(QtCore.QRect(480, 105, 81, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_37.setFont(font)
        self.label_37.setObjectName("label_37")
        self.runTargetRMS = QtWidgets.QPushButton(self.ManageModel)
        self.runTargetRMS.setGeometry(QtCore.QRect(475, 75, 96, 26))
        self.runTargetRMS.setObjectName("runTargetRMS")
        self.targetRMS = QtWidgets.QDoubleSpinBox(self.ManageModel)
        self.targetRMS.setGeometry(QtCore.QRect(580, 105, 56, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.targetRMS.setFont(font)
        self.targetRMS.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.targetRMS.setDecimals(1)
        self.targetRMS.setMaximum(100.0)
        self.targetRMS.setSingleStep(0.1)
        self.targetRMS.setProperty("value", 99.0)
        self.targetRMS.setObjectName("targetRMS")
        self.line_18 = QtWidgets.QFrame(self.ManageModel)
        self.line_18.setGeometry(QtCore.QRect(475, 200, 296, 1))
        self.line_18.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_18.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_18.setObjectName("line_18")
        self.deleteWorstPoint = QtWidgets.QPushButton(self.ManageModel)
        self.deleteWorstPoint.setGeometry(QtCore.QRect(475, 45, 96, 26))
        self.deleteWorstPoint.setObjectName("deleteWorstPoint")
        self.line_44 = QtWidgets.QFrame(self.ManageModel)
        self.line_44.setGeometry(QtCore.QRect(475, 40, 301, 1))
        self.line_44.setAutoFillBackground(False)
        self.line_44.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_44.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_44.setObjectName("line_44")
        self.label_124 = QtWidgets.QLabel(self.ManageModel)
        self.label_124.setGeometry(QtCore.QRect(475, 20, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_124.setFont(font)
        self.label_124.setObjectName("label_124")
        self.label_125 = QtWidgets.QLabel(self.ManageModel)
        self.label_125.setGeometry(QtCore.QRect(475, 180, 251, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_125.setFont(font)
        self.label_125.setObjectName("label_125")
        self.line_46 = QtWidgets.QFrame(self.ManageModel)
        self.line_46.setGeometry(QtCore.QRect(455, 10, 11, 416))
        self.line_46.setStyleSheet("")
        self.line_46.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_46.setLineWidth(2)
        self.line_46.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_46.setObjectName("line_46")
        self.modelPolar = QtWidgets.QWidget(self.ManageModel)
        self.modelPolar.setEnabled(True)
        self.modelPolar.setGeometry(QtCore.QRect(5, 5, 446, 426))
        self.modelPolar.setAutoFillBackground(True)
        self.modelPolar.setObjectName("modelPolar")
        self.cancelTargetRMS = QtWidgets.QPushButton(self.ManageModel)
        self.cancelTargetRMS.setGeometry(QtCore.QRect(580, 75, 96, 26))
        self.cancelTargetRMS.setObjectName("cancelTargetRMS")
        self.errorRMS1 = QtWidgets.QLineEdit(self.ManageModel)
        self.errorRMS1.setEnabled(False)
        self.errorRMS1.setGeometry(QtCore.QRect(735, 105, 41, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.errorRMS1.setFont(font)
        self.errorRMS1.setMouseTracking(False)
        self.errorRMS1.setFocusPolicy(QtCore.Qt.NoFocus)
        self.errorRMS1.setAcceptDrops(False)
        self.errorRMS1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.errorRMS1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.errorRMS1.setReadOnly(True)
        self.errorRMS1.setObjectName("errorRMS1")
        self.label_75 = QtWidgets.QLabel(self.ManageModel)
        self.label_75.setGeometry(QtCore.QRect(655, 105, 76, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_75.setFont(font)
        self.label_75.setObjectName("label_75")
        self.clearModel = QtWidgets.QPushButton(self.ManageModel)
        self.clearModel.setGeometry(QtCore.QRect(580, 45, 96, 26))
        self.clearModel.setObjectName("clearModel")
        self.numberStars1 = QtWidgets.QLineEdit(self.ManageModel)
        self.numberStars1.setEnabled(False)
        self.numberStars1.setGeometry(QtCore.QRect(735, 75, 41, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.numberStars1.setFont(font)
        self.numberStars1.setMouseTracking(False)
        self.numberStars1.setFocusPolicy(QtCore.Qt.NoFocus)
        self.numberStars1.setAcceptDrops(False)
        self.numberStars1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.numberStars1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberStars1.setReadOnly(True)
        self.numberStars1.setObjectName("numberStars1")
        self.label_76 = QtWidgets.QLabel(self.ManageModel)
        self.label_76.setGeometry(QtCore.QRect(690, 75, 36, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_76.setFont(font)
        self.label_76.setObjectName("label_76")
        self.refreshModel = QtWidgets.QPushButton(self.ManageModel)
        self.refreshModel.setGeometry(QtCore.QRect(685, 45, 91, 26))
        self.refreshModel.setObjectName("refreshModel")
        self.saveName = QtWidgets.QPushButton(self.ManageModel)
        self.saveName.setGeometry(QtCore.QRect(695, 240, 81, 26))
        self.saveName.setObjectName("saveName")
        self.nameList = QtWidgets.QListWidget(self.ManageModel)
        self.nameList.setGeometry(QtCore.QRect(475, 210, 211, 216))
        self.nameList.setObjectName("nameList")
        self.loadName = QtWidgets.QPushButton(self.ManageModel)
        self.loadName.setGeometry(QtCore.QRect(695, 210, 81, 26))
        self.loadName.setObjectName("loadName")
        self.deleteName = QtWidgets.QPushButton(self.ManageModel)
        self.deleteName.setGeometry(QtCore.QRect(695, 270, 81, 26))
        self.deleteName.setObjectName("deleteName")
        self.checkShowErrorValues = QtWidgets.QCheckBox(self.ManageModel)
        self.checkShowErrorValues.setGeometry(QtCore.QRect(495, 140, 201, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkShowErrorValues.setFont(font)
        self.checkShowErrorValues.setChecked(False)
        self.checkShowErrorValues.setObjectName("checkShowErrorValues")
        self.refreshName = QtWidgets.QPushButton(self.ManageModel)
        self.refreshName.setGeometry(QtCore.QRect(695, 300, 81, 26))
        self.refreshName.setObjectName("refreshName")
        self.line_18.raise_()
        self.line_44.raise_()
        self.label_37.raise_()
        self.runTargetRMS.raise_()
        self.targetRMS.raise_()
        self.deleteWorstPoint.raise_()
        self.label_124.raise_()
        self.label_125.raise_()
        self.line_46.raise_()
        self.modelPolar.raise_()
        self.cancelTargetRMS.raise_()
        self.errorRMS1.raise_()
        self.label_75.raise_()
        self.clearModel.raise_()
        self.numberStars1.raise_()
        self.label_76.raise_()
        self.refreshModel.raise_()
        self.saveName.raise_()
        self.nameList.raise_()
        self.loadName.raise_()
        self.deleteName.raise_()
        self.checkShowErrorValues.raise_()
        self.refreshName.raise_()
        self.mainTabWidget.addTab(self.ManageModel, "")
        self.Satellite = QtWidgets.QWidget()
        self.Satellite.setObjectName("Satellite")
        self.line_20 = QtWidgets.QFrame(self.Satellite)
        self.line_20.setGeometry(QtCore.QRect(20, 100, 356, 1))
        self.line_20.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_20.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_20.setObjectName("line_20")
        self.label_126 = QtWidgets.QLabel(self.Satellite)
        self.label_126.setGeometry(QtCore.QRect(20, 80, 251, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_126.setFont(font)
        self.label_126.setObjectName("label_126")
        self.listSatelliteNames = QtWidgets.QListWidget(self.Satellite)
        self.listSatelliteNames.setGeometry(QtCore.QRect(20, 105, 356, 316))
        self.listSatelliteNames.setObjectName("listSatelliteNames")
        self.line_67 = QtWidgets.QFrame(self.Satellite)
        self.line_67.setGeometry(QtCore.QRect(20, 40, 356, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_67.setFont(font)
        self.line_67.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_67.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_67.setObjectName("line_67")
        self.label_223 = QtWidgets.QLabel(self.Satellite)
        self.label_223.setGeometry(QtCore.QRect(20, 20, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_223.setFont(font)
        self.label_223.setObjectName("label_223")
        self.line_68 = QtWidgets.QFrame(self.Satellite)
        self.line_68.setGeometry(QtCore.QRect(385, 10, 11, 416))
        self.line_68.setStyleSheet("")
        self.line_68.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_68.setLineWidth(2)
        self.line_68.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_68.setObjectName("line_68")
        self.label_224 = QtWidgets.QLabel(self.Satellite)
        self.label_224.setGeometry(QtCore.QRect(405, 20, 276, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_224.setFont(font)
        self.label_224.setObjectName("label_224")
        self.line_69 = QtWidgets.QFrame(self.Satellite)
        self.line_69.setGeometry(QtCore.QRect(405, 40, 371, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_69.setFont(font)
        self.line_69.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_69.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_69.setObjectName("line_69")
        self.satelliteName = QtWidgets.QLineEdit(self.Satellite)
        self.satelliteName.setEnabled(False)
        self.satelliteName.setGeometry(QtCore.QRect(470, 50, 161, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satelliteName.setFont(font)
        self.satelliteName.setMouseTracking(False)
        self.satelliteName.setAcceptDrops(False)
        self.satelliteName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satelliteName.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satelliteName.setReadOnly(False)
        self.satelliteName.setObjectName("satelliteName")
        self.label_225 = QtWidgets.QLabel(self.Satellite)
        self.label_225.setGeometry(QtCore.QRect(405, 50, 41, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_225.setFont(font)
        self.label_225.setObjectName("label_225")
        self.label_230 = QtWidgets.QLabel(self.Satellite)
        self.label_230.setGeometry(QtCore.QRect(405, 75, 36, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_230.setFont(font)
        self.label_230.setObjectName("label_230")
        self.satelliteEpoch = QtWidgets.QLineEdit(self.Satellite)
        self.satelliteEpoch.setEnabled(False)
        self.satelliteEpoch.setGeometry(QtCore.QRect(470, 75, 161, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satelliteEpoch.setFont(font)
        self.satelliteEpoch.setMouseTracking(False)
        self.satelliteEpoch.setAcceptDrops(False)
        self.satelliteEpoch.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satelliteEpoch.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satelliteEpoch.setReadOnly(False)
        self.satelliteEpoch.setObjectName("satelliteEpoch")
        self.startSatelliteTracking = QtWidgets.QPushButton(self.Satellite)
        self.startSatelliteTracking.setEnabled(False)
        self.startSatelliteTracking.setGeometry(QtCore.QRect(405, 395, 181, 26))
        self.startSatelliteTracking.setStyleSheet("")
        self.startSatelliteTracking.setObjectName("startSatelliteTracking")
        self.satelliteSource = QtWidgets.QComboBox(self.Satellite)
        self.satelliteSource.setGeometry(QtCore.QRect(20, 45, 356, 26))
        self.satelliteSource.setMinimumSize(QtCore.QSize(0, 25))
        self.satelliteSource.setObjectName("satelliteSource")
        self.label_231 = QtWidgets.QLabel(self.Satellite)
        self.label_231.setGeometry(QtCore.QRect(645, 75, 26, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_231.setFont(font)
        self.label_231.setObjectName("label_231")
        self.satelliteDataAge = QtWidgets.QLineEdit(self.Satellite)
        self.satelliteDataAge.setEnabled(False)
        self.satelliteDataAge.setGeometry(QtCore.QRect(695, 75, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satelliteDataAge.setFont(font)
        self.satelliteDataAge.setMouseTracking(False)
        self.satelliteDataAge.setAcceptDrops(False)
        self.satelliteDataAge.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satelliteDataAge.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satelliteDataAge.setReadOnly(False)
        self.satelliteDataAge.setObjectName("satelliteDataAge")
        self.label_233 = QtWidgets.QLabel(self.Satellite)
        self.label_233.setGeometry(QtCore.QRect(745, 75, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_233.setFont(font)
        self.label_233.setObjectName("label_233")
        self.label_234 = QtWidgets.QLabel(self.Satellite)
        self.label_234.setGeometry(QtCore.QRect(405, 105, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_234.setFont(font)
        self.label_234.setObjectName("label_234")
        self.label_235 = QtWidgets.QLabel(self.Satellite)
        self.label_235.setGeometry(QtCore.QRect(405, 130, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_235.setFont(font)
        self.label_235.setObjectName("label_235")
        self.label_236 = QtWidgets.QLabel(self.Satellite)
        self.label_236.setGeometry(QtCore.QRect(405, 160, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_236.setFont(font)
        self.label_236.setObjectName("label_236")
        self.label_237 = QtWidgets.QLabel(self.Satellite)
        self.label_237.setGeometry(QtCore.QRect(405, 185, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_237.setFont(font)
        self.label_237.setObjectName("label_237")
        self.satAzimuth = QtWidgets.QLineEdit(self.Satellite)
        self.satAzimuth.setEnabled(False)
        self.satAzimuth.setGeometry(QtCore.QRect(470, 105, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satAzimuth.setFont(font)
        self.satAzimuth.setMouseTracking(False)
        self.satAzimuth.setAcceptDrops(False)
        self.satAzimuth.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satAzimuth.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satAzimuth.setReadOnly(False)
        self.satAzimuth.setObjectName("satAzimuth")
        self.satAltitude = QtWidgets.QLineEdit(self.Satellite)
        self.satAltitude.setEnabled(False)
        self.satAltitude.setGeometry(QtCore.QRect(470, 130, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satAltitude.setFont(font)
        self.satAltitude.setMouseTracking(False)
        self.satAltitude.setAcceptDrops(False)
        self.satAltitude.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satAltitude.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satAltitude.setReadOnly(False)
        self.satAltitude.setObjectName("satAltitude")
        self.satRa = QtWidgets.QLineEdit(self.Satellite)
        self.satRa.setEnabled(False)
        self.satRa.setGeometry(QtCore.QRect(470, 160, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satRa.setFont(font)
        self.satRa.setMouseTracking(False)
        self.satRa.setAcceptDrops(False)
        self.satRa.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satRa.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satRa.setReadOnly(False)
        self.satRa.setObjectName("satRa")
        self.satDec = QtWidgets.QLineEdit(self.Satellite)
        self.satDec.setEnabled(False)
        self.satDec.setGeometry(QtCore.QRect(470, 185, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satDec.setFont(font)
        self.satDec.setMouseTracking(False)
        self.satDec.setAcceptDrops(False)
        self.satDec.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satDec.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satDec.setReadOnly(False)
        self.satDec.setObjectName("satDec")
        self.label_202 = QtWidgets.QLabel(self.Satellite)
        self.label_202.setGeometry(QtCore.QRect(570, 105, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_202.setFont(font)
        self.label_202.setAlignment(QtCore.Qt.AlignCenter)
        self.label_202.setWordWrap(False)
        self.label_202.setObjectName("label_202")
        self.label_207 = QtWidgets.QLabel(self.Satellite)
        self.label_207.setGeometry(QtCore.QRect(570, 130, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_207.setFont(font)
        self.label_207.setAlignment(QtCore.Qt.AlignCenter)
        self.label_207.setWordWrap(False)
        self.label_207.setObjectName("label_207")
        self.label_221 = QtWidgets.QLabel(self.Satellite)
        self.label_221.setGeometry(QtCore.QRect(570, 160, 36, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_221.setFont(font)
        self.label_221.setAlignment(QtCore.Qt.AlignCenter)
        self.label_221.setWordWrap(False)
        self.label_221.setObjectName("label_221")
        self.label_226 = QtWidgets.QLabel(self.Satellite)
        self.label_226.setGeometry(QtCore.QRect(570, 185, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_226.setFont(font)
        self.label_226.setAlignment(QtCore.Qt.AlignCenter)
        self.label_226.setWordWrap(False)
        self.label_226.setObjectName("label_226")
        self.label_227 = QtWidgets.QLabel(self.Satellite)
        self.label_227.setGeometry(QtCore.QRect(645, 50, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_227.setFont(font)
        self.label_227.setObjectName("label_227")
        self.satelliteNumber = QtWidgets.QLineEdit(self.Satellite)
        self.satelliteNumber.setEnabled(False)
        self.satelliteNumber.setGeometry(QtCore.QRect(695, 50, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satelliteNumber.setFont(font)
        self.satelliteNumber.setMouseTracking(False)
        self.satelliteNumber.setAcceptDrops(False)
        self.satelliteNumber.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satelliteNumber.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satelliteNumber.setReadOnly(False)
        self.satelliteNumber.setObjectName("satelliteNumber")
        self.label_239 = QtWidgets.QLabel(self.Satellite)
        self.label_239.setGeometry(QtCore.QRect(405, 215, 66, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_239.setFont(font)
        self.label_239.setObjectName("label_239")
        self.label_240 = QtWidgets.QLabel(self.Satellite)
        self.label_240.setGeometry(QtCore.QRect(405, 240, 66, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_240.setFont(font)
        self.label_240.setObjectName("label_240")
        self.label_252 = QtWidgets.QLabel(self.Satellite)
        self.label_252.setGeometry(QtCore.QRect(570, 240, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_252.setFont(font)
        self.label_252.setAlignment(QtCore.Qt.AlignCenter)
        self.label_252.setWordWrap(False)
        self.label_252.setObjectName("label_252")
        self.satLongitude = QtWidgets.QLineEdit(self.Satellite)
        self.satLongitude.setEnabled(False)
        self.satLongitude.setGeometry(QtCore.QRect(470, 240, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satLongitude.setFont(font)
        self.satLongitude.setMouseTracking(False)
        self.satLongitude.setAcceptDrops(False)
        self.satLongitude.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satLongitude.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satLongitude.setReadOnly(False)
        self.satLongitude.setObjectName("satLongitude")
        self.satLatitude = QtWidgets.QLineEdit(self.Satellite)
        self.satLatitude.setEnabled(False)
        self.satLatitude.setGeometry(QtCore.QRect(470, 215, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satLatitude.setFont(font)
        self.satLatitude.setMouseTracking(False)
        self.satLatitude.setAcceptDrops(False)
        self.satLatitude.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satLatitude.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satLatitude.setReadOnly(False)
        self.satLatitude.setObjectName("satLatitude")
        self.label_260 = QtWidgets.QLabel(self.Satellite)
        self.label_260.setGeometry(QtCore.QRect(405, 270, 61, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_260.setFont(font)
        self.label_260.setObjectName("label_260")
        self.label_270 = QtWidgets.QLabel(self.Satellite)
        self.label_270.setGeometry(QtCore.QRect(405, 295, 61, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_270.setFont(font)
        self.label_270.setObjectName("label_270")
        self.label_271 = QtWidgets.QLabel(self.Satellite)
        self.label_271.setGeometry(QtCore.QRect(405, 320, 61, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_271.setFont(font)
        self.label_271.setObjectName("label_271")
        self.satNeedFlip = QtWidgets.QLineEdit(self.Satellite)
        self.satNeedFlip.setEnabled(False)
        self.satNeedFlip.setGeometry(QtCore.QRect(470, 320, 101, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satNeedFlip.setFont(font)
        self.satNeedFlip.setMouseTracking(False)
        self.satNeedFlip.setAcceptDrops(False)
        self.satNeedFlip.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satNeedFlip.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satNeedFlip.setReadOnly(False)
        self.satNeedFlip.setObjectName("satNeedFlip")
        self.satTransitStartUTC = QtWidgets.QLineEdit(self.Satellite)
        self.satTransitStartUTC.setEnabled(False)
        self.satTransitStartUTC.setGeometry(QtCore.QRect(470, 270, 151, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satTransitStartUTC.setFont(font)
        self.satTransitStartUTC.setMouseTracking(False)
        self.satTransitStartUTC.setAcceptDrops(False)
        self.satTransitStartUTC.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satTransitStartUTC.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satTransitStartUTC.setReadOnly(False)
        self.satTransitStartUTC.setObjectName("satTransitStartUTC")
        self.satTransitEndUTC = QtWidgets.QLineEdit(self.Satellite)
        self.satTransitEndUTC.setEnabled(False)
        self.satTransitEndUTC.setGeometry(QtCore.QRect(470, 295, 151, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.satTransitEndUTC.setFont(font)
        self.satTransitEndUTC.setMouseTracking(False)
        self.satTransitEndUTC.setAcceptDrops(False)
        self.satTransitEndUTC.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.satTransitEndUTC.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.satTransitEndUTC.setReadOnly(False)
        self.satTransitEndUTC.setObjectName("satTransitEndUTC")
        self.label_272 = QtWidgets.QLabel(self.Satellite)
        self.label_272.setGeometry(QtCore.QRect(625, 270, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_272.setFont(font)
        self.label_272.setObjectName("label_272")
        self.label_273 = QtWidgets.QLabel(self.Satellite)
        self.label_273.setGeometry(QtCore.QRect(625, 295, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_273.setFont(font)
        self.label_273.setObjectName("label_273")
        self.stopSatelliteTracking = QtWidgets.QPushButton(self.Satellite)
        self.stopSatelliteTracking.setEnabled(False)
        self.stopSatelliteTracking.setGeometry(QtCore.QRect(595, 395, 181, 26))
        self.stopSatelliteTracking.setStyleSheet("")
        self.stopSatelliteTracking.setObjectName("stopSatelliteTracking")
        self.label_253 = QtWidgets.QLabel(self.Satellite)
        self.label_253.setGeometry(QtCore.QRect(570, 215, 16, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_253.setFont(font)
        self.label_253.setAlignment(QtCore.Qt.AlignCenter)
        self.label_253.setWordWrap(False)
        self.label_253.setObjectName("label_253")
        self.mainTabWidget.addTab(self.Satellite, "")
        self.Analyse = QtWidgets.QWidget()
        self.Analyse.setObjectName("Analyse")
        self.cancelAnalyse = QtWidgets.QPushButton(self.Analyse)
        self.cancelAnalyse.setEnabled(False)
        self.cancelAnalyse.setGeometry(QtCore.QRect(10, 390, 366, 31))
        self.cancelAnalyse.setObjectName("cancelAnalyse")
        self.line_52 = QtWidgets.QFrame(self.Analyse)
        self.line_52.setGeometry(QtCore.QRect(385, 10, 11, 416))
        self.line_52.setStyleSheet("")
        self.line_52.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_52.setLineWidth(2)
        self.line_52.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_52.setObjectName("line_52")
        self.groupBox_14 = QtWidgets.QGroupBox(self.Analyse)
        self.groupBox_14.setGeometry(QtCore.QRect(10, 300, 366, 76))
        self.groupBox_14.setProperty("large", True)
        self.groupBox_14.setObjectName("groupBox_14")
        self.le_analyseFileName = QtWidgets.QLineEdit(self.groupBox_14)
        self.le_analyseFileName.setEnabled(False)
        self.le_analyseFileName.setGeometry(QtCore.QRect(5, 20, 356, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.le_analyseFileName.setFont(font)
        self.le_analyseFileName.setMouseTracking(False)
        self.le_analyseFileName.setAcceptDrops(False)
        self.le_analyseFileName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_analyseFileName.setText("")
        self.le_analyseFileName.setReadOnly(True)
        self.le_analyseFileName.setObjectName("le_analyseFileName")
        self.loadAnalyseData = QtWidgets.QPushButton(self.groupBox_14)
        self.loadAnalyseData.setGeometry(QtCore.QRect(5, 45, 71, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.loadAnalyseData.setFont(font)
        self.loadAnalyseData.setStyleSheet("")
        self.loadAnalyseData.setObjectName("loadAnalyseData")
        self.groupBox_15 = QtWidgets.QGroupBox(self.Analyse)
        self.groupBox_15.setGeometry(QtCore.QRect(10, 25, 366, 101))
        self.groupBox_15.setProperty("large", True)
        self.groupBox_15.setObjectName("groupBox_15")
        self.label_177 = QtWidgets.QLabel(self.groupBox_15)
        self.label_177.setGeometry(QtCore.QRect(210, 45, 41, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_177.setFont(font)
        self.label_177.setObjectName("label_177")
        self.delayTimeFlexure = QtWidgets.QDoubleSpinBox(self.groupBox_15)
        self.delayTimeFlexure.setGeometry(QtCore.QRect(280, 45, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.delayTimeFlexure.setFont(font)
        self.delayTimeFlexure.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.delayTimeFlexure.setDecimals(0)
        self.delayTimeFlexure.setMinimum(1.0)
        self.delayTimeFlexure.setMaximum(300.0)
        self.delayTimeFlexure.setSingleStep(1.0)
        self.delayTimeFlexure.setProperty("value", 30.0)
        self.delayTimeFlexure.setObjectName("delayTimeFlexure")
        self.label_111 = QtWidgets.QLabel(self.groupBox_15)
        self.label_111.setGeometry(QtCore.QRect(330, 20, 21, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_111.setFont(font)
        self.label_111.setAlignment(QtCore.Qt.AlignCenter)
        self.label_111.setWordWrap(False)
        self.label_111.setObjectName("label_111")
        self.label_174 = QtWidgets.QLabel(self.groupBox_15)
        self.label_174.setGeometry(QtCore.QRect(10, 20, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_174.setFont(font)
        self.label_174.setObjectName("label_174")
        self.label_112 = QtWidgets.QLabel(self.groupBox_15)
        self.label_112.setGeometry(QtCore.QRect(165, 20, 21, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_112.setFont(font)
        self.label_112.setAlignment(QtCore.Qt.AlignCenter)
        self.label_112.setWordWrap(False)
        self.label_112.setObjectName("label_112")
        self.numberRunsTimeChange = QtWidgets.QDoubleSpinBox(self.groupBox_15)
        self.numberRunsTimeChange.setGeometry(QtCore.QRect(115, 45, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.numberRunsTimeChange.setFont(font)
        self.numberRunsTimeChange.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberRunsTimeChange.setDecimals(0)
        self.numberRunsTimeChange.setMinimum(10.0)
        self.numberRunsTimeChange.setMaximum(300.0)
        self.numberRunsTimeChange.setSingleStep(1.0)
        self.numberRunsTimeChange.setProperty("value", 30.0)
        self.numberRunsTimeChange.setObjectName("numberRunsTimeChange")
        self.altitudeTimeChange = QtWidgets.QDoubleSpinBox(self.groupBox_15)
        self.altitudeTimeChange.setGeometry(QtCore.QRect(115, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altitudeTimeChange.setFont(font)
        self.altitudeTimeChange.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeTimeChange.setDecimals(0)
        self.altitudeTimeChange.setMinimum(0.0)
        self.altitudeTimeChange.setMaximum(90.0)
        self.altitudeTimeChange.setSingleStep(1.0)
        self.altitudeTimeChange.setProperty("value", 45.0)
        self.altitudeTimeChange.setObjectName("altitudeTimeChange")
        self.label_176 = QtWidgets.QLabel(self.groupBox_15)
        self.label_176.setGeometry(QtCore.QRect(10, 45, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_176.setFont(font)
        self.label_176.setObjectName("label_176")
        self.azimuthTimeChange = QtWidgets.QDoubleSpinBox(self.groupBox_15)
        self.azimuthTimeChange.setGeometry(QtCore.QRect(280, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.azimuthTimeChange.setFont(font)
        self.azimuthTimeChange.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azimuthTimeChange.setDecimals(0)
        self.azimuthTimeChange.setMaximum(359.0)
        self.azimuthTimeChange.setSingleStep(1.0)
        self.azimuthTimeChange.setProperty("value", 0.0)
        self.azimuthTimeChange.setObjectName("azimuthTimeChange")
        self.label_115 = QtWidgets.QLabel(self.groupBox_15)
        self.label_115.setGeometry(QtCore.QRect(330, 45, 21, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_115.setFont(font)
        self.label_115.setAlignment(QtCore.Qt.AlignCenter)
        self.label_115.setWordWrap(False)
        self.label_115.setObjectName("label_115")
        self.label_175 = QtWidgets.QLabel(self.groupBox_15)
        self.label_175.setGeometry(QtCore.QRect(210, 20, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_175.setFont(font)
        self.label_175.setObjectName("label_175")
        self.runFlexure = QtWidgets.QPushButton(self.groupBox_15)
        self.runFlexure.setEnabled(True)
        self.runFlexure.setGeometry(QtCore.QRect(5, 70, 356, 26))
        self.runFlexure.setObjectName("runFlexure")
        self.groupBox_16 = QtWidgets.QGroupBox(self.Analyse)
        self.groupBox_16.setGeometry(QtCore.QRect(10, 150, 366, 126))
        self.groupBox_16.setProperty("large", True)
        self.groupBox_16.setObjectName("groupBox_16")
        self.label_182 = QtWidgets.QLabel(self.groupBox_16)
        self.label_182.setGeometry(QtCore.QRect(220, 45, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_182.setFont(font)
        self.label_182.setObjectName("label_182")
        self.label_183 = QtWidgets.QLabel(self.groupBox_16)
        self.label_183.setGeometry(QtCore.QRect(10, 70, 91, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_183.setFont(font)
        self.label_183.setObjectName("label_183")
        self.delayTimeHysterese = QtWidgets.QDoubleSpinBox(self.groupBox_16)
        self.delayTimeHysterese.setGeometry(QtCore.QRect(290, 70, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.delayTimeHysterese.setFont(font)
        self.delayTimeHysterese.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.delayTimeHysterese.setDecimals(0)
        self.delayTimeHysterese.setMinimum(1.0)
        self.delayTimeHysterese.setMaximum(300.0)
        self.delayTimeHysterese.setSingleStep(1.0)
        self.delayTimeHysterese.setProperty("value", 30.0)
        self.delayTimeHysterese.setObjectName("delayTimeHysterese")
        self.altitudeHysterese2 = QtWidgets.QDoubleSpinBox(self.groupBox_16)
        self.altitudeHysterese2.setGeometry(QtCore.QRect(130, 45, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altitudeHysterese2.setFont(font)
        self.altitudeHysterese2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeHysterese2.setDecimals(0)
        self.altitudeHysterese2.setMinimum(0.0)
        self.altitudeHysterese2.setMaximum(90.0)
        self.altitudeHysterese2.setSingleStep(1.0)
        self.altitudeHysterese2.setProperty("value", 45.0)
        self.altitudeHysterese2.setObjectName("altitudeHysterese2")
        self.label_119 = QtWidgets.QLabel(self.groupBox_16)
        self.label_119.setGeometry(QtCore.QRect(340, 45, 21, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_119.setFont(font)
        self.label_119.setAlignment(QtCore.Qt.AlignCenter)
        self.label_119.setWordWrap(False)
        self.label_119.setObjectName("label_119")
        self.altitudeHysterese1 = QtWidgets.QDoubleSpinBox(self.groupBox_16)
        self.altitudeHysterese1.setGeometry(QtCore.QRect(130, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.altitudeHysterese1.setFont(font)
        self.altitudeHysterese1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitudeHysterese1.setDecimals(0)
        self.altitudeHysterese1.setMinimum(0.0)
        self.altitudeHysterese1.setMaximum(90.0)
        self.altitudeHysterese1.setSingleStep(1.0)
        self.altitudeHysterese1.setProperty("value", 45.0)
        self.altitudeHysterese1.setObjectName("altitudeHysterese1")
        self.label_120 = QtWidgets.QLabel(self.groupBox_16)
        self.label_120.setGeometry(QtCore.QRect(180, 20, 21, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_120.setFont(font)
        self.label_120.setAlignment(QtCore.Qt.AlignCenter)
        self.label_120.setWordWrap(False)
        self.label_120.setObjectName("label_120")
        self.label_185 = QtWidgets.QLabel(self.groupBox_16)
        self.label_185.setGeometry(QtCore.QRect(10, 20, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_185.setFont(font)
        self.label_185.setObjectName("label_185")
        self.numberRunsHysterese = QtWidgets.QDoubleSpinBox(self.groupBox_16)
        self.numberRunsHysterese.setGeometry(QtCore.QRect(130, 70, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.numberRunsHysterese.setFont(font)
        self.numberRunsHysterese.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberRunsHysterese.setDecimals(0)
        self.numberRunsHysterese.setMinimum(1.0)
        self.numberRunsHysterese.setMaximum(50.0)
        self.numberRunsHysterese.setSingleStep(1.0)
        self.numberRunsHysterese.setProperty("value", 5.0)
        self.numberRunsHysterese.setObjectName("numberRunsHysterese")
        self.label_187 = QtWidgets.QLabel(self.groupBox_16)
        self.label_187.setGeometry(QtCore.QRect(10, 45, 101, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_187.setFont(font)
        self.label_187.setObjectName("label_187")
        self.label_184 = QtWidgets.QLabel(self.groupBox_16)
        self.label_184.setGeometry(QtCore.QRect(220, 70, 41, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_184.setFont(font)
        self.label_184.setObjectName("label_184")
        self.azimuthHysterese2 = QtWidgets.QDoubleSpinBox(self.groupBox_16)
        self.azimuthHysterese2.setGeometry(QtCore.QRect(290, 45, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.azimuthHysterese2.setFont(font)
        self.azimuthHysterese2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azimuthHysterese2.setDecimals(0)
        self.azimuthHysterese2.setMaximum(359.0)
        self.azimuthHysterese2.setSingleStep(1.0)
        self.azimuthHysterese2.setProperty("value", 0.0)
        self.azimuthHysterese2.setObjectName("azimuthHysterese2")
        self.label_114 = QtWidgets.QLabel(self.groupBox_16)
        self.label_114.setGeometry(QtCore.QRect(180, 55, 21, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_114.setFont(font)
        self.label_114.setAlignment(QtCore.Qt.AlignCenter)
        self.label_114.setWordWrap(False)
        self.label_114.setObjectName("label_114")
        self.label_117 = QtWidgets.QLabel(self.groupBox_16)
        self.label_117.setGeometry(QtCore.QRect(340, 70, 21, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_117.setFont(font)
        self.label_117.setAlignment(QtCore.Qt.AlignCenter)
        self.label_117.setWordWrap(False)
        self.label_117.setObjectName("label_117")
        self.label_186 = QtWidgets.QLabel(self.groupBox_16)
        self.label_186.setGeometry(QtCore.QRect(220, 20, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_186.setFont(font)
        self.label_186.setObjectName("label_186")
        self.azimuthHysterese1 = QtWidgets.QDoubleSpinBox(self.groupBox_16)
        self.azimuthHysterese1.setGeometry(QtCore.QRect(290, 20, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.azimuthHysterese1.setFont(font)
        self.azimuthHysterese1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azimuthHysterese1.setDecimals(0)
        self.azimuthHysterese1.setMaximum(359.0)
        self.azimuthHysterese1.setSingleStep(1.0)
        self.azimuthHysterese1.setProperty("value", 0.0)
        self.azimuthHysterese1.setObjectName("azimuthHysterese1")
        self.label_121 = QtWidgets.QLabel(self.groupBox_16)
        self.label_121.setGeometry(QtCore.QRect(340, 20, 21, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_121.setFont(font)
        self.label_121.setAlignment(QtCore.Qt.AlignCenter)
        self.label_121.setWordWrap(False)
        self.label_121.setObjectName("label_121")
        self.runHysteresis = QtWidgets.QPushButton(self.groupBox_16)
        self.runHysteresis.setEnabled(True)
        self.runHysteresis.setGeometry(QtCore.QRect(5, 95, 356, 26))
        self.runHysteresis.setObjectName("runHysteresis")
        self.line_52.raise_()
        self.cancelAnalyse.raise_()
        self.groupBox_14.raise_()
        self.groupBox_15.raise_()
        self.groupBox_16.raise_()
        self.mainTabWidget.addTab(self.Analyse, "")
        self.Relay = QtWidgets.QWidget()
        self.Relay.setObjectName("Relay")
        self.horizontalLayoutWidget = QtWidgets.QWidget(self.Relay)
        self.horizontalLayoutWidget.setGeometry(QtCore.QRect(10, 5, 766, 421))
        self.horizontalLayoutWidget.setObjectName("horizontalLayoutWidget")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.horizontalLayoutWidget)
        self.horizontalLayout_2.setContentsMargins(0, 15, 4, 4)
        self.horizontalLayout_2.setSpacing(4)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSpacing(6)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_96 = QtWidgets.QLabel(self.horizontalLayoutWidget)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_96.setFont(font)
        self.label_96.setObjectName("label_96")
        self.verticalLayout_2.addWidget(self.label_96)
        self.line_31 = QtWidgets.QFrame(self.horizontalLayoutWidget)
        self.line_31.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_31.setMidLineWidth(1)
        self.line_31.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_31.setObjectName("line_31")
        self.verticalLayout_2.addWidget(self.line_31)
        self.relayButton0 = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.relayButton0.setMinimumSize(QtCore.QSize(0, 25))
        self.relayButton0.setObjectName("relayButton0")
        self.verticalLayout_2.addWidget(self.relayButton0)
        self.relayButton1 = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.relayButton1.setMinimumSize(QtCore.QSize(0, 25))
        self.relayButton1.setObjectName("relayButton1")
        self.verticalLayout_2.addWidget(self.relayButton1)
        self.relayButton2 = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.relayButton2.setMinimumSize(QtCore.QSize(0, 25))
        self.relayButton2.setObjectName("relayButton2")
        self.verticalLayout_2.addWidget(self.relayButton2)
        self.relayButton3 = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.relayButton3.setMinimumSize(QtCore.QSize(0, 25))
        self.relayButton3.setObjectName("relayButton3")
        self.verticalLayout_2.addWidget(self.relayButton3)
        self.relayButton4 = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.relayButton4.setMinimumSize(QtCore.QSize(0, 25))
        self.relayButton4.setObjectName("relayButton4")
        self.verticalLayout_2.addWidget(self.relayButton4)
        self.relayButton5 = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.relayButton5.setMinimumSize(QtCore.QSize(0, 25))
        self.relayButton5.setObjectName("relayButton5")
        self.verticalLayout_2.addWidget(self.relayButton5)
        self.relayButton6 = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.relayButton6.setMinimumSize(QtCore.QSize(0, 25))
        self.relayButton6.setObjectName("relayButton6")
        self.verticalLayout_2.addWidget(self.relayButton6)
        self.relayButton7 = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.relayButton7.setMinimumSize(QtCore.QSize(0, 25))
        self.relayButton7.setObjectName("relayButton7")
        self.verticalLayout_2.addWidget(self.relayButton7)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem1)
        self.verticalLayout_2.setStretch(9, 1)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.line_59 = QtWidgets.QFrame(self.horizontalLayoutWidget)
        self.line_59.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_59.setMidLineWidth(1)
        self.line_59.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_59.setObjectName("line_59")
        self.horizontalLayout_2.addWidget(self.line_59)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem2)
        self.line_73 = QtWidgets.QFrame(self.horizontalLayoutWidget)
        self.line_73.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_73.setMidLineWidth(1)
        self.line_73.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_73.setObjectName("line_73")
        self.horizontalLayout_2.addWidget(self.line_73)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.horizontalLayout_2.setStretch(0, 1)
        self.horizontalLayout_2.setStretch(1, 1)
        self.horizontalLayout_2.setStretch(2, 1)
        self.horizontalLayout_2.setStretch(3, 1)
        self.horizontalLayout_2.setStretch(4, 1)
        self.mainTabWidget.addTab(self.Relay, "")
        self.Power = QtWidgets.QWidget()
        self.Power.setObjectName("Power")
        self.groupBox_13 = QtWidgets.QGroupBox(self.Power)
        self.groupBox_13.setGeometry(QtCore.QRect(410, 290, 366, 71))
        self.groupBox_13.setProperty("large", True)
        self.groupBox_13.setObjectName("groupBox_13")
        self.label_283 = QtWidgets.QLabel(self.groupBox_13)
        self.label_283.setGeometry(QtCore.QRect(119, 45, 16, 21))
        self.label_283.setObjectName("label_283")
        self.label_292 = QtWidgets.QLabel(self.groupBox_13)
        self.label_292.setGeometry(QtCore.QRect(11, 45, 46, 21))
        self.label_292.setObjectName("label_292")
        self.powerPort4 = QtWidgets.QCheckBox(self.groupBox_13)
        self.powerPort4.setGeometry(QtCore.QRect(220, 20, 109, 21))
        self.powerPort4.setObjectName("powerPort4")
        self.powerCurrent4 = QtWidgets.QLineEdit(self.groupBox_13)
        self.powerCurrent4.setEnabled(False)
        self.powerCurrent4.setGeometry(QtCore.QRect(59, 45, 56, 21))
        self.powerCurrent4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerCurrent4.setObjectName("powerCurrent4")
        self.powerBootPort4 = QtWidgets.QCheckBox(self.groupBox_13)
        self.powerBootPort4.setGeometry(QtCore.QRect(220, 45, 106, 21))
        self.powerBootPort4.setMinimumSize(QtCore.QSize(0, 20))
        self.powerBootPort4.setObjectName("powerBootPort4")
        self.powerLabel4 = QtWidgets.QLabel(self.groupBox_13)
        self.powerLabel4.setGeometry(QtCore.QRect(10, 20, 91, 21))
        self.powerLabel4.setObjectName("powerLabel4")
        self.groupBox_17 = QtWidgets.QGroupBox(self.Power)
        self.groupBox_17.setGeometry(QtCore.QRect(410, 210, 366, 71))
        self.groupBox_17.setProperty("large", True)
        self.groupBox_17.setObjectName("groupBox_17")
        self.label_317 = QtWidgets.QLabel(self.groupBox_17)
        self.label_317.setGeometry(QtCore.QRect(119, 45, 16, 21))
        self.label_317.setObjectName("label_317")
        self.label_318 = QtWidgets.QLabel(self.groupBox_17)
        self.label_318.setGeometry(QtCore.QRect(11, 45, 46, 21))
        self.label_318.setObjectName("label_318")
        self.powerLabel3 = QtWidgets.QLabel(self.groupBox_17)
        self.powerLabel3.setGeometry(QtCore.QRect(10, 20, 91, 21))
        self.powerLabel3.setObjectName("powerLabel3")
        self.powerCurrent3 = QtWidgets.QLineEdit(self.groupBox_17)
        self.powerCurrent3.setEnabled(False)
        self.powerCurrent3.setGeometry(QtCore.QRect(60, 45, 56, 21))
        self.powerCurrent3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerCurrent3.setObjectName("powerCurrent3")
        self.powerBootPort3 = QtWidgets.QCheckBox(self.groupBox_17)
        self.powerBootPort3.setGeometry(QtCore.QRect(220, 45, 106, 21))
        self.powerBootPort3.setMinimumSize(QtCore.QSize(0, 20))
        self.powerBootPort3.setObjectName("powerBootPort3")
        self.powerPort3 = QtWidgets.QCheckBox(self.groupBox_17)
        self.powerPort3.setGeometry(QtCore.QRect(220, 20, 109, 21))
        self.powerPort3.setObjectName("powerPort3")
        self.groupBox_18 = QtWidgets.QGroupBox(self.Power)
        self.groupBox_18.setGeometry(QtCore.QRect(410, 130, 366, 71))
        self.groupBox_18.setProperty("large", True)
        self.groupBox_18.setObjectName("groupBox_18")
        self.label_319 = QtWidgets.QLabel(self.groupBox_18)
        self.label_319.setGeometry(QtCore.QRect(119, 45, 16, 21))
        self.label_319.setObjectName("label_319")
        self.label_320 = QtWidgets.QLabel(self.groupBox_18)
        self.label_320.setGeometry(QtCore.QRect(11, 45, 46, 21))
        self.label_320.setObjectName("label_320")
        self.powerCurrent2 = QtWidgets.QLineEdit(self.groupBox_18)
        self.powerCurrent2.setEnabled(False)
        self.powerCurrent2.setGeometry(QtCore.QRect(60, 45, 56, 21))
        self.powerCurrent2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerCurrent2.setObjectName("powerCurrent2")
        self.powerLabel2 = QtWidgets.QLabel(self.groupBox_18)
        self.powerLabel2.setGeometry(QtCore.QRect(10, 20, 91, 21))
        self.powerLabel2.setObjectName("powerLabel2")
        self.powerPort2 = QtWidgets.QCheckBox(self.groupBox_18)
        self.powerPort2.setGeometry(QtCore.QRect(220, 20, 109, 21))
        self.powerPort2.setObjectName("powerPort2")
        self.powerBootPort2 = QtWidgets.QCheckBox(self.groupBox_18)
        self.powerBootPort2.setGeometry(QtCore.QRect(220, 45, 106, 21))
        self.powerBootPort2.setMinimumSize(QtCore.QSize(0, 20))
        self.powerBootPort2.setObjectName("powerBootPort2")
        self.groupBox_19 = QtWidgets.QGroupBox(self.Power)
        self.groupBox_19.setGeometry(QtCore.QRect(410, 50, 366, 71))
        self.groupBox_19.setProperty("large", True)
        self.groupBox_19.setObjectName("groupBox_19")
        self.label_321 = QtWidgets.QLabel(self.groupBox_19)
        self.label_321.setGeometry(QtCore.QRect(119, 45, 16, 21))
        self.label_321.setObjectName("label_321")
        self.label_322 = QtWidgets.QLabel(self.groupBox_19)
        self.label_322.setGeometry(QtCore.QRect(11, 45, 46, 21))
        self.label_322.setObjectName("label_322")
        self.powerCurrent1 = QtWidgets.QLineEdit(self.groupBox_19)
        self.powerCurrent1.setEnabled(False)
        self.powerCurrent1.setGeometry(QtCore.QRect(60, 45, 56, 21))
        self.powerCurrent1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerCurrent1.setObjectName("powerCurrent1")
        self.powerLabel1 = QtWidgets.QLabel(self.groupBox_19)
        self.powerLabel1.setGeometry(QtCore.QRect(10, 20, 91, 21))
        self.powerLabel1.setObjectName("powerLabel1")
        self.powerBootPort1 = QtWidgets.QCheckBox(self.groupBox_19)
        self.powerBootPort1.setGeometry(QtCore.QRect(215, 45, 106, 21))
        self.powerBootPort1.setMinimumSize(QtCore.QSize(0, 20))
        self.powerBootPort1.setObjectName("powerBootPort1")
        self.powerPort1 = QtWidgets.QCheckBox(self.groupBox_19)
        self.powerPort1.setGeometry(QtCore.QRect(215, 20, 109, 21))
        self.powerPort1.setObjectName("powerPort1")
        self.line_41 = QtWidgets.QFrame(self.Power)
        self.line_41.setGeometry(QtCore.QRect(390, 10, 11, 416))
        self.line_41.setStyleSheet("")
        self.line_41.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_41.setLineWidth(2)
        self.line_41.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_41.setObjectName("line_41")
        self.label_229 = QtWidgets.QLabel(self.Power)
        self.label_229.setGeometry(QtCore.QRect(410, 20, 276, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_229.setFont(font)
        self.label_229.setObjectName("label_229")
        self.line_70 = QtWidgets.QFrame(self.Power)
        self.line_70.setGeometry(QtCore.QRect(410, 40, 371, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_70.setFont(font)
        self.line_70.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_70.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_70.setObjectName("line_70")
        self.groupBox_20 = QtWidgets.QGroupBox(self.Power)
        self.groupBox_20.setGeometry(QtCore.QRect(410, 375, 366, 46))
        self.groupBox_20.setProperty("large", True)
        self.groupBox_20.setObjectName("groupBox_20")
        self.hubUSB = QtWidgets.QCheckBox(self.groupBox_20)
        self.hubUSB.setGeometry(QtCore.QRect(10, 20, 127, 20))
        self.hubUSB.setObjectName("hubUSB")
        self.line_72 = QtWidgets.QFrame(self.Power)
        self.line_72.setGeometry(QtCore.QRect(15, 40, 366, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_72.setFont(font)
        self.line_72.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_72.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_72.setObjectName("line_72")
        self.label_238 = QtWidgets.QLabel(self.Power)
        self.label_238.setGeometry(QtCore.QRect(15, 20, 276, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_238.setFont(font)
        self.label_238.setObjectName("label_238")
        self.groupBox_21 = QtWidgets.QGroupBox(self.Power)
        self.groupBox_21.setGeometry(QtCore.QRect(15, 50, 366, 96))
        self.groupBox_21.setProperty("large", True)
        self.groupBox_21.setObjectName("groupBox_21")
        self.label_192 = QtWidgets.QLabel(self.groupBox_21)
        self.label_192.setGeometry(QtCore.QRect(330, 45, 16, 21))
        self.label_192.setObjectName("label_192")
        self.label_284 = QtWidgets.QLabel(self.groupBox_21)
        self.label_284.setGeometry(QtCore.QRect(330, 70, 20, 21))
        self.label_284.setObjectName("label_284")
        self.label_193 = QtWidgets.QLabel(self.groupBox_21)
        self.label_193.setGeometry(QtCore.QRect(140, 70, 16, 21))
        self.label_193.setObjectName("label_193")
        self.label_191 = QtWidgets.QLabel(self.groupBox_21)
        self.label_191.setGeometry(QtCore.QRect(140, 45, 16, 21))
        self.label_191.setObjectName("label_191")
        self.label_286 = QtWidgets.QLabel(self.groupBox_21)
        self.label_286.setGeometry(QtCore.QRect(10, 70, 61, 21))
        self.label_286.setObjectName("label_286")
        self.sensorPower = QtWidgets.QLineEdit(self.groupBox_21)
        self.sensorPower.setEnabled(False)
        self.sensorPower.setGeometry(QtCore.QRect(80, 45, 56, 21))
        self.sensorPower.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sensorPower.setObjectName("sensorPower")
        self.label_287 = QtWidgets.QLabel(self.groupBox_21)
        self.label_287.setGeometry(QtCore.QRect(205, 45, 56, 21))
        self.label_287.setObjectName("label_287")
        self.label_165 = QtWidgets.QLabel(self.groupBox_21)
        self.label_165.setGeometry(QtCore.QRect(10, 21, 43, 21))
        self.label_165.setObjectName("label_165")
        self.label_285 = QtWidgets.QLabel(self.groupBox_21)
        self.label_285.setGeometry(QtCore.QRect(200, 70, 62, 21))
        self.label_285.setObjectName("label_285")
        self.label_289 = QtWidgets.QLabel(self.groupBox_21)
        self.label_289.setGeometry(QtCore.QRect(215, 20, 42, 21))
        self.label_289.setObjectName("label_289")
        self.sensorVoltage = QtWidgets.QLineEdit(self.groupBox_21)
        self.sensorVoltage.setEnabled(False)
        self.sensorVoltage.setGeometry(QtCore.QRect(80, 21, 56, 21))
        self.sensorVoltage.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sensorVoltage.setObjectName("sensorVoltage")
        self.label_288 = QtWidgets.QLabel(self.groupBox_21)
        self.label_288.setGeometry(QtCore.QRect(10, 45, 36, 21))
        self.label_288.setObjectName("label_288")
        self.consumptionWattHours = QtWidgets.QLineEdit(self.groupBox_21)
        self.consumptionWattHours.setEnabled(False)
        self.consumptionWattHours.setGeometry(QtCore.QRect(270, 70, 56, 21))
        self.consumptionWattHours.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.consumptionWattHours.setObjectName("consumptionWattHours")
        self.sensorCurrent = QtWidgets.QLineEdit(self.groupBox_21)
        self.sensorCurrent.setEnabled(False)
        self.sensorCurrent.setGeometry(QtCore.QRect(270, 20, 56, 21))
        self.sensorCurrent.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sensorCurrent.setObjectName("sensorCurrent")
        self.consumptionAvgAmps = QtWidgets.QLineEdit(self.groupBox_21)
        self.consumptionAvgAmps.setEnabled(False)
        self.consumptionAvgAmps.setGeometry(QtCore.QRect(270, 45, 56, 21))
        self.consumptionAvgAmps.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.consumptionAvgAmps.setObjectName("consumptionAvgAmps")
        self.label_173 = QtWidgets.QLabel(self.groupBox_21)
        self.label_173.setGeometry(QtCore.QRect(140, 21, 16, 21))
        self.label_173.setObjectName("label_173")
        self.consumptionAmpHours = QtWidgets.QLineEdit(self.groupBox_21)
        self.consumptionAmpHours.setEnabled(False)
        self.consumptionAmpHours.setGeometry(QtCore.QRect(80, 70, 56, 21))
        self.consumptionAmpHours.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.consumptionAmpHours.setObjectName("consumptionAmpHours")
        self.label_172 = QtWidgets.QLabel(self.groupBox_21)
        self.label_172.setGeometry(QtCore.QRect(330, 20, 16, 21))
        self.label_172.setObjectName("label_172")
        self.groupBox_22 = QtWidgets.QGroupBox(self.Power)
        self.groupBox_22.setGeometry(QtCore.QRect(15, 165, 366, 51))
        self.groupBox_22.setProperty("large", True)
        self.groupBox_22.setObjectName("groupBox_22")
        self.dewCurrentA = QtWidgets.QLineEdit(self.groupBox_22)
        self.dewCurrentA.setEnabled(False)
        self.dewCurrentA.setGeometry(QtCore.QRect(270, 20, 56, 21))
        self.dewCurrentA.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dewCurrentA.setObjectName("dewCurrentA")
        self.label_276 = QtWidgets.QLabel(self.groupBox_22)
        self.label_276.setGeometry(QtCore.QRect(150, 20, 16, 21))
        self.label_276.setObjectName("label_276")
        self.label_268 = QtWidgets.QLabel(self.groupBox_22)
        self.label_268.setGeometry(QtCore.QRect(215, 20, 42, 21))
        self.label_268.setObjectName("label_268")
        self.label_279 = QtWidgets.QLabel(self.groupBox_22)
        self.label_279.setGeometry(QtCore.QRect(330, 20, 16, 21))
        self.label_279.setObjectName("label_279")
        self.label_266 = QtWidgets.QLabel(self.groupBox_22)
        self.label_266.setGeometry(QtCore.QRect(11, 20, 79, 21))
        self.label_266.setObjectName("label_266")
        self.dewA = QtWidgets.QSpinBox(self.groupBox_22)
        self.dewA.setGeometry(QtCore.QRect(100, 20, 47, 21))
        self.dewA.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dewA.setMaximum(100)
        self.dewA.setSingleStep(10)
        self.dewA.setObjectName("dewA")
        self.groupBox_23 = QtWidgets.QGroupBox(self.Power)
        self.groupBox_23.setGeometry(QtCore.QRect(15, 235, 366, 51))
        self.groupBox_23.setProperty("large", True)
        self.groupBox_23.setObjectName("groupBox_23")
        self.label_280 = QtWidgets.QLabel(self.groupBox_23)
        self.label_280.setGeometry(QtCore.QRect(150, 20, 16, 21))
        self.label_280.setObjectName("label_280")
        self.label_274 = QtWidgets.QLabel(self.groupBox_23)
        self.label_274.setGeometry(QtCore.QRect(215, 20, 42, 21))
        self.label_274.setObjectName("label_274")
        self.label_281 = QtWidgets.QLabel(self.groupBox_23)
        self.label_281.setGeometry(QtCore.QRect(330, 20, 16, 21))
        self.label_281.setObjectName("label_281")
        self.label_275 = QtWidgets.QLabel(self.groupBox_23)
        self.label_275.setGeometry(QtCore.QRect(11, 20, 79, 21))
        self.label_275.setObjectName("label_275")
        self.dewB = QtWidgets.QSpinBox(self.groupBox_23)
        self.dewB.setGeometry(QtCore.QRect(100, 20, 47, 21))
        self.dewB.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dewB.setMaximum(100)
        self.dewB.setSingleStep(10)
        self.dewB.setObjectName("dewB")
        self.dewCurrentB = QtWidgets.QLineEdit(self.groupBox_23)
        self.dewCurrentB.setEnabled(False)
        self.dewCurrentB.setGeometry(QtCore.QRect(270, 20, 56, 21))
        self.dewCurrentB.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dewCurrentB.setObjectName("dewCurrentB")
        self.groupBox_24 = QtWidgets.QGroupBox(self.Power)
        self.groupBox_24.setGeometry(QtCore.QRect(15, 305, 366, 46))
        self.groupBox_24.setProperty("large", True)
        self.groupBox_24.setObjectName("groupBox_24")
        self.autoDew = QtWidgets.QCheckBox(self.groupBox_24)
        self.autoDew.setGeometry(QtCore.QRect(10, 20, 122, 21))
        self.autoDew.setObjectName("autoDew")
        self.mainTabWidget.addTab(self.Power, "")
        self.Tools = QtWidgets.QWidget()
        self.Tools.setObjectName("Tools")
        self.horizontalLayoutWidget_2 = QtWidgets.QWidget(self.Tools)
        self.horizontalLayoutWidget_2.setGeometry(QtCore.QRect(10, 10, 766, 702))
        self.horizontalLayoutWidget_2.setObjectName("horizontalLayoutWidget_2")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.horizontalLayoutWidget_2)
        self.horizontalLayout_3.setContentsMargins(0, 4, 4, 4)
        self.horizontalLayout_3.setSpacing(4)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.gridLayout_9 = QtWidgets.QGridLayout()
        self.gridLayout_9.setContentsMargins(4, 4, 4, 4)
        self.gridLayout_9.setSpacing(8)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.label_305 = QtWidgets.QLabel(self.horizontalLayoutWidget_2)
        self.label_305.setMinimumSize(QtCore.QSize(120, 0))
        self.label_305.setObjectName("label_305")
        self.gridLayout_9.addWidget(self.label_305, 2, 0, 1, 2)
        self.label_304 = QtWidgets.QLabel(self.horizontalLayoutWidget_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_304.setFont(font)
        self.label_304.setObjectName("label_304")
        self.gridLayout_9.addWidget(self.label_304, 0, 0, 1, 5)
        self.checkIncludeSubdirs = QtWidgets.QCheckBox(self.horizontalLayoutWidget_2)
        self.checkIncludeSubdirs.setMinimumSize(QtCore.QSize(0, 25))
        self.checkIncludeSubdirs.setObjectName("checkIncludeSubdirs")
        self.gridLayout_9.addWidget(self.checkIncludeSubdirs, 4, 0, 1, 5)
        self.line_88 = QtWidgets.QFrame(self.horizontalLayoutWidget_2)
        self.line_88.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_88.setMidLineWidth(1)
        self.line_88.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_88.setObjectName("line_88")
        self.gridLayout_9.addWidget(self.line_88, 1, 0, 1, 5)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_9.addItem(spacerItem4, 13, 2, 1, 1)
        self.renameDir = QtWidgets.QLineEdit(self.horizontalLayoutWidget_2)
        self.renameDir.setEnabled(False)
        self.renameDir.setMinimumSize(QtCore.QSize(0, 25))
        self.renameDir.setObjectName("renameDir")
        self.gridLayout_9.addWidget(self.renameDir, 3, 0, 1, 5)
        self.renameInputSelect = QtWidgets.QPushButton(self.horizontalLayoutWidget_2)
        self.renameInputSelect.setMinimumSize(QtCore.QSize(0, 25))
        self.renameInputSelect.setObjectName("renameInputSelect")
        self.gridLayout_9.addWidget(self.renameInputSelect, 2, 4, 1, 1)
        self.renameStart = QtWidgets.QPushButton(self.horizontalLayoutWidget_2)
        self.renameStart.setMinimumSize(QtCore.QSize(80, 25))
        self.renameStart.setObjectName("renameStart")
        self.gridLayout_9.addWidget(self.renameStart, 12, 4, 1, 1)
        self.label_60 = QtWidgets.QLabel(self.horizontalLayoutWidget_2)
        self.label_60.setObjectName("label_60")
        self.gridLayout_9.addWidget(self.label_60, 5, 0, 1, 5)
        self.renameProgress = QtWidgets.QProgressBar(self.horizontalLayoutWidget_2)
        self.renameProgress.setMinimumSize(QtCore.QSize(0, 25))
        self.renameProgress.setProperty("value", 0)
        self.renameProgress.setAlignment(QtCore.Qt.AlignCenter)
        self.renameProgress.setObjectName("renameProgress")
        self.gridLayout_9.addWidget(self.renameProgress, 11, 1, 1, 4)
        self.rename5 = QtWidgets.QComboBox(self.horizontalLayoutWidget_2)
        self.rename5.setMinimumSize(QtCore.QSize(0, 25))
        self.rename5.setObjectName("rename5")
        self.gridLayout_9.addWidget(self.rename5, 10, 1, 1, 4)
        self.rename4 = QtWidgets.QComboBox(self.horizontalLayoutWidget_2)
        self.rename4.setMinimumSize(QtCore.QSize(0, 25))
        self.rename4.setObjectName("rename4")
        self.gridLayout_9.addWidget(self.rename4, 9, 1, 1, 4)
        self.rename3 = QtWidgets.QComboBox(self.horizontalLayoutWidget_2)
        self.rename3.setMinimumSize(QtCore.QSize(0, 25))
        self.rename3.setObjectName("rename3")
        self.gridLayout_9.addWidget(self.rename3, 8, 1, 1, 4)
        self.rename2 = QtWidgets.QComboBox(self.horizontalLayoutWidget_2)
        self.rename2.setMinimumSize(QtCore.QSize(0, 25))
        self.rename2.setObjectName("rename2")
        self.gridLayout_9.addWidget(self.rename2, 7, 1, 1, 4)
        self.rename1 = QtWidgets.QComboBox(self.horizontalLayoutWidget_2)
        self.rename1.setMinimumSize(QtCore.QSize(0, 25))
        self.rename1.setObjectName("rename1")
        self.gridLayout_9.addWidget(self.rename1, 6, 1, 1, 4)
        self.label_8 = QtWidgets.QLabel(self.horizontalLayoutWidget_2)
        self.label_8.setObjectName("label_8")
        self.gridLayout_9.addWidget(self.label_8, 6, 0, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.horizontalLayoutWidget_2)
        self.label_9.setObjectName("label_9")
        self.gridLayout_9.addWidget(self.label_9, 7, 0, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.horizontalLayoutWidget_2)
        self.label_10.setObjectName("label_10")
        self.gridLayout_9.addWidget(self.label_10, 8, 0, 1, 1)
        self.label_11 = QtWidgets.QLabel(self.horizontalLayoutWidget_2)
        self.label_11.setObjectName("label_11")
        self.gridLayout_9.addWidget(self.label_11, 9, 0, 1, 1)
        self.label_39 = QtWidgets.QLabel(self.horizontalLayoutWidget_2)
        self.label_39.setObjectName("label_39")
        self.gridLayout_9.addWidget(self.label_39, 10, 0, 1, 1)
        self.label_57 = QtWidgets.QLabel(self.horizontalLayoutWidget_2)
        self.label_57.setObjectName("label_57")
        self.gridLayout_9.addWidget(self.label_57, 11, 0, 1, 1)
        self.horizontalLayout_4.addLayout(self.gridLayout_9)
        self.horizontalLayout_3.addLayout(self.horizontalLayout_4)
        self.line_36 = QtWidgets.QFrame(self.horizontalLayoutWidget_2)
        self.line_36.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_36.setMidLineWidth(1)
        self.line_36.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_36.setObjectName("line_36")
        self.horizontalLayout_3.addWidget(self.line_36)
        self.gridLayout_10 = QtWidgets.QGridLayout()
        self.gridLayout_10.setContentsMargins(4, 4, 4, 4)
        self.gridLayout_10.setSpacing(8)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.line_45 = QtWidgets.QFrame(self.horizontalLayoutWidget_2)
        self.line_45.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_45.setMidLineWidth(1)
        self.line_45.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_45.setObjectName("line_45")
        self.gridLayout_10.addWidget(self.line_45, 1, 0, 1, 2)
        self.posButton6 = QtWidgets.QPushButton(self.horizontalLayoutWidget_2)
        self.posButton6.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.posButton6.setFont(font)
        self.posButton6.setObjectName("posButton6")
        self.gridLayout_10.addWidget(self.posButton6, 5, 0, 1, 1)
        self.label_83 = QtWidgets.QLabel(self.horizontalLayoutWidget_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_83.setFont(font)
        self.label_83.setObjectName("label_83")
        self.gridLayout_10.addWidget(self.label_83, 0, 0, 1, 2)
        self.posButton4 = QtWidgets.QPushButton(self.horizontalLayoutWidget_2)
        self.posButton4.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.posButton4.setFont(font)
        self.posButton4.setObjectName("posButton4")
        self.gridLayout_10.addWidget(self.posButton4, 4, 0, 1, 1)
        self.posButton5 = QtWidgets.QPushButton(self.horizontalLayoutWidget_2)
        self.posButton5.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.posButton5.setFont(font)
        self.posButton5.setObjectName("posButton5")
        self.gridLayout_10.addWidget(self.posButton5, 4, 1, 1, 1)
        self.posButton3 = QtWidgets.QPushButton(self.horizontalLayoutWidget_2)
        self.posButton3.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.posButton3.setFont(font)
        self.posButton3.setObjectName("posButton3")
        self.gridLayout_10.addWidget(self.posButton3, 3, 1, 1, 1)
        self.posButton1 = QtWidgets.QPushButton(self.horizontalLayoutWidget_2)
        self.posButton1.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.posButton1.setFont(font)
        self.posButton1.setObjectName("posButton1")
        self.gridLayout_10.addWidget(self.posButton1, 2, 1, 1, 1)
        self.posButton0 = QtWidgets.QPushButton(self.horizontalLayoutWidget_2)
        self.posButton0.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.posButton0.setFont(font)
        self.posButton0.setObjectName("posButton0")
        self.gridLayout_10.addWidget(self.posButton0, 2, 0, 1, 1)
        self.posButton2 = QtWidgets.QPushButton(self.horizontalLayoutWidget_2)
        self.posButton2.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.posButton2.setFont(font)
        self.posButton2.setObjectName("posButton2")
        self.gridLayout_10.addWidget(self.posButton2, 3, 0, 1, 1)
        self.posButton7 = QtWidgets.QPushButton(self.horizontalLayoutWidget_2)
        self.posButton7.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.posButton7.setFont(font)
        self.posButton7.setObjectName("posButton7")
        self.gridLayout_10.addWidget(self.posButton7, 5, 1, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_10.addItem(spacerItem5, 6, 0, 1, 1)
        self.horizontalLayout_3.addLayout(self.gridLayout_10)
        self.mainTabWidget.addTab(self.Tools, "")
        self.Setting = QtWidgets.QWidget()
        self.Setting.setObjectName("Setting")
        self.settingsTabWidget = QtWidgets.QTabWidget(self.Setting)
        self.settingsTabWidget.setGeometry(QtCore.QRect(5, 5, 776, 421))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.settingsTabWidget.sizePolicy().hasHeightForWidth())
        self.settingsTabWidget.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.settingsTabWidget.setFont(font)
        self.settingsTabWidget.setAutoFillBackground(True)
        self.settingsTabWidget.setStyleSheet("")
        self.settingsTabWidget.setTabPosition(QtWidgets.QTabWidget.North)
        self.settingsTabWidget.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.settingsTabWidget.setElideMode(QtCore.Qt.ElideLeft)
        self.settingsTabWidget.setUsesScrollButtons(True)
        self.settingsTabWidget.setObjectName("settingsTabWidget")
        self.Devices = QtWidgets.QWidget()
        self.Devices.setObjectName("Devices")
        self.gridLayoutWidget = QtWidgets.QWidget(self.Devices)
        self.gridLayoutWidget.setGeometry(QtCore.QRect(10, 5, 756, 381))
        self.gridLayoutWidget.setObjectName("gridLayoutWidget")
        self.gridLayout = QtWidgets.QGridLayout(self.gridLayoutWidget)
        self.gridLayout.setContentsMargins(6, 15, 0, 0)
        self.gridLayout.setSpacing(6)
        self.gridLayout.setObjectName("gridLayout")
        spacerItem6 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem6, 16, 0, 1, 2)
        self.domeDevice = QtWidgets.QComboBox(self.gridLayoutWidget)
        self.domeDevice.setMinimumSize(QtCore.QSize(200, 25))
        self.domeDevice.setObjectName("domeDevice")
        self.gridLayout.addWidget(self.domeDevice, 7, 1, 1, 1)
        self.line_81 = QtWidgets.QFrame(self.gridLayoutWidget)
        self.line_81.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_81.setMidLineWidth(1)
        self.line_81.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_81.setObjectName("line_81")
        self.gridLayout.addWidget(self.line_81, 1, 6, 1, 2)
        self.label_302 = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_302.setFont(font)
        self.label_302.setObjectName("label_302")
        self.gridLayout.addWidget(self.label_302, 9, 0, 2, 2)
        self.measureDevice = QtWidgets.QComboBox(self.gridLayoutWidget)
        self.measureDevice.setMinimumSize(QtCore.QSize(200, 25))
        self.measureDevice.setObjectName("measureDevice")
        self.gridLayout.addWidget(self.measureDevice, 2, 7, 2, 1)
        self.label_49 = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.label_49.setFont(font)
        self.label_49.setObjectName("label_49")
        self.gridLayout.addWidget(self.label_49, 4, 6, 3, 1)
        self.environDevice = QtWidgets.QComboBox(self.gridLayoutWidget)
        self.environDevice.setMinimumSize(QtCore.QSize(200, 25))
        self.environDevice.setObjectName("environDevice")
        self.gridLayout.addWidget(self.environDevice, 14, 1, 2, 1)
        self.remoteDevice = QtWidgets.QComboBox(self.gridLayoutWidget)
        self.remoteDevice.setMinimumSize(QtCore.QSize(200, 25))
        self.remoteDevice.setObjectName("remoteDevice")
        self.gridLayout.addWidget(self.remoteDevice, 4, 7, 3, 1)
        self.line_86 = QtWidgets.QFrame(self.gridLayoutWidget)
        self.line_86.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_86.setLineWidth(2)
        self.line_86.setMidLineWidth(2)
        self.line_86.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_86.setObjectName("line_86")
        self.gridLayout.addWidget(self.line_86, 0, 4, 17, 1)
        self.line_80 = QtWidgets.QFrame(self.gridLayoutWidget)
        self.line_80.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_80.setMidLineWidth(1)
        self.line_80.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_80.setObjectName("line_80")
        self.gridLayout.addWidget(self.line_80, 1, 0, 1, 2)
        self.label_263 = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_263.setFont(font)
        self.label_263.setObjectName("label_263")
        self.gridLayout.addWidget(self.label_263, 0, 0, 1, 2)
        self.powerDevice = QtWidgets.QComboBox(self.gridLayoutWidget)
        self.powerDevice.setMinimumSize(QtCore.QSize(200, 25))
        self.powerDevice.setObjectName("powerDevice")
        self.gridLayout.addWidget(self.powerDevice, 7, 7, 1, 1)
        self.label_264 = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_264.setFont(font)
        self.label_264.setObjectName("label_264")
        self.gridLayout.addWidget(self.label_264, 0, 6, 1, 1)
        self.label_46 = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.label_46.setFont(font)
        self.label_46.setObjectName("label_46")
        self.gridLayout.addWidget(self.label_46, 8, 6, 1, 1)
        self.sky = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.sky.setFont(font)
        self.sky.setObjectName("sky")
        self.gridLayout.addWidget(self.sky, 12, 0, 2, 1)
        spacerItem7 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem7, 0, 5, 1, 1)
        self.relayDevice = QtWidgets.QComboBox(self.gridLayoutWidget)
        self.relayDevice.setMinimumSize(QtCore.QSize(200, 25))
        self.relayDevice.setObjectName("relayDevice")
        self.gridLayout.addWidget(self.relayDevice, 8, 7, 1, 1)
        self.env = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.env.setFont(font)
        self.env.setObjectName("env")
        self.gridLayout.addWidget(self.env, 14, 0, 2, 1)
        spacerItem8 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem8, 0, 2, 1, 1)
        self.line_82 = QtWidgets.QFrame(self.gridLayoutWidget)
        self.line_82.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_82.setMidLineWidth(1)
        self.line_82.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_82.setObjectName("line_82")
        self.gridLayout.addWidget(self.line_82, 11, 0, 1, 2)
        self.label_48 = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.label_48.setFont(font)
        self.label_48.setObjectName("label_48")
        self.gridLayout.addWidget(self.label_48, 2, 6, 2, 1)
        self.ccd = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.ccd.setFont(font)
        self.ccd.setObjectName("ccd")
        self.gridLayout.addWidget(self.ccd, 2, 0, 2, 1)
        self.astrometryDevice = QtWidgets.QComboBox(self.gridLayoutWidget)
        self.astrometryDevice.setMinimumSize(QtCore.QSize(200, 25))
        self.astrometryDevice.setObjectName("astrometryDevice")
        self.gridLayout.addWidget(self.astrometryDevice, 4, 1, 3, 1)
        self.skymeterDevice = QtWidgets.QComboBox(self.gridLayoutWidget)
        self.skymeterDevice.setMinimumSize(QtCore.QSize(200, 25))
        self.skymeterDevice.setObjectName("skymeterDevice")
        self.gridLayout.addWidget(self.skymeterDevice, 12, 1, 2, 1)
        self.ast = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.ast.setFont(font)
        self.ast.setObjectName("ast")
        self.gridLayout.addWidget(self.ast, 4, 0, 3, 1)
        self.dome = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.dome.setFont(font)
        self.dome.setObjectName("dome")
        self.gridLayout.addWidget(self.dome, 7, 0, 1, 1)
        self.imagingDevice = QtWidgets.QComboBox(self.gridLayoutWidget)
        self.imagingDevice.setMinimumSize(QtCore.QSize(200, 25))
        self.imagingDevice.setObjectName("imagingDevice")
        self.gridLayout.addWidget(self.imagingDevice, 2, 1, 2, 1)
        self.po = QtWidgets.QLabel(self.gridLayoutWidget)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.po.setFont(font)
        self.po.setObjectName("po")
        self.gridLayout.addWidget(self.po, 7, 6, 1, 1)
        self.settingsTabWidget.addTab(self.Devices, "")
        self.INDI = QtWidgets.QWidget()
        self.INDI.setObjectName("INDI")
        self.gridLayoutWidget_2 = QtWidgets.QWidget(self.INDI)
        self.gridLayoutWidget_2.setGeometry(QtCore.QRect(10, 5, 751, 376))
        self.gridLayoutWidget_2.setObjectName("gridLayoutWidget_2")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.gridLayoutWidget_2)
        self.gridLayout_2.setContentsMargins(0, 15, 0, 0)
        self.gridLayout_2.setSpacing(6)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.domePort = QtWidgets.QLineEdit(self.gridLayoutWidget_2)
        self.domePort.setMinimumSize(QtCore.QSize(0, 0))
        self.domePort.setMaximumSize(QtCore.QSize(50, 16777215))
        self.domePort.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.domePort.setObjectName("domePort")
        self.gridLayout_2.addWidget(self.domePort, 10, 2, 1, 1)
        self.label_13 = QtWidgets.QLabel(self.gridLayoutWidget_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_13.setFont(font)
        self.label_13.setObjectName("label_13")
        self.gridLayout_2.addWidget(self.label_13, 2, 0, 1, 1)
        self.label_53 = QtWidgets.QLabel(self.gridLayoutWidget_2)
        self.label_53.setObjectName("label_53")
        self.gridLayout_2.addWidget(self.label_53, 7, 8, 1, 1)
        self.label_51 = QtWidgets.QLabel(self.gridLayoutWidget_2)
        self.label_51.setObjectName("label_51")
        self.gridLayout_2.addWidget(self.label_51, 2, 8, 1, 1)
        self.domeDeviceName = QtWidgets.QComboBox(self.gridLayoutWidget_2)
        self.domeDeviceName.setObjectName("domeDeviceName")
        self.gridLayout_2.addWidget(self.domeDeviceName, 11, 1, 1, 1)
        self.indiMessage = QtWidgets.QCheckBox(self.gridLayoutWidget_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.indiMessage.setFont(font)
        self.indiMessage.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.indiMessage.setChecked(False)
        self.indiMessage.setObjectName("indiMessage")
        self.gridLayout_2.addWidget(self.indiMessage, 13, 0, 1, 3)
        spacerItem9 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem9, 1, 3, 1, 1)
        self.label_mountIP_6 = QtWidgets.QLabel(self.gridLayoutWidget_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountIP_6.setFont(font)
        self.label_mountIP_6.setObjectName("label_mountIP_6")
        self.gridLayout_2.addWidget(self.label_mountIP_6, 6, 0, 1, 1)
        spacerItem10 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem10, 14, 0, 1, 1)
        self.line_55 = QtWidgets.QFrame(self.gridLayoutWidget_2)
        self.line_55.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_55.setMidLineWidth(1)
        self.line_55.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_55.setObjectName("line_55")
        self.gridLayout_2.addWidget(self.line_55, 8, 0, 1, 3)
        self.line_71 = QtWidgets.QFrame(self.gridLayoutWidget_2)
        self.line_71.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_71.setMidLineWidth(1)
        self.line_71.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_71.setObjectName("line_71")
        self.gridLayout_2.addWidget(self.line_71, 3, 8, 1, 3)
        self.label_17 = QtWidgets.QLabel(self.gridLayoutWidget_2)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_17.setFont(font)
        self.label_17.setObjectName("label_17")
        self.gridLayout_2.addWidget(self.label_17, 5, 0, 1, 3)
        self.domeHost = QtWidgets.QLineEdit(self.gridLayoutWidget_2)
        self.domeHost.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.domeHost.setFont(font)
        self.domeHost.setMouseTracking(False)
        self.domeHost.setAcceptDrops(False)
        self.domeHost.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.domeHost.setMaxLength(50)
        self.domeHost.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.domeHost.setDragEnabled(True)
        self.domeHost.setReadOnly(False)
        self.domeHost.setObjectName("domeHost")
        self.gridLayout_2.addWidget(self.domeHost, 10, 1, 1, 1)
        self.imagingPort = QtWidgets.QLineEdit(self.gridLayoutWidget_2)
        self.imagingPort.setMinimumSize(QtCore.QSize(0, 0))
        self.imagingPort.setMaximumSize(QtCore.QSize(50, 16777215))
        self.imagingPort.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.imagingPort.setObjectName("imagingPort")
        self.gridLayout_2.addWidget(self.imagingPort, 6, 2, 1, 1)
        self.label_23 = QtWidgets.QLabel(self.gridLayoutWidget_2)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_23.setFont(font)
        self.label_23.setObjectName("label_23")
        self.gridLayout_2.addWidget(self.label_23, 0, 8, 1, 3)
        self.label_22 = QtWidgets.QLabel(self.gridLayoutWidget_2)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_22.setFont(font)
        self.label_22.setObjectName("label_22")
        self.gridLayout_2.addWidget(self.label_22, 9, 0, 1, 3)
        self.label_36 = QtWidgets.QLabel(self.gridLayoutWidget_2)
        self.label_36.setObjectName("label_36")
        self.gridLayout_2.addWidget(self.label_36, 1, 8, 1, 1)
        self.environPort = QtWidgets.QLineEdit(self.gridLayoutWidget_2)
        self.environPort.setMinimumSize(QtCore.QSize(0, 0))
        self.environPort.setMaximumSize(QtCore.QSize(50, 16777215))
        self.environPort.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.environPort.setObjectName("environPort")
        self.gridLayout_2.addWidget(self.environPort, 1, 2, 1, 1)
        self.line_60 = QtWidgets.QFrame(self.gridLayoutWidget_2)
        self.line_60.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_60.setMidLineWidth(1)
        self.line_60.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_60.setObjectName("line_60")
        self.gridLayout_2.addWidget(self.line_60, 8, 8, 1, 3)
        self.label_16 = QtWidgets.QLabel(self.gridLayoutWidget_2)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_16.setFont(font)
        self.label_16.setObjectName("label_16")
        self.gridLayout_2.addWidget(self.label_16, 0, 0, 1, 3)
        self.label_110 = QtWidgets.QLabel(self.gridLayoutWidget_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_110.setFont(font)
        self.label_110.setObjectName("label_110")
        self.gridLayout_2.addWidget(self.label_110, 5, 8, 1, 3)
        self.skymeterPort = QtWidgets.QLineEdit(self.gridLayoutWidget_2)
        self.skymeterPort.setMaximumSize(QtCore.QSize(50, 16777215))
        self.skymeterPort.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.skymeterPort.setObjectName("skymeterPort")
        self.gridLayout_2.addWidget(self.skymeterPort, 1, 10, 1, 1)
        self.powerDeviceName = QtWidgets.QComboBox(self.gridLayoutWidget_2)
        self.powerDeviceName.setObjectName("powerDeviceName")
        self.gridLayout_2.addWidget(self.powerDeviceName, 7, 9, 1, 1)
        self.environDeviceName = QtWidgets.QComboBox(self.gridLayoutWidget_2)
        self.environDeviceName.setObjectName("environDeviceName")
        self.gridLayout_2.addWidget(self.environDeviceName, 2, 1, 1, 1)
        spacerItem11 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem11, 1, 5, 1, 1)
        self.powerPort = QtWidgets.QLineEdit(self.gridLayoutWidget_2)
        self.powerPort.setMaximumSize(QtCore.QSize(50, 16777215))
        self.powerPort.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerPort.setObjectName("powerPort")
        self.gridLayout_2.addWidget(self.powerPort, 6, 10, 1, 1)
        self.skymeterDeviceName = QtWidgets.QComboBox(self.gridLayoutWidget_2)
        self.skymeterDeviceName.setObjectName("skymeterDeviceName")
        self.gridLayout_2.addWidget(self.skymeterDeviceName, 2, 9, 1, 1)
        self.label_66 = QtWidgets.QLabel(self.gridLayoutWidget_2)
        self.label_66.setObjectName("label_66")
        self.gridLayout_2.addWidget(self.label_66, 6, 8, 1, 1)
        self.powerHost = QtWidgets.QLineEdit(self.gridLayoutWidget_2)
        self.powerHost.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.powerHost.setObjectName("powerHost")
        self.gridLayout_2.addWidget(self.powerHost, 6, 9, 1, 1)
        self.imagingDeviceName = QtWidgets.QComboBox(self.gridLayoutWidget_2)
        self.imagingDeviceName.setObjectName("imagingDeviceName")
        self.gridLayout_2.addWidget(self.imagingDeviceName, 7, 1, 1, 1)
        self.skymeterHost = QtWidgets.QLineEdit(self.gridLayoutWidget_2)
        self.skymeterHost.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.skymeterHost.setObjectName("skymeterHost")
        self.gridLayout_2.addWidget(self.skymeterHost, 1, 9, 1, 1)
        self.label_19 = QtWidgets.QLabel(self.gridLayoutWidget_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_19.setFont(font)
        self.label_19.setObjectName("label_19")
        self.gridLayout_2.addWidget(self.label_19, 7, 0, 1, 1)
        self.line_50 = QtWidgets.QFrame(self.gridLayoutWidget_2)
        self.line_50.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_50.setMidLineWidth(1)
        self.line_50.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_50.setObjectName("line_50")
        self.gridLayout_2.addWidget(self.line_50, 12, 0, 1, 3)
        self.label_mountIP_3 = QtWidgets.QLabel(self.gridLayoutWidget_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountIP_3.setFont(font)
        self.label_mountIP_3.setObjectName("label_mountIP_3")
        self.gridLayout_2.addWidget(self.label_mountIP_3, 1, 0, 1, 1)
        self.imagingHost = QtWidgets.QLineEdit(self.gridLayoutWidget_2)
        self.imagingHost.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.imagingHost.setFont(font)
        self.imagingHost.setMouseTracking(False)
        self.imagingHost.setAcceptDrops(False)
        self.imagingHost.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.imagingHost.setMaxLength(50)
        self.imagingHost.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.imagingHost.setDragEnabled(True)
        self.imagingHost.setReadOnly(False)
        self.imagingHost.setObjectName("imagingHost")
        self.gridLayout_2.addWidget(self.imagingHost, 6, 1, 1, 1)
        self.environHost = QtWidgets.QLineEdit(self.gridLayoutWidget_2)
        self.environHost.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.environHost.setFont(font)
        self.environHost.setMouseTracking(False)
        self.environHost.setAcceptDrops(False)
        self.environHost.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.environHost.setMaxLength(50)
        self.environHost.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.environHost.setDragEnabled(True)
        self.environHost.setReadOnly(False)
        self.environHost.setObjectName("environHost")
        self.gridLayout_2.addWidget(self.environHost, 1, 1, 1, 1)
        self.label_mountIP_7 = QtWidgets.QLabel(self.gridLayoutWidget_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountIP_7.setFont(font)
        self.label_mountIP_7.setObjectName("label_mountIP_7")
        self.gridLayout_2.addWidget(self.label_mountIP_7, 10, 0, 1, 1)
        self.line_49 = QtWidgets.QFrame(self.gridLayoutWidget_2)
        self.line_49.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_49.setMidLineWidth(1)
        self.line_49.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_49.setObjectName("line_49")
        self.gridLayout_2.addWidget(self.line_49, 3, 0, 2, 3)
        self.label_21 = QtWidgets.QLabel(self.gridLayoutWidget_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_21.setFont(font)
        self.label_21.setObjectName("label_21")
        self.gridLayout_2.addWidget(self.label_21, 11, 0, 1, 1)
        self.line_57 = QtWidgets.QFrame(self.gridLayoutWidget_2)
        self.line_57.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_57.setLineWidth(2)
        self.line_57.setMidLineWidth(2)
        self.line_57.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_57.setObjectName("line_57")
        self.gridLayout_2.addWidget(self.line_57, 0, 4, 15, 1)
        self.settingsTabWidget.addTab(self.INDI, "")
        self.Imaging = QtWidgets.QWidget()
        self.Imaging.setObjectName("Imaging")
        self.label_294 = QtWidgets.QLabel(self.Imaging)
        self.label_294.setGeometry(QtCore.QRect(355, 50, 16, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_294.setFont(font)
        self.label_294.setObjectName("label_294")
        self.label_295 = QtWidgets.QLabel(self.Imaging)
        self.label_295.setGeometry(QtCore.QRect(355, 255, 16, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_295.setFont(font)
        self.label_295.setObjectName("label_295")
        self.label_296 = QtWidgets.QLabel(self.Imaging)
        self.label_296.setGeometry(QtCore.QRect(355, 155, 16, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_296.setFont(font)
        self.label_296.setObjectName("label_296")
        self.label_297 = QtWidgets.QLabel(self.Imaging)
        self.label_297.setGeometry(QtCore.QRect(10, 220, 166, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_297.setFont(font)
        self.label_297.setObjectName("label_297")
        self.label_40 = QtWidgets.QLabel(self.Imaging)
        self.label_40.setGeometry(QtCore.QRect(10, 255, 166, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_40.setFont(font)
        self.label_40.setObjectName("label_40")
        self.label_298 = QtWidgets.QLabel(self.Imaging)
        self.label_298.setGeometry(QtCore.QRect(10, 20, 146, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_298.setFont(font)
        self.label_298.setObjectName("label_298")
        self.label_41 = QtWidgets.QLabel(self.Imaging)
        self.label_41.setGeometry(QtCore.QRect(10, 85, 166, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_41.setFont(font)
        self.label_41.setObjectName("label_41")
        self.isoSetting = QtWidgets.QDoubleSpinBox(self.Imaging)
        self.isoSetting.setGeometry(QtCore.QRect(300, 220, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.isoSetting.setFont(font)
        self.isoSetting.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.isoSetting.setDecimals(0)
        self.isoSetting.setMinimum(100.0)
        self.isoSetting.setMaximum(128000.0)
        self.isoSetting.setSingleStep(100.0)
        self.isoSetting.setProperty("value", 800.0)
        self.isoSetting.setObjectName("isoSetting")
        self.subFrame = QtWidgets.QDoubleSpinBox(self.Imaging)
        self.subFrame.setGeometry(QtCore.QRect(300, 155, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.subFrame.setFont(font)
        self.subFrame.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.subFrame.setDecimals(0)
        self.subFrame.setMinimum(10.0)
        self.subFrame.setMaximum(100.0)
        self.subFrame.setSingleStep(10.0)
        self.subFrame.setProperty("value", 100.0)
        self.subFrame.setObjectName("subFrame")
        self.expTime = QtWidgets.QDoubleSpinBox(self.Imaging)
        self.expTime.setGeometry(QtCore.QRect(300, 50, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.expTime.setFont(font)
        self.expTime.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.expTime.setDecimals(0)
        self.expTime.setMinimum(1.0)
        self.expTime.setMaximum(300.0)
        self.expTime.setSingleStep(1.0)
        self.expTime.setProperty("value", 1.0)
        self.expTime.setObjectName("expTime")
        self.settleTimeMount = QtWidgets.QDoubleSpinBox(self.Imaging)
        self.settleTimeMount.setGeometry(QtCore.QRect(300, 255, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.settleTimeMount.setFont(font)
        self.settleTimeMount.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.settleTimeMount.setDecimals(0)
        self.settleTimeMount.setMinimum(0.0)
        self.settleTimeMount.setMaximum(100.0)
        self.settleTimeMount.setSingleStep(1.0)
        self.settleTimeMount.setProperty("value", 1.0)
        self.settleTimeMount.setObjectName("settleTimeMount")
        self.checkFastDownload = QtWidgets.QCheckBox(self.Imaging)
        self.checkFastDownload.setGeometry(QtCore.QRect(20, 190, 166, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkFastDownload.setFont(font)
        self.checkFastDownload.setChecked(False)
        self.checkFastDownload.setObjectName("checkFastDownload")
        self.checkKeepImages = QtWidgets.QCheckBox(self.Imaging)
        self.checkKeepImages.setGeometry(QtCore.QRect(10, 330, 346, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkKeepImages.setFont(font)
        self.checkKeepImages.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.checkKeepImages.setChecked(False)
        self.checkKeepImages.setObjectName("checkKeepImages")
        self.label_299 = QtWidgets.QLabel(self.Imaging)
        self.label_299.setGeometry(QtCore.QRect(10, 50, 166, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_299.setFont(font)
        self.label_299.setObjectName("label_299")
        self.line_91 = QtWidgets.QFrame(self.Imaging)
        self.line_91.setGeometry(QtCore.QRect(10, 40, 361, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_91.setFont(font)
        self.line_91.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_91.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_91.setObjectName("line_91")
        self.binning = QtWidgets.QDoubleSpinBox(self.Imaging)
        self.binning.setGeometry(QtCore.QRect(300, 85, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.binning.setFont(font)
        self.binning.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.binning.setDecimals(0)
        self.binning.setMinimum(1.0)
        self.binning.setMaximum(9.0)
        self.binning.setSingleStep(1.0)
        self.binning.setProperty("value", 1.0)
        self.binning.setObjectName("binning")
        self.line_92 = QtWidgets.QFrame(self.Imaging)
        self.line_92.setGeometry(QtCore.QRect(380, 10, 11, 371))
        self.line_92.setStyleSheet("")
        self.line_92.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_92.setLineWidth(2)
        self.line_92.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_92.setObjectName("line_92")
        self.label_307 = QtWidgets.QLabel(self.Imaging)
        self.label_307.setGeometry(QtCore.QRect(10, 155, 166, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_307.setFont(font)
        self.label_307.setObjectName("label_307")
        self.label_166 = QtWidgets.QLabel(self.Imaging)
        self.label_166.setGeometry(QtCore.QRect(10, 120, 166, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_166.setFont(font)
        self.label_166.setObjectName("label_166")
        self.label_313 = QtWidgets.QLabel(self.Imaging)
        self.label_313.setGeometry(QtCore.QRect(355, 120, 21, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_313.setFont(font)
        self.label_313.setObjectName("label_313")
        self.pixelSize = QtWidgets.QDoubleSpinBox(self.Imaging)
        self.pixelSize.setEnabled(False)
        self.pixelSize.setGeometry(QtCore.QRect(300, 120, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.pixelSize.setFont(font)
        self.pixelSize.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.pixelSize.setDecimals(1)
        self.pixelSize.setMinimum(1.0)
        self.pixelSize.setMaximum(50.0)
        self.pixelSize.setSingleStep(0.1)
        self.pixelSize.setProperty("value", 5.0)
        self.pixelSize.setObjectName("pixelSize")
        self.label_100 = QtWidgets.QLabel(self.Imaging)
        self.label_100.setGeometry(QtCore.QRect(10, 290, 166, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_100.setFont(font)
        self.label_100.setObjectName("label_100")
        self.settleTimeDome = QtWidgets.QDoubleSpinBox(self.Imaging)
        self.settleTimeDome.setGeometry(QtCore.QRect(300, 290, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.settleTimeDome.setFont(font)
        self.settleTimeDome.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.settleTimeDome.setDecimals(0)
        self.settleTimeDome.setMinimum(0.0)
        self.settleTimeDome.setMaximum(100.0)
        self.settleTimeDome.setSingleStep(1.0)
        self.settleTimeDome.setProperty("value", 1.0)
        self.settleTimeDome.setObjectName("settleTimeDome")
        self.label_314 = QtWidgets.QLabel(self.Imaging)
        self.label_314.setGeometry(QtCore.QRect(355, 290, 16, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_314.setFont(font)
        self.label_314.setObjectName("label_314")
        self.label_306 = QtWidgets.QLabel(self.Imaging)
        self.label_306.setGeometry(QtCore.QRect(400, 20, 171, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_306.setFont(font)
        self.label_306.setObjectName("label_306")
        self.line_93 = QtWidgets.QFrame(self.Imaging)
        self.line_93.setGeometry(QtCore.QRect(400, 40, 361, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_93.setFont(font)
        self.line_93.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_93.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_93.setObjectName("line_93")
        self.searchRadius = QtWidgets.QDoubleSpinBox(self.Imaging)
        self.searchRadius.setGeometry(QtCore.QRect(705, 50, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.searchRadius.setFont(font)
        self.searchRadius.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.searchRadius.setDecimals(1)
        self.searchRadius.setMinimum(1.0)
        self.searchRadius.setMaximum(5.0)
        self.searchRadius.setSingleStep(0.2)
        self.searchRadius.setProperty("value", 2.0)
        self.searchRadius.setObjectName("searchRadius")
        self.label_315 = QtWidgets.QLabel(self.Imaging)
        self.label_315.setGeometry(QtCore.QRect(405, 50, 166, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_315.setFont(font)
        self.label_315.setObjectName("label_315")
        self.label_316 = QtWidgets.QLabel(self.Imaging)
        self.label_316.setGeometry(QtCore.QRect(405, 80, 166, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_316.setFont(font)
        self.label_316.setObjectName("label_316")
        self.solveTimeout = QtWidgets.QDoubleSpinBox(self.Imaging)
        self.solveTimeout.setGeometry(QtCore.QRect(705, 85, 51, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setBold(False)
        font.setWeight(50)
        self.solveTimeout.setFont(font)
        self.solveTimeout.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.solveTimeout.setDecimals(0)
        self.solveTimeout.setMinimum(2.0)
        self.solveTimeout.setMaximum(300.0)
        self.solveTimeout.setSingleStep(5.0)
        self.solveTimeout.setProperty("value", 30.0)
        self.solveTimeout.setObjectName("solveTimeout")
        self.settingsTabWidget.addTab(self.Imaging, "")
        self.MountRemote = QtWidgets.QWidget()
        self.MountRemote.setObjectName("MountRemote")
        self.openWeatherMapKey = QtWidgets.QLineEdit(self.MountRemote)
        self.openWeatherMapKey.setGeometry(QtCore.QRect(400, 45, 356, 22))
        self.openWeatherMapKey.setObjectName("openWeatherMapKey")
        self.line_63 = QtWidgets.QFrame(self.MountRemote)
        self.line_63.setGeometry(QtCore.QRect(400, 35, 356, 1))
        self.line_63.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_63.setMidLineWidth(1)
        self.line_63.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_63.setObjectName("line_63")
        self.line_97 = QtWidgets.QFrame(self.MountRemote)
        self.line_97.setGeometry(QtCore.QRect(380, 10, 11, 371))
        self.line_97.setStyleSheet("")
        self.line_97.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_97.setLineWidth(2)
        self.line_97.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_97.setObjectName("line_97")
        self.label_399 = QtWidgets.QLabel(self.MountRemote)
        self.label_399.setGeometry(QtCore.QRect(400, 15, 201, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_399.setFont(font)
        self.label_399.setObjectName("label_399")
        self.line_77 = QtWidgets.QFrame(self.MountRemote)
        self.line_77.setGeometry(QtCore.QRect(13, 205, 356, 1))
        self.line_77.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_77.setMidLineWidth(1)
        self.line_77.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_77.setObjectName("line_77")
        self.mountTypeConnection = QtWidgets.QLineEdit(self.MountRemote)
        self.mountTypeConnection.setEnabled(False)
        self.mountTypeConnection.setGeometry(QtCore.QRect(121, 105, 116, 26))
        self.mountTypeConnection.setMinimumSize(QtCore.QSize(25, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mountTypeConnection.setFont(font)
        self.mountTypeConnection.setMouseTracking(False)
        self.mountTypeConnection.setAcceptDrops(False)
        self.mountTypeConnection.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.mountTypeConnection.setMaxLength(18)
        self.mountTypeConnection.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.mountTypeConnection.setDragEnabled(True)
        self.mountTypeConnection.setReadOnly(True)
        self.mountTypeConnection.setObjectName("mountTypeConnection")
        self.label_200 = QtWidgets.QLabel(self.MountRemote)
        self.label_200.setGeometry(QtCore.QRect(15, 15, 116, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_200.setFont(font)
        self.label_200.setObjectName("label_200")
        self.mountHost = QtWidgets.QLineEdit(self.MountRemote)
        self.mountHost.setGeometry(QtCore.QRect(121, 45, 251, 26))
        self.mountHost.setMinimumSize(QtCore.QSize(25, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mountHost.setFont(font)
        self.mountHost.setMouseTracking(False)
        self.mountHost.setAcceptDrops(False)
        self.mountHost.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.mountHost.setMaxLength(50)
        self.mountHost.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.mountHost.setDragEnabled(True)
        self.mountHost.setReadOnly(False)
        self.mountHost.setObjectName("mountHost")
        self.label_mountMAC = QtWidgets.QLabel(self.MountRemote)
        self.label_mountMAC.setGeometry(QtCore.QRect(13, 75, 106, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountMAC.setFont(font)
        self.label_mountMAC.setObjectName("label_mountMAC")
        self.label_mountIP = QtWidgets.QLabel(self.MountRemote)
        self.label_mountIP.setGeometry(QtCore.QRect(15, 45, 101, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountIP.setFont(font)
        self.label_mountIP.setObjectName("label_mountIP")
        self.label_259 = QtWidgets.QLabel(self.MountRemote)
        self.label_259.setGeometry(QtCore.QRect(13, 215, 101, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_259.setFont(font)
        self.label_259.setObjectName("label_259")
        self.bootRackComp = QtWidgets.QPushButton(self.MountRemote)
        self.bootRackComp.setGeometry(QtCore.QRect(260, 215, 112, 26))
        self.bootRackComp.setMinimumSize(QtCore.QSize(0, 25))
        self.bootRackComp.setObjectName("bootRackComp")
        self.line_56 = QtWidgets.QFrame(self.MountRemote)
        self.line_56.setGeometry(QtCore.QRect(15, 35, 356, 1))
        self.line_56.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_56.setMidLineWidth(1)
        self.line_56.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_56.setObjectName("line_56")
        self.mountMAC = QtWidgets.QLineEdit(self.MountRemote)
        self.mountMAC.setEnabled(False)
        self.mountMAC.setGeometry(QtCore.QRect(121, 75, 116, 26))
        self.mountMAC.setMinimumSize(QtCore.QSize(25, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mountMAC.setFont(font)
        self.mountMAC.setMouseTracking(False)
        self.mountMAC.setAcceptDrops(False)
        self.mountMAC.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.mountMAC.setMaxLength(18)
        self.mountMAC.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.mountMAC.setDragEnabled(True)
        self.mountMAC.setReadOnly(True)
        self.mountMAC.setObjectName("mountMAC")
        self.label_195 = QtWidgets.QLabel(self.MountRemote)
        self.label_195.setGeometry(QtCore.QRect(13, 183, 126, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_195.setFont(font)
        self.label_195.setObjectName("label_195")
        self.label_mountMAC_2 = QtWidgets.QLabel(self.MountRemote)
        self.label_mountMAC_2.setGeometry(QtCore.QRect(13, 105, 106, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountMAC_2.setFont(font)
        self.label_mountMAC_2.setObjectName("label_mountMAC_2")
        self.rackCompMAC = QtWidgets.QLineEdit(self.MountRemote)
        self.rackCompMAC.setGeometry(QtCore.QRect(121, 215, 116, 26))
        self.rackCompMAC.setMinimumSize(QtCore.QSize(25, 0))
        self.rackCompMAC.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.rackCompMAC.setObjectName("rackCompMAC")
        self.settingsTabWidget.addTab(self.MountRemote, "")
        self.Horizon = QtWidgets.QWidget()
        self.Horizon.setObjectName("Horizon")
        self.line_96 = QtWidgets.QFrame(self.Horizon)
        self.line_96.setGeometry(QtCore.QRect(380, 10, 11, 371))
        self.line_96.setStyleSheet("")
        self.line_96.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_96.setLineWidth(2)
        self.line_96.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_96.setObjectName("line_96")
        self.checkAutoDeletePoints = QtWidgets.QCheckBox(self.Horizon)
        self.checkAutoDeletePoints.setGeometry(QtCore.QRect(10, 210, 211, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkAutoDeletePoints.setFont(font)
        self.checkAutoDeletePoints.setObjectName("checkAutoDeletePoints")
        self.groupBoxCoordinateSystems_2 = QtWidgets.QGroupBox(self.Horizon)
        self.groupBoxCoordinateSystems_2.setGeometry(QtCore.QRect(10, 250, 361, 86))
        self.groupBoxCoordinateSystems_2.setObjectName("groupBoxCoordinateSystems_2")
        self.checkSortHL = QtWidgets.QRadioButton(self.groupBoxCoordinateSystems_2)
        self.checkSortHL.setGeometry(QtCore.QRect(15, 60, 111, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkSortHL.setFont(font)
        self.checkSortHL.setObjectName("checkSortHL")
        self.checkSortEW = QtWidgets.QRadioButton(self.groupBoxCoordinateSystems_2)
        self.checkSortEW.setGeometry(QtCore.QRect(15, 40, 106, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkSortEW.setFont(font)
        self.checkSortEW.setObjectName("checkSortEW")
        self.checkSortNothing = QtWidgets.QRadioButton(self.groupBoxCoordinateSystems_2)
        self.checkSortNothing.setGeometry(QtCore.QRect(15, 20, 106, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkSortNothing.setFont(font)
        self.checkSortNothing.setObjectName("checkSortNothing")
        self.saveHorizonMaskAs = QtWidgets.QPushButton(self.Horizon)
        self.saveHorizonMaskAs.setEnabled(True)
        self.saveHorizonMaskAs.setGeometry(QtCore.QRect(190, 80, 86, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.saveHorizonMaskAs.setFont(font)
        self.saveHorizonMaskAs.setStyleSheet("")
        self.saveHorizonMaskAs.setObjectName("saveHorizonMaskAs")
        self.horizonFileName = QtWidgets.QLineEdit(self.Horizon)
        self.horizonFileName.setEnabled(False)
        self.horizonFileName.setGeometry(QtCore.QRect(10, 50, 361, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.horizonFileName.setFont(font)
        self.horizonFileName.setMouseTracking(False)
        self.horizonFileName.setFocusPolicy(QtCore.Qt.NoFocus)
        self.horizonFileName.setAcceptDrops(False)
        self.horizonFileName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.horizonFileName.setText("")
        self.horizonFileName.setReadOnly(True)
        self.horizonFileName.setObjectName("horizonFileName")
        self.line_58 = QtWidgets.QFrame(self.Horizon)
        self.line_58.setGeometry(QtCore.QRect(10, 40, 361, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_58.setFont(font)
        self.line_58.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_58.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_58.setObjectName("line_58")
        self.saveHorizonMask = QtWidgets.QPushButton(self.Horizon)
        self.saveHorizonMask.setEnabled(True)
        self.saveHorizonMask.setGeometry(QtCore.QRect(100, 80, 81, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.saveHorizonMask.setFont(font)
        self.saveHorizonMask.setStyleSheet("")
        self.saveHorizonMask.setObjectName("saveHorizonMask")
        self.label_194 = QtWidgets.QLabel(self.Horizon)
        self.label_194.setGeometry(QtCore.QRect(10, 20, 121, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_194.setFont(font)
        self.label_194.setObjectName("label_194")
        self.loadHorizonMask = QtWidgets.QPushButton(self.Horizon)
        self.loadHorizonMask.setGeometry(QtCore.QRect(10, 80, 81, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.loadHorizonMask.setFont(font)
        self.loadHorizonMask.setStyleSheet("")
        self.loadHorizonMask.setObjectName("loadHorizonMask")
        self.settingsTabWidget.addTab(self.Horizon, "")
        self.ParkPos = QtWidgets.QWidget()
        self.ParkPos.setObjectName("ParkPos")
        self.posAlt2 = QtWidgets.QLineEdit(self.ParkPos)
        self.posAlt2.setGeometry(QtCore.QRect(235, 130, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt2.setFont(font)
        self.posAlt2.setMouseTracking(False)
        self.posAlt2.setAcceptDrops(False)
        self.posAlt2.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt2.setReadOnly(False)
        self.posAlt2.setObjectName("posAlt2")
        self.posText2 = QtWidgets.QLineEdit(self.ParkPos)
        self.posText2.setGeometry(QtCore.QRect(80, 130, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText2.setFont(font)
        self.posText2.setMouseTracking(False)
        self.posText2.setAcceptDrops(False)
        self.posText2.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText2.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.posText2.setReadOnly(False)
        self.posText2.setObjectName("posText2")
        self.posText0 = QtWidgets.QLineEdit(self.ParkPos)
        self.posText0.setGeometry(QtCore.QRect(80, 70, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText0.setFont(font)
        self.posText0.setMouseTracking(False)
        self.posText0.setAcceptDrops(False)
        self.posText0.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText0.setReadOnly(False)
        self.posText0.setObjectName("posText0")
        self.label_54 = QtWidgets.QLabel(self.ParkPos)
        self.label_54.setGeometry(QtCore.QRect(10, 20, 256, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_54.setFont(font)
        self.label_54.setObjectName("label_54")
        self.posAz2 = QtWidgets.QLineEdit(self.ParkPos)
        self.posAz2.setGeometry(QtCore.QRect(315, 130, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz2.setFont(font)
        self.posAz2.setMouseTracking(False)
        self.posAz2.setAcceptDrops(False)
        self.posAz2.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz2.setReadOnly(False)
        self.posAz2.setObjectName("posAz2")
        self.posText1 = QtWidgets.QLineEdit(self.ParkPos)
        self.posText1.setGeometry(QtCore.QRect(80, 100, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText1.setFont(font)
        self.posText1.setMouseTracking(False)
        self.posText1.setAcceptDrops(False)
        self.posText1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText1.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.posText1.setReadOnly(False)
        self.posText1.setObjectName("posText1")
        self.posAz0 = QtWidgets.QLineEdit(self.ParkPos)
        self.posAz0.setGeometry(QtCore.QRect(315, 70, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz0.setFont(font)
        self.posAz0.setMouseTracking(False)
        self.posAz0.setAcceptDrops(False)
        self.posAz0.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz0.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz0.setReadOnly(False)
        self.posAz0.setObjectName("posAz0")
        self.posAlt3 = QtWidgets.QLineEdit(self.ParkPos)
        self.posAlt3.setGeometry(QtCore.QRect(235, 160, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt3.setFont(font)
        self.posAlt3.setMouseTracking(False)
        self.posAlt3.setAcceptDrops(False)
        self.posAlt3.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt3.setReadOnly(False)
        self.posAlt3.setObjectName("posAlt3")
        self.posAlt0 = QtWidgets.QLineEdit(self.ParkPos)
        self.posAlt0.setGeometry(QtCore.QRect(235, 70, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt0.setFont(font)
        self.posAlt0.setMouseTracking(False)
        self.posAlt0.setAcceptDrops(False)
        self.posAlt0.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt0.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt0.setReadOnly(False)
        self.posAlt0.setObjectName("posAlt0")
        self.label_2 = QtWidgets.QLabel(self.ParkPos)
        self.label_2.setGeometry(QtCore.QRect(10, 100, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.posAlt1 = QtWidgets.QLineEdit(self.ParkPos)
        self.posAlt1.setGeometry(QtCore.QRect(235, 100, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt1.setFont(font)
        self.posAlt1.setMouseTracking(False)
        self.posAlt1.setAcceptDrops(False)
        self.posAlt1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt1.setReadOnly(False)
        self.posAlt1.setObjectName("posAlt1")
        self.label_50 = QtWidgets.QLabel(self.ParkPos)
        self.label_50.setGeometry(QtCore.QRect(10, 190, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_50.setFont(font)
        self.label_50.setObjectName("label_50")
        self.label_44 = QtWidgets.QLabel(self.ParkPos)
        self.label_44.setGeometry(QtCore.QRect(10, 130, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_44.setFont(font)
        self.label_44.setObjectName("label_44")
        self.posAz1 = QtWidgets.QLineEdit(self.ParkPos)
        self.posAz1.setGeometry(QtCore.QRect(315, 100, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz1.setFont(font)
        self.posAz1.setMouseTracking(False)
        self.posAz1.setAcceptDrops(False)
        self.posAz1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz1.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz1.setReadOnly(False)
        self.posAz1.setObjectName("posAz1")
        self.line_4 = QtWidgets.QFrame(self.ParkPos)
        self.line_4.setGeometry(QtCore.QRect(10, 40, 361, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_4.setFont(font)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setObjectName("line_4")
        self.posAz3 = QtWidgets.QLineEdit(self.ParkPos)
        self.posAz3.setGeometry(QtCore.QRect(315, 160, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz3.setFont(font)
        self.posAz3.setMouseTracking(False)
        self.posAz3.setAcceptDrops(False)
        self.posAz3.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz3.setReadOnly(False)
        self.posAz3.setObjectName("posAz3")
        self.posText3 = QtWidgets.QLineEdit(self.ParkPos)
        self.posText3.setGeometry(QtCore.QRect(80, 160, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText3.setFont(font)
        self.posText3.setMouseTracking(False)
        self.posText3.setAcceptDrops(False)
        self.posText3.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText3.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.posText3.setReadOnly(False)
        self.posText3.setObjectName("posText3")
        self.label_47 = QtWidgets.QLabel(self.ParkPos)
        self.label_47.setGeometry(QtCore.QRect(10, 160, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_47.setFont(font)
        self.label_47.setObjectName("label_47")
        self.posAz4 = QtWidgets.QLineEdit(self.ParkPos)
        self.posAz4.setGeometry(QtCore.QRect(315, 190, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz4.setFont(font)
        self.posAz4.setMouseTracking(False)
        self.posAz4.setAcceptDrops(False)
        self.posAz4.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz4.setReadOnly(False)
        self.posAz4.setObjectName("posAz4")
        self.label_92 = QtWidgets.QLabel(self.ParkPos)
        self.label_92.setGeometry(QtCore.QRect(10, 70, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_92.setFont(font)
        self.label_92.setObjectName("label_92")
        self.posAlt5 = QtWidgets.QLineEdit(self.ParkPos)
        self.posAlt5.setGeometry(QtCore.QRect(235, 220, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt5.setFont(font)
        self.posAlt5.setMouseTracking(False)
        self.posAlt5.setAcceptDrops(False)
        self.posAlt5.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt5.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt5.setReadOnly(False)
        self.posAlt5.setObjectName("posAlt5")
        self.label_94 = QtWidgets.QLabel(self.ParkPos)
        self.label_94.setGeometry(QtCore.QRect(235, 45, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_94.setFont(font)
        self.label_94.setObjectName("label_94")
        self.label_95 = QtWidgets.QLabel(self.ParkPos)
        self.label_95.setGeometry(QtCore.QRect(10, 220, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_95.setFont(font)
        self.label_95.setObjectName("label_95")
        self.posAlt4 = QtWidgets.QLineEdit(self.ParkPos)
        self.posAlt4.setGeometry(QtCore.QRect(235, 190, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt4.setFont(font)
        self.posAlt4.setMouseTracking(False)
        self.posAlt4.setAcceptDrops(False)
        self.posAlt4.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt4.setReadOnly(False)
        self.posAlt4.setObjectName("posAlt4")
        self.posText5 = QtWidgets.QLineEdit(self.ParkPos)
        self.posText5.setGeometry(QtCore.QRect(80, 220, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText5.setFont(font)
        self.posText5.setMouseTracking(False)
        self.posText5.setAcceptDrops(False)
        self.posText5.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText5.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.posText5.setReadOnly(False)
        self.posText5.setObjectName("posText5")
        self.posText4 = QtWidgets.QLineEdit(self.ParkPos)
        self.posText4.setGeometry(QtCore.QRect(80, 190, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText4.setFont(font)
        self.posText4.setMouseTracking(False)
        self.posText4.setAcceptDrops(False)
        self.posText4.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText4.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.posText4.setReadOnly(False)
        self.posText4.setObjectName("posText4")
        self.label_132 = QtWidgets.QLabel(self.ParkPos)
        self.label_132.setGeometry(QtCore.QRect(315, 45, 61, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_132.setFont(font)
        self.label_132.setObjectName("label_132")
        self.posAz5 = QtWidgets.QLineEdit(self.ParkPos)
        self.posAz5.setGeometry(QtCore.QRect(315, 220, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz5.setFont(font)
        self.posAz5.setMouseTracking(False)
        self.posAz5.setAcceptDrops(False)
        self.posAz5.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz5.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz5.setReadOnly(False)
        self.posAz5.setObjectName("posAz5")
        self.line_83 = QtWidgets.QFrame(self.ParkPos)
        self.line_83.setGeometry(QtCore.QRect(380, 10, 11, 371))
        self.line_83.setStyleSheet("")
        self.line_83.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_83.setLineWidth(2)
        self.line_83.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_83.setObjectName("line_83")
        self.label_97 = QtWidgets.QLabel(self.ParkPos)
        self.label_97.setGeometry(QtCore.QRect(80, 50, 51, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_97.setFont(font)
        self.label_97.setObjectName("label_97")
        self.label_93 = QtWidgets.QLabel(self.ParkPos)
        self.label_93.setGeometry(QtCore.QRect(270, 70, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_93.setFont(font)
        self.label_93.setAlignment(QtCore.Qt.AlignCenter)
        self.label_93.setWordWrap(False)
        self.label_93.setObjectName("label_93")
        self.label_131 = QtWidgets.QLabel(self.ParkPos)
        self.label_131.setGeometry(QtCore.QRect(270, 190, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_131.setFont(font)
        self.label_131.setAlignment(QtCore.Qt.AlignCenter)
        self.label_131.setWordWrap(False)
        self.label_131.setObjectName("label_131")
        self.label_209 = QtWidgets.QLabel(self.ParkPos)
        self.label_209.setGeometry(QtCore.QRect(270, 160, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_209.setFont(font)
        self.label_209.setAlignment(QtCore.Qt.AlignCenter)
        self.label_209.setWordWrap(False)
        self.label_209.setObjectName("label_209")
        self.label_243 = QtWidgets.QLabel(self.ParkPos)
        self.label_243.setGeometry(QtCore.QRect(270, 220, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_243.setFont(font)
        self.label_243.setAlignment(QtCore.Qt.AlignCenter)
        self.label_243.setWordWrap(False)
        self.label_243.setObjectName("label_243")
        self.label_244 = QtWidgets.QLabel(self.ParkPos)
        self.label_244.setGeometry(QtCore.QRect(355, 70, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_244.setFont(font)
        self.label_244.setAlignment(QtCore.Qt.AlignCenter)
        self.label_244.setWordWrap(False)
        self.label_244.setObjectName("label_244")
        self.label_245 = QtWidgets.QLabel(self.ParkPos)
        self.label_245.setGeometry(QtCore.QRect(270, 100, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_245.setFont(font)
        self.label_245.setAlignment(QtCore.Qt.AlignCenter)
        self.label_245.setWordWrap(False)
        self.label_245.setObjectName("label_245")
        self.label_246 = QtWidgets.QLabel(self.ParkPos)
        self.label_246.setGeometry(QtCore.QRect(270, 130, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_246.setFont(font)
        self.label_246.setAlignment(QtCore.Qt.AlignCenter)
        self.label_246.setWordWrap(False)
        self.label_246.setObjectName("label_246")
        self.label_247 = QtWidgets.QLabel(self.ParkPos)
        self.label_247.setGeometry(QtCore.QRect(355, 190, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_247.setFont(font)
        self.label_247.setAlignment(QtCore.Qt.AlignCenter)
        self.label_247.setWordWrap(False)
        self.label_247.setObjectName("label_247")
        self.label_248 = QtWidgets.QLabel(self.ParkPos)
        self.label_248.setGeometry(QtCore.QRect(355, 130, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_248.setFont(font)
        self.label_248.setAlignment(QtCore.Qt.AlignCenter)
        self.label_248.setWordWrap(False)
        self.label_248.setObjectName("label_248")
        self.label_249 = QtWidgets.QLabel(self.ParkPos)
        self.label_249.setGeometry(QtCore.QRect(355, 160, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_249.setFont(font)
        self.label_249.setAlignment(QtCore.Qt.AlignCenter)
        self.label_249.setWordWrap(False)
        self.label_249.setObjectName("label_249")
        self.label_250 = QtWidgets.QLabel(self.ParkPos)
        self.label_250.setGeometry(QtCore.QRect(355, 100, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_250.setFont(font)
        self.label_250.setAlignment(QtCore.Qt.AlignCenter)
        self.label_250.setWordWrap(False)
        self.label_250.setObjectName("label_250")
        self.label_251 = QtWidgets.QLabel(self.ParkPos)
        self.label_251.setGeometry(QtCore.QRect(355, 220, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_251.setFont(font)
        self.label_251.setAlignment(QtCore.Qt.AlignCenter)
        self.label_251.setWordWrap(False)
        self.label_251.setObjectName("label_251")
        self.posAlt6 = QtWidgets.QLineEdit(self.ParkPos)
        self.posAlt6.setGeometry(QtCore.QRect(235, 250, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt6.setFont(font)
        self.posAlt6.setMouseTracking(False)
        self.posAlt6.setAcceptDrops(False)
        self.posAlt6.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt6.setReadOnly(False)
        self.posAlt6.setObjectName("posAlt6")
        self.label_257 = QtWidgets.QLabel(self.ParkPos)
        self.label_257.setGeometry(QtCore.QRect(270, 250, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_257.setFont(font)
        self.label_257.setAlignment(QtCore.Qt.AlignCenter)
        self.label_257.setWordWrap(False)
        self.label_257.setObjectName("label_257")
        self.posAz6 = QtWidgets.QLineEdit(self.ParkPos)
        self.posAz6.setGeometry(QtCore.QRect(315, 250, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz6.setFont(font)
        self.posAz6.setMouseTracking(False)
        self.posAz6.setAcceptDrops(False)
        self.posAz6.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz6.setReadOnly(False)
        self.posAz6.setObjectName("posAz6")
        self.posText6 = QtWidgets.QLineEdit(self.ParkPos)
        self.posText6.setGeometry(QtCore.QRect(80, 250, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText6.setFont(font)
        self.posText6.setMouseTracking(False)
        self.posText6.setAcceptDrops(False)
        self.posText6.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText6.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.posText6.setReadOnly(False)
        self.posText6.setObjectName("posText6")
        self.label_101 = QtWidgets.QLabel(self.ParkPos)
        self.label_101.setGeometry(QtCore.QRect(10, 250, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_101.setFont(font)
        self.label_101.setObjectName("label_101")
        self.label_258 = QtWidgets.QLabel(self.ParkPos)
        self.label_258.setGeometry(QtCore.QRect(355, 250, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_258.setFont(font)
        self.label_258.setAlignment(QtCore.Qt.AlignCenter)
        self.label_258.setWordWrap(False)
        self.label_258.setObjectName("label_258")
        self.posAlt7 = QtWidgets.QLineEdit(self.ParkPos)
        self.posAlt7.setGeometry(QtCore.QRect(235, 280, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAlt7.setFont(font)
        self.posAlt7.setMouseTracking(False)
        self.posAlt7.setAcceptDrops(False)
        self.posAlt7.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAlt7.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAlt7.setReadOnly(False)
        self.posAlt7.setObjectName("posAlt7")
        self.label_261 = QtWidgets.QLabel(self.ParkPos)
        self.label_261.setGeometry(QtCore.QRect(270, 280, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_261.setFont(font)
        self.label_261.setAlignment(QtCore.Qt.AlignCenter)
        self.label_261.setWordWrap(False)
        self.label_261.setObjectName("label_261")
        self.posAz7 = QtWidgets.QLineEdit(self.ParkPos)
        self.posAz7.setGeometry(QtCore.QRect(315, 280, 41, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posAz7.setFont(font)
        self.posAz7.setMouseTracking(False)
        self.posAz7.setAcceptDrops(False)
        self.posAz7.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posAz7.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.posAz7.setReadOnly(False)
        self.posAz7.setObjectName("posAz7")
        self.posText7 = QtWidgets.QLineEdit(self.ParkPos)
        self.posText7.setGeometry(QtCore.QRect(80, 280, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.posText7.setFont(font)
        self.posText7.setMouseTracking(False)
        self.posText7.setAcceptDrops(False)
        self.posText7.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.posText7.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.posText7.setReadOnly(False)
        self.posText7.setObjectName("posText7")
        self.label_103 = QtWidgets.QLabel(self.ParkPos)
        self.label_103.setGeometry(QtCore.QRect(10, 280, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_103.setFont(font)
        self.label_103.setObjectName("label_103")
        self.label_262 = QtWidgets.QLabel(self.ParkPos)
        self.label_262.setGeometry(QtCore.QRect(355, 280, 16, 20))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_262.setFont(font)
        self.label_262.setAlignment(QtCore.Qt.AlignCenter)
        self.label_262.setWordWrap(False)
        self.label_262.setObjectName("label_262")
        self.posAlt2.raise_()
        self.posText2.raise_()
        self.posText0.raise_()
        self.label_54.raise_()
        self.posAz2.raise_()
        self.posText1.raise_()
        self.posAz0.raise_()
        self.posAlt3.raise_()
        self.posAlt0.raise_()
        self.label_2.raise_()
        self.posAlt1.raise_()
        self.label_50.raise_()
        self.label_44.raise_()
        self.posAz1.raise_()
        self.posAz3.raise_()
        self.posText3.raise_()
        self.label_47.raise_()
        self.posAz4.raise_()
        self.label_92.raise_()
        self.posAlt5.raise_()
        self.label_94.raise_()
        self.label_95.raise_()
        self.posAlt4.raise_()
        self.posText5.raise_()
        self.posText4.raise_()
        self.label_132.raise_()
        self.posAz5.raise_()
        self.line_83.raise_()
        self.label_97.raise_()
        self.line_4.raise_()
        self.label_93.raise_()
        self.label_131.raise_()
        self.label_209.raise_()
        self.label_243.raise_()
        self.label_244.raise_()
        self.label_245.raise_()
        self.label_246.raise_()
        self.label_247.raise_()
        self.label_248.raise_()
        self.label_249.raise_()
        self.label_250.raise_()
        self.label_251.raise_()
        self.posAlt6.raise_()
        self.label_257.raise_()
        self.posAz6.raise_()
        self.posText6.raise_()
        self.label_101.raise_()
        self.label_258.raise_()
        self.posAlt7.raise_()
        self.label_261.raise_()
        self.posAz7.raise_()
        self.posText7.raise_()
        self.label_103.raise_()
        self.label_262.raise_()
        self.settingsTabWidget.addTab(self.ParkPos, "")
        self.KMTronic = QtWidgets.QWidget()
        self.KMTronic.setObjectName("KMTronic")
        self.label_129 = QtWidgets.QLabel(self.KMTronic)
        self.label_129.setGeometry(QtCore.QRect(410, 130, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_129.setFont(font)
        self.label_129.setObjectName("label_129")
        self.relayFun5 = QtWidgets.QComboBox(self.KMTronic)
        self.relayFun5.setGeometry(QtCore.QRect(600, 220, 161, 21))
        self.relayFun5.setObjectName("relayFun5")
        self.label_56 = QtWidgets.QLabel(self.KMTronic)
        self.label_56.setGeometry(QtCore.QRect(405, 50, 51, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_56.setFont(font)
        self.label_56.setObjectName("label_56")
        self.label_198 = QtWidgets.QLabel(self.KMTronic)
        self.label_198.setGeometry(QtCore.QRect(600, 50, 51, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_198.setFont(font)
        self.label_198.setObjectName("label_198")
        self.label_199 = QtWidgets.QLabel(self.KMTronic)
        self.label_199.setGeometry(QtCore.QRect(410, 250, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_199.setFont(font)
        self.label_199.setObjectName("label_199")
        self.relayFun4 = QtWidgets.QComboBox(self.KMTronic)
        self.relayFun4.setGeometry(QtCore.QRect(600, 190, 161, 21))
        self.relayFun4.setObjectName("relayFun4")
        self.relayFun1 = QtWidgets.QComboBox(self.KMTronic)
        self.relayFun1.setGeometry(QtCore.QRect(600, 100, 161, 21))
        self.relayFun1.setObjectName("relayFun1")
        self.relayButtonText0 = QtWidgets.QLineEdit(self.KMTronic)
        self.relayButtonText0.setGeometry(QtCore.QRect(455, 70, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayButtonText0.setFont(font)
        self.relayButtonText0.setMouseTracking(False)
        self.relayButtonText0.setAcceptDrops(False)
        self.relayButtonText0.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayButtonText0.setReadOnly(False)
        self.relayButtonText0.setObjectName("relayButtonText0")
        self.relayButtonText7 = QtWidgets.QLineEdit(self.KMTronic)
        self.relayButtonText7.setGeometry(QtCore.QRect(455, 280, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayButtonText7.setFont(font)
        self.relayButtonText7.setMouseTracking(False)
        self.relayButtonText7.setAcceptDrops(False)
        self.relayButtonText7.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayButtonText7.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayButtonText7.setReadOnly(False)
        self.relayButtonText7.setObjectName("relayButtonText7")
        self.label_211 = QtWidgets.QLabel(self.KMTronic)
        self.label_211.setGeometry(QtCore.QRect(410, 100, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_211.setFont(font)
        self.label_211.setObjectName("label_211")
        self.line_85 = QtWidgets.QFrame(self.KMTronic)
        self.line_85.setGeometry(QtCore.QRect(380, 10, 11, 371))
        self.line_85.setStyleSheet("")
        self.line_85.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_85.setLineWidth(2)
        self.line_85.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_85.setObjectName("line_85")
        self.line_23 = QtWidgets.QFrame(self.KMTronic)
        self.line_23.setGeometry(QtCore.QRect(400, 40, 361, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_23.setFont(font)
        self.line_23.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_23.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_23.setObjectName("line_23")
        self.relayButtonText5 = QtWidgets.QLineEdit(self.KMTronic)
        self.relayButtonText5.setGeometry(QtCore.QRect(455, 220, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayButtonText5.setFont(font)
        self.relayButtonText5.setMouseTracking(False)
        self.relayButtonText5.setAcceptDrops(False)
        self.relayButtonText5.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayButtonText5.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayButtonText5.setReadOnly(False)
        self.relayButtonText5.setObjectName("relayButtonText5")
        self.label_220 = QtWidgets.QLabel(self.KMTronic)
        self.label_220.setGeometry(QtCore.QRect(410, 160, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_220.setFont(font)
        self.label_220.setObjectName("label_220")
        self.label_146 = QtWidgets.QLabel(self.KMTronic)
        self.label_146.setGeometry(QtCore.QRect(400, 20, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_146.setFont(font)
        self.label_146.setObjectName("label_146")
        self.relayFun3 = QtWidgets.QComboBox(self.KMTronic)
        self.relayFun3.setGeometry(QtCore.QRect(600, 160, 161, 21))
        self.relayFun3.setObjectName("relayFun3")
        self.label_255 = QtWidgets.QLabel(self.KMTronic)
        self.label_255.setGeometry(QtCore.QRect(410, 280, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_255.setFont(font)
        self.label_255.setObjectName("label_255")
        self.relayButtonText1 = QtWidgets.QLineEdit(self.KMTronic)
        self.relayButtonText1.setGeometry(QtCore.QRect(455, 100, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayButtonText1.setFont(font)
        self.relayButtonText1.setMouseTracking(False)
        self.relayButtonText1.setAcceptDrops(False)
        self.relayButtonText1.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayButtonText1.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayButtonText1.setReadOnly(False)
        self.relayButtonText1.setObjectName("relayButtonText1")
        self.relayFun0 = QtWidgets.QComboBox(self.KMTronic)
        self.relayFun0.setGeometry(QtCore.QRect(600, 70, 161, 21))
        self.relayFun0.setObjectName("relayFun0")
        self.label_256 = QtWidgets.QLabel(self.KMTronic)
        self.label_256.setGeometry(QtCore.QRect(410, 220, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_256.setFont(font)
        self.label_256.setObjectName("label_256")
        self.relayButtonText6 = QtWidgets.QLineEdit(self.KMTronic)
        self.relayButtonText6.setGeometry(QtCore.QRect(455, 250, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayButtonText6.setFont(font)
        self.relayButtonText6.setMouseTracking(False)
        self.relayButtonText6.setAcceptDrops(False)
        self.relayButtonText6.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayButtonText6.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayButtonText6.setReadOnly(False)
        self.relayButtonText6.setObjectName("relayButtonText6")
        self.label_149 = QtWidgets.QLabel(self.KMTronic)
        self.label_149.setGeometry(QtCore.QRect(410, 190, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_149.setFont(font)
        self.label_149.setObjectName("label_149")
        self.relayButtonText3 = QtWidgets.QLineEdit(self.KMTronic)
        self.relayButtonText3.setGeometry(QtCore.QRect(455, 160, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayButtonText3.setFont(font)
        self.relayButtonText3.setMouseTracking(False)
        self.relayButtonText3.setAcceptDrops(False)
        self.relayButtonText3.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayButtonText3.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayButtonText3.setReadOnly(False)
        self.relayButtonText3.setObjectName("relayButtonText3")
        self.relayFun2 = QtWidgets.QComboBox(self.KMTronic)
        self.relayFun2.setGeometry(QtCore.QRect(600, 130, 161, 21))
        self.relayFun2.setObjectName("relayFun2")
        self.relayFun7 = QtWidgets.QComboBox(self.KMTronic)
        self.relayFun7.setGeometry(QtCore.QRect(600, 280, 161, 21))
        self.relayFun7.setObjectName("relayFun7")
        self.label_43 = QtWidgets.QLabel(self.KMTronic)
        self.label_43.setGeometry(QtCore.QRect(410, 70, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_43.setFont(font)
        self.label_43.setObjectName("label_43")
        self.relayFun6 = QtWidgets.QComboBox(self.KMTronic)
        self.relayFun6.setGeometry(QtCore.QRect(600, 250, 161, 21))
        self.relayFun6.setObjectName("relayFun6")
        self.relayButtonText2 = QtWidgets.QLineEdit(self.KMTronic)
        self.relayButtonText2.setGeometry(QtCore.QRect(455, 130, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayButtonText2.setFont(font)
        self.relayButtonText2.setMouseTracking(False)
        self.relayButtonText2.setAcceptDrops(False)
        self.relayButtonText2.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayButtonText2.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayButtonText2.setReadOnly(False)
        self.relayButtonText2.setObjectName("relayButtonText2")
        self.label_45 = QtWidgets.QLabel(self.KMTronic)
        self.label_45.setGeometry(QtCore.QRect(455, 50, 51, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_45.setFont(font)
        self.label_45.setObjectName("label_45")
        self.relayButtonText4 = QtWidgets.QLineEdit(self.KMTronic)
        self.relayButtonText4.setGeometry(QtCore.QRect(455, 190, 121, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayButtonText4.setFont(font)
        self.relayButtonText4.setMouseTracking(False)
        self.relayButtonText4.setAcceptDrops(False)
        self.relayButtonText4.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayButtonText4.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayButtonText4.setReadOnly(False)
        self.relayButtonText4.setObjectName("relayButtonText4")
        self.relayPassword = QtWidgets.QLineEdit(self.KMTronic)
        self.relayPassword.setGeometry(QtCore.QRect(215, 100, 151, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayPassword.setFont(font)
        self.relayPassword.setMouseTracking(False)
        self.relayPassword.setAcceptDrops(False)
        self.relayPassword.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayPassword.setMaxLength(15)
        self.relayPassword.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.relayPassword.setDragEnabled(True)
        self.relayPassword.setReadOnly(False)
        self.relayPassword.setObjectName("relayPassword")
        self.line_48 = QtWidgets.QFrame(self.KMTronic)
        self.line_48.setGeometry(QtCore.QRect(10, 40, 361, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_48.setFont(font)
        self.line_48.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_48.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_48.setObjectName("line_48")
        self.label_196 = QtWidgets.QLabel(self.KMTronic)
        self.label_196.setGeometry(QtCore.QRect(15, 75, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_196.setFont(font)
        self.label_196.setObjectName("label_196")
        self.label_mountIP_2 = QtWidgets.QLabel(self.KMTronic)
        self.label_mountIP_2.setGeometry(QtCore.QRect(15, 50, 71, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_mountIP_2.setFont(font)
        self.label_mountIP_2.setObjectName("label_mountIP_2")
        self.label_197 = QtWidgets.QLabel(self.KMTronic)
        self.label_197.setGeometry(QtCore.QRect(15, 100, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_197.setFont(font)
        self.label_197.setObjectName("label_197")
        self.label_141 = QtWidgets.QLabel(self.KMTronic)
        self.label_141.setGeometry(QtCore.QRect(10, 20, 306, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_141.setFont(font)
        self.label_141.setObjectName("label_141")
        self.relayUser = QtWidgets.QLineEdit(self.KMTronic)
        self.relayUser.setGeometry(QtCore.QRect(215, 75, 151, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayUser.setFont(font)
        self.relayUser.setMouseTracking(False)
        self.relayUser.setAcceptDrops(False)
        self.relayUser.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayUser.setMaxLength(15)
        self.relayUser.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.relayUser.setDragEnabled(True)
        self.relayUser.setReadOnly(False)
        self.relayUser.setObjectName("relayUser")
        self.relayHost = QtWidgets.QLineEdit(self.KMTronic)
        self.relayHost.setGeometry(QtCore.QRect(115, 50, 251, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayHost.setFont(font)
        self.relayHost.setMouseTracking(False)
        self.relayHost.setAcceptDrops(False)
        self.relayHost.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayHost.setMaxLength(50)
        self.relayHost.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.relayHost.setDragEnabled(True)
        self.relayHost.setReadOnly(False)
        self.relayHost.setObjectName("relayHost")
        self.settingsTabWidget.addTab(self.KMTronic, "")
        self.AudioMisc = QtWidgets.QWidget()
        self.AudioMisc.setObjectName("AudioMisc")
        self.soundMountSlewFinished = QtWidgets.QComboBox(self.AudioMisc)
        self.soundMountSlewFinished.setGeometry(QtCore.QRect(580, 245, 181, 21))
        self.soundMountSlewFinished.setObjectName("soundMountSlewFinished")
        self.label_210 = QtWidgets.QLabel(self.AudioMisc)
        self.label_210.setGeometry(QtCore.QRect(400, 215, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_210.setFont(font)
        self.label_210.setObjectName("label_210")
        self.line_24 = QtWidgets.QFrame(self.AudioMisc)
        self.line_24.setGeometry(QtCore.QRect(400, 235, 361, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_24.setFont(font)
        self.line_24.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_24.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_24.setObjectName("line_24")
        self.line_84 = QtWidgets.QFrame(self.AudioMisc)
        self.line_84.setGeometry(QtCore.QRect(380, 10, 11, 381))
        self.line_84.setStyleSheet("")
        self.line_84.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_84.setLineWidth(2)
        self.line_84.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_84.setObjectName("line_84")
        self.soundMountAlert = QtWidgets.QComboBox(self.AudioMisc)
        self.soundMountAlert.setGeometry(QtCore.QRect(580, 295, 181, 21))
        self.soundMountAlert.setObjectName("soundMountAlert")
        self.soundDomeSlewFinished = QtWidgets.QComboBox(self.AudioMisc)
        self.soundDomeSlewFinished.setGeometry(QtCore.QRect(580, 270, 181, 21))
        self.soundDomeSlewFinished.setObjectName("soundDomeSlewFinished")
        self.label_212 = QtWidgets.QLabel(self.AudioMisc)
        self.label_212.setGeometry(QtCore.QRect(400, 245, 136, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_212.setFont(font)
        self.label_212.setObjectName("label_212")
        self.label_214 = QtWidgets.QLabel(self.AudioMisc)
        self.label_214.setGeometry(QtCore.QRect(400, 270, 136, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_214.setFont(font)
        self.label_214.setObjectName("label_214")
        self.label_215 = QtWidgets.QLabel(self.AudioMisc)
        self.label_215.setGeometry(QtCore.QRect(400, 295, 136, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_215.setFont(font)
        self.label_215.setObjectName("label_215")
        self.label_217 = QtWidgets.QLabel(self.AudioMisc)
        self.label_217.setGeometry(QtCore.QRect(400, 320, 136, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_217.setFont(font)
        self.label_217.setObjectName("label_217")
        self.soundModelingFinished = QtWidgets.QComboBox(self.AudioMisc)
        self.soundModelingFinished.setGeometry(QtCore.QRect(580, 320, 181, 21))
        self.soundModelingFinished.setObjectName("soundModelingFinished")
        self.logLevelMW = QtWidgets.QGroupBox(self.AudioMisc)
        self.logLevelMW.setGeometry(QtCore.QRect(400, 40, 356, 101))
        self.logLevelMW.setObjectName("logLevelMW")
        self.loglevelDebug = QtWidgets.QRadioButton(self.logLevelMW)
        self.loglevelDebug.setGeometry(QtCore.QRect(15, 15, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.loglevelDebug.setFont(font)
        self.loglevelDebug.setChecked(True)
        self.loglevelDebug.setObjectName("loglevelDebug")
        self.loglevelInfo = QtWidgets.QRadioButton(self.logLevelMW)
        self.loglevelInfo.setGeometry(QtCore.QRect(15, 35, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.loglevelInfo.setFont(font)
        self.loglevelInfo.setChecked(False)
        self.loglevelInfo.setObjectName("loglevelInfo")
        self.loglevelWarning = QtWidgets.QRadioButton(self.logLevelMW)
        self.loglevelWarning.setGeometry(QtCore.QRect(15, 55, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.loglevelWarning.setFont(font)
        self.loglevelWarning.setChecked(False)
        self.loglevelWarning.setObjectName("loglevelWarning")
        self.loglevelError = QtWidgets.QRadioButton(self.logLevelMW)
        self.loglevelError.setGeometry(QtCore.QRect(15, 75, 81, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.loglevelError.setFont(font)
        self.loglevelError.setChecked(False)
        self.loglevelError.setObjectName("loglevelError")
        self.label_59 = QtWidgets.QLabel(self.AudioMisc)
        self.label_59.setGeometry(QtCore.QRect(400, 15, 161, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_59.setFont(font)
        self.label_59.setObjectName("label_59")
        self.btn_resetWindowSizes = QtWidgets.QPushButton(self.AudioMisc)
        self.btn_resetWindowSizes.setGeometry(QtCore.QRect(10, 110, 176, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.btn_resetWindowSizes.setFont(font)
        self.btn_resetWindowSizes.setStyleSheet("")
        self.btn_resetWindowSizes.setObjectName("btn_resetWindowSizes")
        self.line_9 = QtWidgets.QFrame(self.AudioMisc)
        self.line_9.setGeometry(QtCore.QRect(400, 35, 361, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_9.setFont(font)
        self.line_9.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_9.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_9.setObjectName("line_9")
        self.label_62 = QtWidgets.QLabel(self.AudioMisc)
        self.label_62.setGeometry(QtCore.QRect(400, 155, 196, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_62.setFont(font)
        self.label_62.setObjectName("label_62")
        self.line_17 = QtWidgets.QFrame(self.AudioMisc)
        self.line_17.setGeometry(QtCore.QRect(400, 175, 361, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_17.setFont(font)
        self.line_17.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_17.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_17.setObjectName("line_17")
        self.label_401 = QtWidgets.QLabel(self.AudioMisc)
        self.label_401.setGeometry(QtCore.QRect(10, 15, 161, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_401.setFont(font)
        self.label_401.setObjectName("label_401")
        self.firmwareProductName_3 = QtWidgets.QLabel(self.AudioMisc)
        self.firmwareProductName_3.setGeometry(QtCore.QRect(10, 65, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.firmwareProductName_3.setFont(font)
        self.firmwareProductName_3.setObjectName("firmwareProductName_3")
        self.numberString = QtWidgets.QLineEdit(self.AudioMisc)
        self.numberString.setEnabled(False)
        self.numberString.setGeometry(QtCore.QRect(65, 65, 56, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.numberString.setFont(font)
        self.numberString.setMouseTracking(False)
        self.numberString.setFocusPolicy(QtCore.Qt.NoFocus)
        self.numberString.setAcceptDrops(False)
        self.numberString.setToolTip("")
        self.numberString.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.numberString.setMaxLength(8)
        self.numberString.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.numberString.setReadOnly(True)
        self.numberString.setObjectName("numberString")
        self.firmwareProductName_4 = QtWidgets.QLabel(self.AudioMisc)
        self.firmwareProductName_4.setGeometry(QtCore.QRect(140, 65, 36, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.firmwareProductName_4.setFont(font)
        self.firmwareProductName_4.setObjectName("firmwareProductName_4")
        self.fwtime = QtWidgets.QLineEdit(self.AudioMisc)
        self.fwtime.setEnabled(False)
        self.fwtime.setGeometry(QtCore.QRect(310, 65, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.fwtime.setFont(font)
        self.fwtime.setMouseTracking(False)
        self.fwtime.setFocusPolicy(QtCore.Qt.NoFocus)
        self.fwtime.setAcceptDrops(False)
        self.fwtime.setToolTip("")
        self.fwtime.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.fwtime.setMaxLength(8)
        self.fwtime.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.fwtime.setReadOnly(True)
        self.fwtime.setObjectName("fwtime")
        self.firmwareProductName_5 = QtWidgets.QLabel(self.AudioMisc)
        self.firmwareProductName_5.setGeometry(QtCore.QRect(275, 65, 36, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.firmwareProductName_5.setFont(font)
        self.firmwareProductName_5.setObjectName("firmwareProductName_5")
        self.productName = QtWidgets.QLineEdit(self.AudioMisc)
        self.productName.setEnabled(False)
        self.productName.setGeometry(QtCore.QRect(65, 40, 166, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.productName.setFont(font)
        self.productName.setMouseTracking(False)
        self.productName.setFocusPolicy(QtCore.Qt.NoFocus)
        self.productName.setAcceptDrops(False)
        self.productName.setToolTip("")
        self.productName.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.productName.setMaxLength(25)
        self.productName.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.productName.setReadOnly(True)
        self.productName.setObjectName("productName")
        self.firmwareProductName_2 = QtWidgets.QLabel(self.AudioMisc)
        self.firmwareProductName_2.setGeometry(QtCore.QRect(10, 40, 56, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.firmwareProductName_2.setFont(font)
        self.firmwareProductName_2.setObjectName("firmwareProductName_2")
        self.hwVersion = QtWidgets.QLineEdit(self.AudioMisc)
        self.hwVersion.setEnabled(False)
        self.hwVersion.setGeometry(QtCore.QRect(275, 40, 96, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.hwVersion.setFont(font)
        self.hwVersion.setMouseTracking(False)
        self.hwVersion.setFocusPolicy(QtCore.Qt.NoFocus)
        self.hwVersion.setAcceptDrops(False)
        self.hwVersion.setToolTip("")
        self.hwVersion.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.hwVersion.setMaxLength(25)
        self.hwVersion.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.hwVersion.setReadOnly(True)
        self.hwVersion.setObjectName("hwVersion")
        self.fwdate = QtWidgets.QLineEdit(self.AudioMisc)
        self.fwdate.setEnabled(False)
        self.fwdate.setGeometry(QtCore.QRect(175, 65, 86, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.fwdate.setFont(font)
        self.fwdate.setMouseTracking(False)
        self.fwdate.setFocusPolicy(QtCore.Qt.NoFocus)
        self.fwdate.setAcceptDrops(False)
        self.fwdate.setToolTip("")
        self.fwdate.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.fwdate.setMaxLength(25)
        self.fwdate.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.fwdate.setReadOnly(True)
        self.fwdate.setObjectName("fwdate")
        self.line_116 = QtWidgets.QFrame(self.AudioMisc)
        self.line_116.setGeometry(QtCore.QRect(10, 35, 361, 1))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.line_116.setFont(font)
        self.line_116.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_116.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_116.setObjectName("line_116")
        self.Hardware = QtWidgets.QLabel(self.AudioMisc)
        self.Hardware.setGeometry(QtCore.QRect(245, 40, 31, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.Hardware.setFont(font)
        self.Hardware.setObjectName("Hardware")
        self.isOnline = QtWidgets.QCheckBox(self.AudioMisc)
        self.isOnline.setGeometry(QtCore.QRect(405, 180, 331, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.isOnline.setFont(font)
        self.isOnline.setChecked(False)
        self.isOnline.setObjectName("isOnline")
        self.enableNoise = QtWidgets.QCheckBox(self.AudioMisc)
        self.enableNoise.setGeometry(QtCore.QRect(15, 220, 331, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.enableNoise.setFont(font)
        self.enableNoise.setChecked(False)
        self.enableNoise.setObjectName("enableNoise")
        self.groupBox = QtWidgets.QGroupBox(self.AudioMisc)
        self.groupBox.setGeometry(QtCore.QRect(15, 290, 351, 91))
        self.groupBox.setObjectName("groupBox")
        self.le_firmwareProductName_6 = QtWidgets.QLabel(self.groupBox)
        self.le_firmwareProductName_6.setGeometry(QtCore.QRect(5, 65, 341, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        self.le_firmwareProductName_6.setFont(font)
        self.le_firmwareProductName_6.setObjectName("le_firmwareProductName_6")
        self.le_firmwareProductName_3 = QtWidgets.QLabel(self.groupBox)
        self.le_firmwareProductName_3.setGeometry(QtCore.QRect(5, 20, 341, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        self.le_firmwareProductName_3.setFont(font)
        self.le_firmwareProductName_3.setObjectName("le_firmwareProductName_3")
        self.le_firmwareProductName_4 = QtWidgets.QLabel(self.groupBox)
        self.le_firmwareProductName_4.setGeometry(QtCore.QRect(5, 35, 341, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        self.le_firmwareProductName_4.setFont(font)
        self.le_firmwareProductName_4.setObjectName("le_firmwareProductName_4")
        self.le_firmwareProductName_5 = QtWidgets.QLabel(self.groupBox)
        self.le_firmwareProductName_5.setGeometry(QtCore.QRect(5, 50, 341, 21))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        self.le_firmwareProductName_5.setFont(font)
        self.le_firmwareProductName_5.setObjectName("le_firmwareProductName_5")
        self.settingsTabWidget.addTab(self.AudioMisc, "")
        self.mainTabWidget.addTab(self.Setting, "")
        self.verticalLayout.addWidget(self.mainTabWidget)
        self.verticalLayout.setStretch(3, 1)

        self.retranslateUi(MainWindow)
        self.mainTabWidget.setCurrentIndex(10)
        self.settingsTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        MainWindow.setTabOrder(self.powerCurrent1, self.powerCurrent2)
        MainWindow.setTabOrder(self.powerCurrent2, self.dewB)
        MainWindow.setTabOrder(self.dewB, self.tracking)
        MainWindow.setTabOrder(self.tracking, self.flipMount)
        MainWindow.setTabOrder(self.flipMount, self.setLunarTracking)
        MainWindow.setTabOrder(self.setLunarTracking, self.setSolarTracking)
        MainWindow.setTabOrder(self.setSolarTracking, self.setSiderealTracking)
        MainWindow.setTabOrder(self.setSiderealTracking, self.park)
        MainWindow.setTabOrder(self.park, self.stop)
        MainWindow.setTabOrder(self.stop, self.checkRefracNone)
        MainWindow.setTabOrder(self.checkRefracNone, self.checkRefracCont)
        MainWindow.setTabOrder(self.checkRefracCont, self.checkRefracNoTrack)
        MainWindow.setTabOrder(self.checkRefracNoTrack, self.setRefractionManual)
        MainWindow.setTabOrder(self.setRefractionManual, self.deleteWorstPoint)
        MainWindow.setTabOrder(self.deleteWorstPoint, self.clearModel)
        MainWindow.setTabOrder(self.clearModel, self.refreshModel)
        MainWindow.setTabOrder(self.refreshModel, self.runTargetRMS)
        MainWindow.setTabOrder(self.runTargetRMS, self.cancelTargetRMS)
        MainWindow.setTabOrder(self.cancelTargetRMS, self.targetRMS)
        MainWindow.setTabOrder(self.targetRMS, self.checkShowErrorValues)
        MainWindow.setTabOrder(self.checkShowErrorValues, self.nameList)
        MainWindow.setTabOrder(self.nameList, self.loadName)
        MainWindow.setTabOrder(self.loadName, self.saveName)
        MainWindow.setTabOrder(self.saveName, self.deleteName)
        MainWindow.setTabOrder(self.deleteName, self.refreshName)
        MainWindow.setTabOrder(self.refreshName, self.cancelAnalyse)
        MainWindow.setTabOrder(self.cancelAnalyse, self.relayButton1)
        MainWindow.setTabOrder(self.relayButton1, self.relayButton2)
        MainWindow.setTabOrder(self.relayButton2, self.relayButton3)
        MainWindow.setTabOrder(self.relayButton3, self.relayButton4)
        MainWindow.setTabOrder(self.relayButton4, self.relayButton5)
        MainWindow.setTabOrder(self.relayButton5, self.relayButton6)
        MainWindow.setTabOrder(self.relayButton6, self.relayButton7)
        MainWindow.setTabOrder(self.relayButton7, self.expTime)
        MainWindow.setTabOrder(self.expTime, self.binning)
        MainWindow.setTabOrder(self.binning, self.subFrame)
        MainWindow.setTabOrder(self.subFrame, self.checkFastDownload)
        MainWindow.setTabOrder(self.checkFastDownload, self.isoSetting)
        MainWindow.setTabOrder(self.isoSetting, self.settleTimeMount)
        MainWindow.setTabOrder(self.settleTimeMount, self.checkKeepImages)
        MainWindow.setTabOrder(self.checkKeepImages, self.mountHost)
        MainWindow.setTabOrder(self.mountHost, self.mountMAC)
        MainWindow.setTabOrder(self.mountMAC, self.mountTypeConnection)
        MainWindow.setTabOrder(self.mountTypeConnection, self.loadHorizonMask)
        MainWindow.setTabOrder(self.loadHorizonMask, self.saveHorizonMask)
        MainWindow.setTabOrder(self.saveHorizonMask, self.saveHorizonMaskAs)
        MainWindow.setTabOrder(self.saveHorizonMaskAs, self.checkAutoDeletePoints)
        MainWindow.setTabOrder(self.checkAutoDeletePoints, self.satelliteName)
        MainWindow.setTabOrder(self.satelliteName, self.checkSortNothing)
        MainWindow.setTabOrder(self.checkSortNothing, self.checkSortEW)
        MainWindow.setTabOrder(self.checkSortEW, self.checkSortHL)
        MainWindow.setTabOrder(self.checkSortHL, self.posText0)
        MainWindow.setTabOrder(self.posText0, self.posAlt0)
        MainWindow.setTabOrder(self.posAlt0, self.posAz0)
        MainWindow.setTabOrder(self.posAz0, self.posText1)
        MainWindow.setTabOrder(self.posText1, self.posAlt1)
        MainWindow.setTabOrder(self.posAlt1, self.posAz1)
        MainWindow.setTabOrder(self.posAz1, self.posText2)
        MainWindow.setTabOrder(self.posText2, self.posAlt2)
        MainWindow.setTabOrder(self.posAlt2, self.posAz2)
        MainWindow.setTabOrder(self.posAz2, self.posText3)
        MainWindow.setTabOrder(self.posText3, self.posAlt3)
        MainWindow.setTabOrder(self.posAlt3, self.posAz3)
        MainWindow.setTabOrder(self.posAz3, self.posText4)
        MainWindow.setTabOrder(self.posText4, self.posAlt4)
        MainWindow.setTabOrder(self.posAlt4, self.posAz4)
        MainWindow.setTabOrder(self.posAz4, self.posText5)
        MainWindow.setTabOrder(self.posText5, self.posAlt5)
        MainWindow.setTabOrder(self.posAlt5, self.posAz5)
        MainWindow.setTabOrder(self.posAz5, self.posText6)
        MainWindow.setTabOrder(self.posText6, self.posAlt6)
        MainWindow.setTabOrder(self.posAlt6, self.posAz6)
        MainWindow.setTabOrder(self.posAz6, self.posText7)
        MainWindow.setTabOrder(self.posText7, self.posAlt7)
        MainWindow.setTabOrder(self.posAlt7, self.posAz7)
        MainWindow.setTabOrder(self.posAz7, self.relayHost)
        MainWindow.setTabOrder(self.relayHost, self.relayUser)
        MainWindow.setTabOrder(self.relayUser, self.relayPassword)
        MainWindow.setTabOrder(self.relayPassword, self.relayButtonText0)
        MainWindow.setTabOrder(self.relayButtonText0, self.relayFun0)
        MainWindow.setTabOrder(self.relayFun0, self.relayButtonText1)
        MainWindow.setTabOrder(self.relayButtonText1, self.relayFun1)
        MainWindow.setTabOrder(self.relayFun1, self.relayButtonText2)
        MainWindow.setTabOrder(self.relayButtonText2, self.relayFun2)
        MainWindow.setTabOrder(self.relayFun2, self.relayButtonText3)
        MainWindow.setTabOrder(self.relayButtonText3, self.relayFun3)
        MainWindow.setTabOrder(self.relayFun3, self.relayButtonText4)
        MainWindow.setTabOrder(self.relayButtonText4, self.relayFun4)
        MainWindow.setTabOrder(self.relayFun4, self.relayButtonText5)
        MainWindow.setTabOrder(self.relayButtonText5, self.relayFun5)
        MainWindow.setTabOrder(self.relayFun5, self.relayButtonText6)
        MainWindow.setTabOrder(self.relayButtonText6, self.relayFun6)
        MainWindow.setTabOrder(self.relayFun6, self.relayButtonText7)
        MainWindow.setTabOrder(self.relayButtonText7, self.relayFun7)
        MainWindow.setTabOrder(self.relayFun7, self.soundMountSlewFinished)
        MainWindow.setTabOrder(self.soundMountSlewFinished, self.soundDomeSlewFinished)
        MainWindow.setTabOrder(self.soundDomeSlewFinished, self.soundMountAlert)
        MainWindow.setTabOrder(self.soundMountAlert, self.soundModelingFinished)
        MainWindow.setTabOrder(self.soundModelingFinished, self.btn_resetWindowSizes)
        MainWindow.setTabOrder(self.btn_resetWindowSizes, self.loglevelDebug)
        MainWindow.setTabOrder(self.loglevelDebug, self.loglevelInfo)
        MainWindow.setTabOrder(self.loglevelInfo, self.loglevelWarning)
        MainWindow.setTabOrder(self.loglevelWarning, self.loglevelError)
        MainWindow.setTabOrder(self.loglevelError, self.loadFrom)
        MainWindow.setTabOrder(self.loadFrom, self.saveConfigQuit)
        MainWindow.setTabOrder(self.saveConfigQuit, self.saveConfig)
        MainWindow.setTabOrder(self.saveConfig, self.saveConfigAs)
        MainWindow.setTabOrder(self.saveConfigAs, self.mountOn)
        MainWindow.setTabOrder(self.mountOn, self.mountOff)
        MainWindow.setTabOrder(self.mountOff, self.startSatelliteTracking)
        MainWindow.setTabOrder(self.startSatelliteTracking, self.imagingConnected)
        MainWindow.setTabOrder(self.imagingConnected, self.openImageW)
        MainWindow.setTabOrder(self.openImageW, self.astrometryConnected)
        MainWindow.setTabOrder(self.astrometryConnected, self.environConnected)
        MainWindow.setTabOrder(self.environConnected, self.domeConnected)
        MainWindow.setTabOrder(self.domeConnected, self.openSatelliteW)
        MainWindow.setTabOrder(self.openSatelliteW, self.openMeasureW)
        MainWindow.setTabOrder(self.openMeasureW, self.openHemisphereW)
        MainWindow.setTabOrder(self.openHemisphereW, self.openMessageW)
        MainWindow.setTabOrder(self.openMessageW, self.mountConnected)
        MainWindow.setTabOrder(self.mountConnected, self.satelliteEpoch)
        MainWindow.setTabOrder(self.satelliteEpoch, self.settingsTabWidget)
        MainWindow.setTabOrder(self.settingsTabWidget, self.listSatelliteNames)
        MainWindow.setTabOrder(self.listSatelliteNames, self.mainTabWidget)
        MainWindow.setTabOrder(self.mainTabWidget, self.relayButton0)
        MainWindow.setTabOrder(self.relayButton0, self.environHost)
        MainWindow.setTabOrder(self.environHost, self.checkJ2000)
        MainWindow.setTabOrder(self.checkJ2000, self.powerPort2)
        MainWindow.setTabOrder(self.powerPort2, self.indiMessage)
        MainWindow.setTabOrder(self.indiMessage, self.imagingHost)
        MainWindow.setTabOrder(self.imagingHost, self.powerPort1)
        MainWindow.setTabOrder(self.powerPort1, self.domeHost)
        MainWindow.setTabOrder(self.domeHost, self.powerBootPort1)
        MainWindow.setTabOrder(self.powerBootPort1, self.powerBootPort2)
        MainWindow.setTabOrder(self.powerBootPort2, self.checkJNow)
        MainWindow.setTabOrder(self.checkJNow, self.dewCurrentB)
        MainWindow.setTabOrder(self.dewCurrentB, self.astrometryDevice)
        MainWindow.setTabOrder(self.astrometryDevice, self.imagingDevice)
        MainWindow.setTabOrder(self.imagingDevice, self.environPort)
        MainWindow.setTabOrder(self.environPort, self.imagingDeviceName)
        MainWindow.setTabOrder(self.imagingDeviceName, self.skymeterHost)
        MainWindow.setTabOrder(self.skymeterHost, self.powerPort)
        MainWindow.setTabOrder(self.powerPort, self.skymeterDeviceName)
        MainWindow.setTabOrder(self.skymeterDeviceName, self.skymeterPort)
        MainWindow.setTabOrder(self.skymeterPort, self.imagingPort)
        MainWindow.setTabOrder(self.imagingPort, self.domePort)
        MainWindow.setTabOrder(self.domePort, self.powerDeviceName)
        MainWindow.setTabOrder(self.powerDeviceName, self.powerHost)
        MainWindow.setTabOrder(self.powerHost, self.environDeviceName)
        MainWindow.setTabOrder(self.environDeviceName, self.domeDeviceName)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MountWizzard4 (C) MW 2018"))
        self.domeConnected.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of dome: </p><p>Red: Error</p><p>Gray: No Configured</p><p>Green: Connected</p></body></html>"))
        self.domeConnected.setText(_translate("MainWindow", "Dome"))
        self.imagingText.setToolTip(_translate("MainWindow", "Status feedback from mount "))
        self.mountConnected.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of mount</p><p>Red: Error</p><p>Yellow: Connecting</p><p>Green: Connected</p></body></html>"))
        self.mountConnected.setText(_translate("MainWindow", "Mount"))
        self.mountConnected.setProperty("color", _translate("MainWindow", "gray"))
        self.astrometryText.setToolTip(_translate("MainWindow", "Status feedback from mount "))
        self.imagingConnected.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of camera:</p><p>Red: Error</p><p>Yellow: Connecting</p><p>Green: Connected</p><p>Gray: No Configured</p></body></html>"))
        self.imagingConnected.setText(_translate("MainWindow", "Camera"))
        self.domeText.setToolTip(_translate("MainWindow", "Status feedback from mount "))
        self.astrometryConnected.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of plate solver:</p><p>Red: Error</p><p>Yellow: Connecting</p><p>Green: Connected</p><p>Gray: No Configured</p></body></html>"))
        self.astrometryConnected.setText(_translate("MainWindow", "Solver"))
        self.environConnected.setToolTip(_translate("MainWindow", "<html><head/><body><p>Status of environment: </p><p>Red: Error</p><p>Gray: No Configured</p><p>Green: Connected</p></body></html>"))
        self.environConnected.setText(_translate("MainWindow", "Environ"))
        self.label_38.setText(_translate("MainWindow", "Status"))
        self.openHemisphereW.setToolTip(_translate("MainWindow", "Opens and closes the hemishpere window. If blue, the windows should be visible. Position and size will be stored in case of closing."))
        self.openHemisphereW.setText(_translate("MainWindow", "Hemis"))
        self.openSatelliteW.setToolTip(_translate("MainWindow", "Opens and closes the satellite window. If blue, the windows should be visible. Position and size will be stored in case of closing."))
        self.openSatelliteW.setText(_translate("MainWindow", "Satellite"))
        self.openImageW.setToolTip(_translate("MainWindow", "Opens and closes the image window. If blue, the windows should be visible. Position and size will be stored in case of closing."))
        self.openImageW.setText(_translate("MainWindow", "Image"))
        self.statusText.setToolTip(_translate("MainWindow", "Status feedback from mount "))
        self.statusText.setText(_translate("MainWindow", "-"))
        self.openMeasureW.setToolTip(_translate("MainWindow", "Opens and closes the measure window. If blue, the windows should be visible. Position and size will be stored in case of closing."))
        self.openMeasureW.setText(_translate("MainWindow", "Measure"))
        self.openMessageW.setToolTip(_translate("MainWindow", "Opens and closes the message window. If blue, the windows should be visible. Position and size will be stored in case of closing."))
        self.openMessageW.setText(_translate("MainWindow", "Message"))
        self.timeUTC.setToolTip(_translate("MainWindow", "Shows the actual selected profile name."))
        self.timeUTC.setText(_translate("MainWindow", "-"))
        self.timeComputer.setToolTip(_translate("MainWindow", "Shows the actual time of the computer"))
        self.timeComputer.setText(_translate("MainWindow", "time"))
        self.profile.setToolTip(_translate("MainWindow", "Shows the actual selected profile name."))
        self.profile.setText(_translate("MainWindow", "-"))
        self.label_3.setText(_translate("MainWindow", "Profile: "))
        self.saveConfigAs.setToolTip(_translate("MainWindow", "Saves the actual profile under a selectable new name on the disk without closing the appliation."))
        self.saveConfigAs.setText(_translate("MainWindow", "Save as"))
        self.mountOn.setToolTip(_translate("MainWindow", "Boots the mount if Wake on Lan is anabled, a MAC address is set in the configuration and the mount computer is connected to the same subnet as your computer. A wired connection is needed. "))
        self.mountOn.setText(_translate("MainWindow", "ON"))
        self.loadFrom.setToolTip(_translate("MainWindow", "Loads a profile from disk"))
        self.loadFrom.setText(_translate("MainWindow", "Load"))
        self.saveConfig.setToolTip(_translate("MainWindow", "Saves the actual profile to disk under the same name without closing the application."))
        self.saveConfig.setText(_translate("MainWindow", "Save"))
        self.saveConfigQuit.setToolTip(_translate("MainWindow", "Saves the actual profile and closes the application"))
        self.saveConfigQuit.setText(_translate("MainWindow", "Save/Quit"))
        self.mountOff.setToolTip(_translate("MainWindow", "Shuts the mount down."))
        self.mountOff.setText(_translate("MainWindow", "Off"))
        self.statusOnline.setText(_translate("MainWindow", "Main"))
        self.mainTabWidget.setToolTip(_translate("MainWindow", "<html><head/><body><p>Tabs for the main functions of model making</p></body></html>"))
        self.RA.setToolTip(_translate("MainWindow", "Display of right ascention in HH:MM:SS"))
        self.RA.setText(_translate("MainWindow", "RA"))
        self.AZ.setToolTip(_translate("MainWindow", "Display of azimuth"))
        self.AZ.setText(_translate("MainWindow", "AZ"))
        self.DEC.setToolTip(_translate("MainWindow", "Display of declination in DD:MM:SS"))
        self.DEC.setText(_translate("MainWindow", "DEC"))
        self.ALT.setToolTip(_translate("MainWindow", "Display of altitude"))
        self.ALT.setText(_translate("MainWindow", "ALT"))
        self.pierside.setToolTip(_translate("MainWindow", "Shows the pierside reported from the mout. Eeast or West. Definitions are given in the manual of the mount computer."))
        self.pierside.setText(_translate("MainWindow", "-"))
        self.label_18.setText(_translate("MainWindow", "Pierside is"))
        self.label_52.setText(_translate("MainWindow", "Time to Flip"))
        self.timeToFlip.setToolTip(_translate("MainWindow", "Shows the time until tracking ends because of flip / horizon limit"))
        self.timeToFlip.setText(_translate("MainWindow", "-"))
        self.label_61.setText(_translate("MainWindow", "min"))
        self.label_80.setText(_translate("MainWindow", "Status from mount computer"))
        self.tracking.setToolTip(_translate("MainWindow", "Switching tracking on and off. Blue button show tracking is on."))
        self.tracking.setText(_translate("MainWindow", "Tracking"))
        self.park.setToolTip(_translate("MainWindow", "Switches the parkings state on and on. Blue button means mount parked. It is the position, which is in the mount computer defined as park position. If the mount is not parked, it slews to the park position. If parked, the button press just unparks and starts tracking."))
        self.park.setText(_translate("MainWindow", "Mount Park"))
        self.label_84.setText(_translate("MainWindow", "Tracking / Flip"))
        self.label_85.setText(_translate("MainWindow", "Parking"))
        self.stop.setToolTip(_translate("MainWindow", "Stops all mount movements immediatly"))
        self.stop.setText(_translate("MainWindow", "STOP MOUNT"))
        self.flipMount.setToolTip(_translate("MainWindow", "Flips the mount if possible"))
        self.flipMount.setText(_translate("MainWindow", "Flip"))
        self.setLunarTracking.setToolTip(_translate("MainWindow", "Set tracking speed to lunar"))
        self.setLunarTracking.setText(_translate("MainWindow", "Lunar"))
        self.setSolarTracking.setToolTip(_translate("MainWindow", "Set tracking speed to solar"))
        self.setSolarTracking.setText(_translate("MainWindow", "Solar"))
        self.setSiderealTracking.setToolTip(_translate("MainWindow", "Set tracking speed to sideral"))
        self.setSiderealTracking.setText(_translate("MainWindow", "Sidereal"))
        self.label_90.setText(_translate("MainWindow", "Time to Meridian"))
        self.timeToMeridian.setToolTip(_translate("MainWindow", "Shows the time until meridian is reached"))
        self.timeToMeridian.setText(_translate("MainWindow", "-"))
        self.label_91.setText(_translate("MainWindow", "min"))
        self.label_98.setText(_translate("MainWindow", "Telescope pointing coordinates"))
        self.groupBoxCoordinateSystems.setTitle(_translate("MainWindow", "Epoch"))
        self.checkJNow.setToolTip(_translate("MainWindow", "Sets the epoch for showing the pointing coordinates to JNow"))
        self.checkJNow.setText(_translate("MainWindow", "JNow"))
        self.checkJ2000.setToolTip(_translate("MainWindow", "Sets the epoch for showing the pointing coordinates to J2000"))
        self.checkJ2000.setText(_translate("MainWindow", "J2000"))
        self.UTCExpire.setToolTip(_translate("MainWindow", "Shows the expiration date of UTC timetable in the moutn computer. If it\'s happening within 1 month, the frame turns to yellow. If expired, the frame will turn to red. "))
        self.UTCExpire.setText(_translate("MainWindow", "-"))
        self.label_152.setText(_translate("MainWindow", "UTC expires"))
        self.timeSidereal.setToolTip(_translate("MainWindow", "Shows the local sidereal time from the mount computer"))
        self.timeSidereal.setText(_translate("MainWindow", "-"))
        self.label_7.setText(_translate("MainWindow", "Local Sidereal"))
        self.timeJD.setToolTip(_translate("MainWindow", "Shows the julian date of the mount computer"))
        self.timeJD.setText(_translate("MainWindow", "-"))
        self.label_63.setText(_translate("MainWindow", "Julian Date"))
        self.label_64.setText(_translate("MainWindow", "°C"))
        self.label_74.setText(_translate("MainWindow", "Dual Axis Tracking"))
        self.statusUnattendedFlip.setToolTip(_translate("MainWindow", "Status of unattended Flip.\n"
"When clicking the button, you could change the value."))
        self.statusUnattendedFlip.setText(_translate("MainWindow", "-"))
        self.label_55.setText(_translate("MainWindow", "Unattended Flip"))
        self.statusDualTracking.setToolTip(_translate("MainWindow", "Status of dual tracking.\n"
"When clicking the button, you could change the value."))
        self.statusDualTracking.setText(_translate("MainWindow", "-"))
        self.label_232.setText(_translate("MainWindow", "Refrac. Temp."))
        self.label_242.setText(_translate("MainWindow", "Refrac. Press."))
        self.refractionTemp.setToolTip(_translate("MainWindow", "Shows the refraction temperature used in mount"))
        self.refractionTemp.setText(_translate("MainWindow", "-"))
        self.refractionPress.setToolTip(_translate("MainWindow", "Shows the refraction pressure used in mount"))
        self.refractionPress.setText(_translate("MainWindow", "-"))
        self.label_81.setText(_translate("MainWindow", "hPa"))
        self.label_130.setText(_translate("MainWindow", "HMS"))
        self.label_222.setText(_translate("MainWindow", "YMD"))
        self.statusRefraction.setToolTip(_translate("MainWindow", "Shows the status if the mount takes refraction parameters into account. When clicking the button, you could change the value."))
        self.statusRefraction.setText(_translate("MainWindow", "-"))
        self.label_86.setText(_translate("MainWindow", "Refraction correction"))
        self.label_139.setText(_translate("MainWindow", "HMS"))
        self.label_148.setText(_translate("MainWindow", "GPS time synced"))
        self.statusGPSSynced.setToolTip(_translate("MainWindow", "Shows if the mount is synced to an attache GPS receiver."))
        self.statusGPSSynced.setText(_translate("MainWindow", "-"))
        self.meridianLimitSlew.setToolTip(_translate("MainWindow", "Shows the amount of degrees when the mount will flip after the meridian is passed when slewing. Positive values show after meridian, negative values before meridian.\n"
"When clicking the button, you could change the value.\n"
"Please refer to the mount manual for details"))
        self.meridianLimitSlew.setText(_translate("MainWindow", "-"))
        self.label_28.setText(_translate("MainWindow", "Slew Rate"))
        self.label_20.setText(_translate("MainWindow", "Horizon Lim high"))
        self.label_4.setText(_translate("MainWindow", "Horizon Lim low"))
        self.label_133.setText(_translate("MainWindow", "°/s"))
        self.meridianLimitTrack.setToolTip(_translate("MainWindow", "Shows the amount of degrees when the mount will flip after the meridian is passed when tracking. Positive values show after meridian, negative values before meridian.\n"
"When clicking the button, you could change the value.\n"
"Please refer to the mount manual for details"))
        self.meridianLimitTrack.setText(_translate("MainWindow", "-"))
        self.label_109.setText(_translate("MainWindow", "°"))
        self.label_65.setText(_translate("MainWindow", "Meridian Lim Slew"))
        self.label_89.setText(_translate("MainWindow", "°"))
        self.horizonLimitLow.setToolTip(_translate("MainWindow", "Actual horizon limit for slewing for low limit (lower boundary).\n"
"When clicking the button, you could change the value."))
        self.horizonLimitLow.setText(_translate("MainWindow", "-"))
        self.horizonLimitHigh.setToolTip(_translate("MainWindow", "Actual horizon limit for slewing for high values (upper boundary).\n"
"When clicking the button, you could change the value."))
        self.horizonLimitHigh.setText(_translate("MainWindow", "-"))
        self.label_88.setText(_translate("MainWindow", "°"))
        self.label_104.setText(_translate("MainWindow", "°"))
        self.label_34.setText(_translate("MainWindow", "Meridian Lim Track"))
        self.slewRate.setToolTip(_translate("MainWindow", "Shows the actual set slew rate for the mount.\n"
"When clicking the button, you could change the value."))
        self.slewRate.setText(_translate("MainWindow", "-"))
        self.label_6.setText(_translate("MainWindow", "Site Latitude"))
        self.label_5.setText(_translate("MainWindow", "Site Longitude"))
        self.label_12.setText(_translate("MainWindow", "Site Elevation"))
        self.siteLongitude.setToolTip(_translate("MainWindow", "Shows the logitude of the site.\n"
"When clicking the button, you could change the value."))
        self.siteLongitude.setText(_translate("MainWindow", "-"))
        self.siteLatitude.setToolTip(_translate("MainWindow", "Shows the latitude of the site.\n"
"When clicking the button, you could change the value."))
        self.siteLatitude.setText(_translate("MainWindow", "-"))
        self.label_105.setText(_translate("MainWindow", "DMS"))
        self.label_82.setText(_translate("MainWindow", "m"))
        self.label_106.setText(_translate("MainWindow", "DMS"))
        self.siteElevation.setToolTip(_translate("MainWindow", "Shows the elevation of the site.\n"
"When clicking the button, you could change the value."))
        self.siteElevation.setText(_translate("MainWindow", "-"))
        self.label_151.setText(_translate("MainWindow", "WakeOnLan"))
        self.statusWOL.setToolTip(_translate("MainWindow", "Shows the status of Wake on Lan status. It could be none (not available), on or off."))
        self.statusWOL.setText(_translate("MainWindow", "-"))
        self.HA.setToolTip(_translate("MainWindow", "Display of right ascention in HH:MM:SS"))
        self.HA.setText(_translate("MainWindow", "HA"))
        self.label_58.setText(_translate("MainWindow", "HA"))
        self.label_77.setText(_translate("MainWindow", "RA"))
        self.label_78.setText(_translate("MainWindow", "ALT"))
        self.label_79.setText(_translate("MainWindow", "AZ"))
        self.label_102.setText(_translate("MainWindow", "DEC"))
        self.HAfloat.setToolTip(_translate("MainWindow", "Status feedback from mount "))
        self.HAfloat.setText(_translate("MainWindow", "-"))
        self.RAfloat.setToolTip(_translate("MainWindow", "Status feedback from mount "))
        self.RAfloat.setText(_translate("MainWindow", "-"))
        self.DECfloat.setToolTip(_translate("MainWindow", "Status feedback from mount "))
        self.DECfloat.setText(_translate("MainWindow", "-"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Mount), _translate("MainWindow", " Mount"))
        self.label_32.setText(_translate("MainWindow", "WindSpeed"))
        self.label_25.setText(_translate("MainWindow", "DewPoint"))
        self.label_31.setText(_translate("MainWindow", "Rain vol."))
        self.label_26.setText(_translate("MainWindow", "Humidity"))
        self.setRefractionManual.setToolTip(_translate("MainWindow", "<html><head/><body><p>Pushes the refraction temperature and pressure to mount.</p></body></html>"))
        self.setRefractionManual.setText(_translate("MainWindow", "Push parameters\n"
"to mount manually"))
        self.label_30.setText(_translate("MainWindow", "CloudCover"))
        self.label_24.setText(_translate("MainWindow", "Airtemp."))
        self.label_27.setText(_translate("MainWindow", "Pressure"))
        self.label_142.setText(_translate("MainWindow", "SkyQuality"))
        self.refractionGroup.setTitle(_translate("MainWindow", "Periodic refraction update"))
        self.checkRefracCont.setToolTip(_translate("MainWindow", "<html><head/><body><p>Update the refraction parameters to the mount when camera is connected and IDLE.</p></body></html>"))
        self.checkRefracCont.setText(_translate("MainWindow", "continously"))
        self.checkRefracNoTrack.setToolTip(_translate("MainWindow", "<html><head/><body><p>Update the refraction parameters to the mount when the mount is not tracking.</p></body></html>"))
        self.checkRefracNoTrack.setText(_translate("MainWindow", "no tracking"))
        self.checkRefracNone.setToolTip(_translate("MainWindow", "<html><head/><body><p>Update the refraction parameters to the mount when camera is connected and IDLE.</p></body></html>"))
        self.checkRefracNone.setText(_translate("MainWindow", "no update"))
        self.mountGroup.setTitle(_translate("MainWindow", "Mount"))
        self.refractionTemp1.setToolTip(_translate("MainWindow", "shows the refraction temperature used in mount"))
        self.refractionTemp1.setText(_translate("MainWindow", "-"))
        self.refractionPress1.setToolTip(_translate("MainWindow", "shows the refraction pressure used in mount"))
        self.refractionPress1.setText(_translate("MainWindow", "-"))
        self.environGroup.setTitle(_translate("MainWindow", "Sensor"))
        self.environTemp.setToolTip(_translate("MainWindow", "shows the refraction temperature used in mount"))
        self.environTemp.setText(_translate("MainWindow", "-"))
        self.environPress.setToolTip(_translate("MainWindow", "shows the refraction pressure used in mount"))
        self.environPress.setText(_translate("MainWindow", "-"))
        self.environDewPoint.setText(_translate("MainWindow", "-"))
        self.environHumidity.setText(_translate("MainWindow", "-"))
        self.label_33.setText(_translate("MainWindow", "Wind Dir"))
        self.weatherGroup.setTitle(_translate("MainWindow", "Weather"))
        self.weatherRainVol.setText(_translate("MainWindow", "-"))
        self.weatherWindSpeed.setToolTip(_translate("MainWindow", "shows the refraction correction status on / off"))
        self.weatherWindSpeed.setText(_translate("MainWindow", "-"))
        self.weatherCloudCover.setText(_translate("MainWindow", "-"))
        self.weatherTemp.setText(_translate("MainWindow", "-"))
        self.weatherPress.setText(_translate("MainWindow", "-"))
        self.weatherHumidity.setText(_translate("MainWindow", "-"))
        self.weatherWindDir.setText(_translate("MainWindow", "-"))
        self.skymeterGroup.setTitle(_translate("MainWindow", "SkyMeter"))
        self.skymeterSQR.setToolTip(_translate("MainWindow", "shows the refraction correction status on / off"))
        self.skymeterSQR.setText(_translate("MainWindow", "-"))
        self.skymeterTemp.setToolTip(_translate("MainWindow", "shows the refraction correction status on / off"))
        self.skymeterTemp.setText(_translate("MainWindow", "-"))
        self.label_70.setText(_translate("MainWindow", "[°C]"))
        self.label_73.setText(_translate("MainWindow", "[hPa]"))
        self.label_87.setText(_translate("MainWindow", "[mpas]"))
        self.label_71.setText(_translate("MainWindow", "[°C]"))
        self.label_72.setText(_translate("MainWindow", "[%]"))
        self.label_143.setText(_translate("MainWindow", "[%]"))
        self.label_68.setText(_translate("MainWindow", "[mm]"))
        self.label_69.setText(_translate("MainWindow", "[°]"))
        self.label_67.setText(_translate("MainWindow", "[m/s]"))
        self.powerGroup.setTitle(_translate("MainWindow", "Power"))
        self.powerTemp.setToolTip(_translate("MainWindow", "shows the refraction temperature used in mount"))
        self.powerTemp.setText(_translate("MainWindow", "-"))
        self.powerDewPoint.setText(_translate("MainWindow", "-"))
        self.powerHumidity.setText(_translate("MainWindow", "-"))
        self.label_228.setText(_translate("MainWindow", "Environment data from various sensors"))
        self.picClearOutside.setText(_translate("MainWindow", "Clear Outside panel when connected to internet"))
        self.picClearOutsideHeader.setText(_translate("MainWindow", "Clear Outside"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Environ), _translate("MainWindow", "Environ"))
        self.label.setText(_translate("MainWindow", "Number Stars"))
        self.azimuthError.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual Polar Error</p></body></html>"))
        self.azimuthError.setText(_translate("MainWindow", "-"))
        self.positionAngle.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual RMS Error in arc sec</p><p><br/></p></body></html>"))
        self.positionAngle.setText(_translate("MainWindow", "-"))
        self.azimuthTurns.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of Turns azimuth ajustment knobs for correction (left if polar axis has to be moved to the left) </p></body></html>"))
        self.azimuthTurns.setText(_translate("MainWindow", "--- left/right"))
        self.polarError.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual Polar Error</p></body></html>"))
        self.polarError.setText(_translate("MainWindow", "-"))
        self.label_14.setText(_translate("MainWindow", "RMS"))
        self.label_153.setText(_translate("MainWindow", "Ortho Align Error"))
        self.terms.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of Terms in actual used pointing model</p></body></html>"))
        self.terms.setText(_translate("MainWindow", "-"))
        self.altitudeTurns.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of Turns altitude ajustment knobs for correction (down if the polar axis has to be moved downwards) </p></body></html>"))
        self.altitudeTurns.setText(_translate("MainWindow", "--- left/right"))
        self.label_156.setText(_translate("MainWindow", "Turn Knob AZ"))
        self.label_158.setText(_translate("MainWindow", "Azimuth Pointing"))
        self.numberStars.setToolTip(_translate("MainWindow", "Number of alignment stars in actual used pointing model"))
        self.numberStars.setText(_translate("MainWindow", "-"))
        self.errorRMS.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual RMS Error in arc sec</p><p><br/></p></body></html>"))
        self.errorRMS.setText(_translate("MainWindow", "-"))
        self.label_155.setText(_translate("MainWindow", "Alignment Info from Mount Computer"))
        self.orthoError.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual Ortho Error</p></body></html>"))
        self.orthoError.setText(_translate("MainWindow", "-"))
        self.label_159.setText(_translate("MainWindow", "Altitude Pointing"))
        self.label_15.setText(_translate("MainWindow", "Polar Align Error "))
        self.altitudeError.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual Ortho Error</p></body></html>"))
        self.altitudeError.setText(_translate("MainWindow", "-"))
        self.label_154.setText(_translate("MainWindow", "Number Terms"))
        self.label_157.setText(_translate("MainWindow", "Turn Knob ALT"))
        self.label_160.setText(_translate("MainWindow", "PosAngle"))
        self.picAZ.setText(_translate("MainWindow", "TextLabel"))
        self.picALT.setText(_translate("MainWindow", "TextLabel"))
        self.label_167.setText(_translate("MainWindow", "°"))
        self.label_168.setText(_translate("MainWindow", "°"))
        self.label_169.setText(_translate("MainWindow", "arcmin"))
        self.label_218.setText(_translate("MainWindow", "°"))
        self.label_201.setText(_translate("MainWindow", "arcmin"))
        self.label_219.setText(_translate("MainWindow", "Ajust Polar Alignment"))
        self.groupBox_10.setTitle(_translate("MainWindow", "Align model build"))
        self.cancelAlignModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Cancels model run.</p></body></html>"))
        self.cancelAlignModel.setText(_translate("MainWindow", "Cancel Align Model"))
        self.runAlignModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Runs the base model.</p></body></html>"))
        self.runAlignModel.setText(_translate("MainWindow", "Run Align Model"))
        self.groupBox_11.setTitle(_translate("MainWindow", "Generate align points as grid"))
        self.genAlignBuild.setToolTip(_translate("MainWindow", "<html><head/><body><p>Generates base points based on number, altitude and azimuth settings.</p></body></html>"))
        self.genAlignBuild.setText(_translate("MainWindow", "Generate Align Build Points"))
        self.label_127.setText(_translate("MainWindow", "°"))
        self.label_206.setText(_translate("MainWindow", "Number"))
        self.label_171.setText(_translate("MainWindow", "Azimuth"))
        self.label_170.setText(_translate("MainWindow", "Altitude"))
        self.label_128.setText(_translate("MainWindow", "°"))
        self.groupBox_12.setTitle(_translate("MainWindow", "Load align model points from file"))
        self.alignBuildPFileName.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Model Points file, where the data is stored.</p><p>Please Double Click to Select</p></body></html>"))
        self.saveAlignBuildPointsAs.setToolTip(_translate("MainWindow", "<html><head/><body><p>Saves the initial model points in new file</p></body></html>"))
        self.saveAlignBuildPointsAs.setText(_translate("MainWindow", "Save as"))
        self.saveAlignBuildPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Saves the initial model points in file</p></body></html>"))
        self.saveAlignBuildPoints.setText(_translate("MainWindow", "Save"))
        self.genAlignBuildFile.setToolTip(_translate("MainWindow", "<html><head/><body><p>Show the initial model points from given file</p></body></html>"))
        self.genAlignBuildFile.setText(_translate("MainWindow", "Generate"))
        self.loadAlignBuildPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Loads the initial model points from file</p></body></html>"))
        self.loadAlignBuildPoints.setText(_translate("MainWindow", "Load"))
        self.groupBox_9.setTitle(_translate("MainWindow", "Fast align"))
        self.plateSolveSync.setToolTip(_translate("MainWindow", "<html><head/><body><p>Image, Platesolve and Sync the alignment model of the mount</p></body></html>"))
        self.plateSolveSync.setText(_translate("MainWindow", "Plate Solve & Sync Model"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.AlignMount), _translate("MainWindow", "Align Mount"))
        self.groupBox_2.setTitle(_translate("MainWindow", "Load model points from file"))
        self.buildPFileName.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Model Points file, where the data is stored.</p><p>Please Double Click to Select</p></body></html>"))
        self.saveBuildPointsAs.setToolTip(_translate("MainWindow", "<html><head/><body><p>Saves the full model points in new file</p></body></html>"))
        self.saveBuildPointsAs.setText(_translate("MainWindow", "Save as"))
        self.saveBuildPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Saves the full model points in file</p></body></html>"))
        self.saveBuildPoints.setText(_translate("MainWindow", "Save"))
        self.genBuildFile.setToolTip(_translate("MainWindow", "<html><head/><body><p>Show the full model points from given file</p></body></html>"))
        self.genBuildFile.setText(_translate("MainWindow", "Generate"))
        self.loadBuildPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Loads the full model points from file</p></body></html>"))
        self.loadBuildPoints.setText(_translate("MainWindow", "Load"))
        self.groupBox_3.setTitle(_translate("MainWindow", "Generate model points as grid"))
        self.label_178.setText(_translate("MainWindow", "Rows"))
        self.numberGridPointsRow.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of rows in the grid build</p></body></html>"))
        self.label_179.setText(_translate("MainWindow", "Cols"))
        self.label_190.setText(_translate("MainWindow", "AltMin"))
        self.label_203.setText(_translate("MainWindow", "°"))
        self.altitudeMax.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of coloms in the grid build</p></body></html>"))
        self.label_144.setText(_translate("MainWindow", "°"))
        self.numberGridPointsCol.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of coloms in the grid build</p></body></html>"))
        self.label_180.setText(_translate("MainWindow", "AltMax"))
        self.genBuildGrid.setToolTip(_translate("MainWindow", "<html><head/><body><p>Generates a regular pattern with the given parameters.</p></body></html>"))
        self.genBuildGrid.setText(_translate("MainWindow", "Generate Grid"))
        self.altitudeMin.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of rows in the grid build</p></body></html>"))
        self.groupBox_4.setTitle(_translate("MainWindow", "Generate model points as greater circles"))
        self.genBuildMax.setToolTip(_translate("MainWindow", "<html><head/><body><p>Generates a pointcloud with greater circles: high number of stars.</p></body></html>"))
        self.genBuildMax.setText(_translate("MainWindow", "Maximal"))
        self.genBuildMed.setToolTip(_translate("MainWindow", "<html><head/><body><p>Generates a pointcloud with greater circles: medium number of stars.</p></body></html>"))
        self.genBuildMed.setText(_translate("MainWindow", "Medium"))
        self.genBuildMin.setToolTip(_translate("MainWindow", "<html><head/><body><p>Generates a pointcloud with greater circles: low number of stars.</p></body></html>"))
        self.genBuildMin.setText(_translate("MainWindow", "Minimal"))
        self.genBuildNorm.setToolTip(_translate("MainWindow", "<html><head/><body><p>Generates a pointcloud with greater circles: medium number of stars.</p></body></html>"))
        self.genBuildNorm.setText(_translate("MainWindow", "Norm"))
        self.groupBox_5.setTitle(_translate("MainWindow", "Generate model points from tracks"))
        self.label_188.setText(_translate("MainWindow", "Points"))
        self.numberDSOPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of points in the DSO track for the given time in hours</p></body></html>"))
        self.durationDSO.setToolTip(_translate("MainWindow", "<html><head/><body><p>Length of the DSO track in hours.</p></body></html>"))
        self.timeShiftDSO.setToolTip(_translate("MainWindow", "<html><head/><body><p>Length of the DSO track in hours.</p></body></html>"))
        self.label_204.setText(_translate("MainWindow", "h"))
        self.label_189.setText(_translate("MainWindow", "TimeShift"))
        self.label_303.setText(_translate("MainWindow", "h"))
        self.label_181.setText(_translate("MainWindow", "Duration"))
        self.genBuildDSO.setToolTip(_translate("MainWindow", "<html><head/><body><p>Generates model points along DSO track for a give time and other parameters.</p></body></html>"))
        self.genBuildDSO.setText(_translate("MainWindow", "Generate DSO Track"))
        self.groupBox_6.setTitle(_translate("MainWindow", "Generate model points equally distributed"))
        self.genBuildGoldenSpiral.setToolTip(_translate("MainWindow", "<html><head/><body><p>Generates model points along DSO track for a give time and other parameters.</p></body></html>"))
        self.genBuildGoldenSpiral.setText(_translate("MainWindow", "Generate Golden Spiral"))
        self.numberSpiralPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Number of points in the DSO track for the given time in hours</p></body></html>"))
        self.label_309.setText(_translate("MainWindow", "Points"))
        self.groupBox_7.setTitle(_translate("MainWindow", "Build model from data file"))
        self.batchModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Run model making from a data file stored as analyse file</p></body></html>"))
        self.batchModel.setText(_translate("MainWindow", "Choose model from file and program"))
        self.groupBox_8.setTitle(_translate("MainWindow", "Full model build"))
        self.cancelFullModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Cancels model run.</p></body></html>"))
        self.cancelFullModel.setText(_translate("MainWindow", "Cancel Model Build"))
        self.mSlew.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.mSlew.setText(_translate("MainWindow", "0"))
        self.label_35.setText(_translate("MainWindow", "Time to finish:"))
        self.runFullModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Run full alignment model.</p></body></html>"))
        self.runFullModel.setText(_translate("MainWindow", "Run Model Build"))
        self.mPoints.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.mPoints.setText(_translate("MainWindow", "0"))
        self.label_29.setText(_translate("MainWindow", "Elapsed time:"))
        self.mSolve.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.mSolve.setText(_translate("MainWindow", "0"))
        self.label_310.setText(_translate("MainWindow", "Slew:"))
        self.label_311.setText(_translate("MainWindow", "Solve:"))
        self.mImage.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.mImage.setText(_translate("MainWindow", "0"))
        self.timeElapsed.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.timeElapsed.setText(_translate("MainWindow", "00:00"))
        self.label_216.setText(_translate("MainWindow", "Points:"))
        self.timeFinished.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.timeFinished.setText(_translate("MainWindow", "00:00:00"))
        self.label_312.setText(_translate("MainWindow", "Image:"))
        self.timeToFinish.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.timeToFinish.setText(_translate("MainWindow", "00:00"))
        self.label_42.setText(_translate("MainWindow", "Finishing Time:"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.BuildModel), _translate("MainWindow", "Build Model"))
        self.label_37.setText(_translate("MainWindow", "Target RMS"))
        self.runTargetRMS.setToolTip(_translate("MainWindow", "<html><head/><body><p>Deletes worst point of actual alignment model until error is below target RMS.</p></body></html>"))
        self.runTargetRMS.setText(_translate("MainWindow", "Optimise"))
        self.deleteWorstPoint.setToolTip(_translate("MainWindow", "<html><head/><body><p>Deletes worst point of actual alignment model in the mount. It could happen, that the overall RMS rises.</p></body></html>"))
        self.deleteWorstPoint.setText(_translate("MainWindow", "Delete Worst"))
        self.label_124.setText(_translate("MainWindow", "Optimise actual model"))
        self.label_125.setText(_translate("MainWindow", "Models stored in mount computer:"))
        self.cancelTargetRMS.setToolTip(_translate("MainWindow", "<html><head/><body><p>Cancels the optimise run.</p></body></html>"))
        self.cancelTargetRMS.setText(_translate("MainWindow", "Cancel"))
        self.errorRMS1.setToolTip(_translate("MainWindow", "<html><head/><body><p>Actual RMS Error in arc sec</p><p><br/></p></body></html>"))
        self.errorRMS1.setText(_translate("MainWindow", "0.0"))
        self.label_75.setText(_translate("MainWindow", "Actual RMS"))
        self.clearModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Clear the alignment model in mount. Can\'t be revised.</p></body></html>"))
        self.clearModel.setText(_translate("MainWindow", "Clear Model"))
        self.numberStars1.setToolTip(_translate("MainWindow", "Number of alignment stars in actual used pointing model"))
        self.numberStars1.setText(_translate("MainWindow", "0"))
        self.label_76.setText(_translate("MainWindow", "Stars"))
        self.refreshModel.setToolTip(_translate("MainWindow", "<html><head/><body><p>Reloads the refinement model from mount. Needed if you change things manually with handcontroller or other software</p></body></html>"))
        self.refreshModel.setText(_translate("MainWindow", "Refresh"))
        self.saveName.setToolTip(_translate("MainWindow", "<html><head/><body><p>Saves actual alignment model under selected name or edited name.</p></body></html>"))
        self.saveName.setText(_translate("MainWindow", "Save"))
        self.loadName.setToolTip(_translate("MainWindow", "<html><head/><body><p>Loads the actual selected alignment model.</p></body></html>"))
        self.loadName.setText(_translate("MainWindow", "Load"))
        self.deleteName.setToolTip(_translate("MainWindow", "<html><head/><body><p>Deletes the actual selected alignment model.</p></body></html>"))
        self.deleteName.setText(_translate("MainWindow", "Delete"))
        self.checkShowErrorValues.setToolTip(_translate("MainWindow", "Use Highspeed download of CCD camera if availabe"))
        self.checkShowErrorValues.setText(_translate("MainWindow", "Show error values in polar plot"))
        self.refreshName.setToolTip(_translate("MainWindow", "<html><head/><body><p>Deletes the actual selected alignment model.</p></body></html>"))
        self.refreshName.setText(_translate("MainWindow", "Refresh"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.ManageModel), _translate("MainWindow", "Manage Model"))
        self.label_126.setText(_translate("MainWindow", "List of available satellites"))
        self.label_223.setText(_translate("MainWindow", "Satellite data file"))
        self.label_224.setText(_translate("MainWindow", "TLE satellite data calculation"))
        self.satelliteName.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Horizon Mask file, where the data is stored.</p><p>Please Double Click to Select.</p></body></html>"))
        self.satelliteName.setText(_translate("MainWindow", "-"))
        self.label_225.setText(_translate("MainWindow", "Name"))
        self.label_230.setText(_translate("MainWindow", "Epoch"))
        self.satelliteEpoch.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Horizon Mask file, where the data is stored.</p><p>Please Double Click to Select.</p></body></html>"))
        self.satelliteEpoch.setText(_translate("MainWindow", "-"))
        self.startSatelliteTracking.setToolTip(_translate("MainWindow", "<html><head/><body><p>Cancels model run.</p></body></html>"))
        self.startSatelliteTracking.setText(_translate("MainWindow", "Start satellite tracking"))
        self.label_231.setText(_translate("MainWindow", "Age"))
        self.satelliteDataAge.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Horizon Mask file, where the data is stored.</p><p>Please Double Click to Select.</p></body></html>"))
        self.satelliteDataAge.setText(_translate("MainWindow", "-"))
        self.label_233.setText(_translate("MainWindow", "days"))
        self.label_234.setText(_translate("MainWindow", "Azimuth"))
        self.label_235.setText(_translate("MainWindow", "Altitude"))
        self.label_236.setText(_translate("MainWindow", "RA"))
        self.label_237.setText(_translate("MainWindow", "DEC"))
        self.satAzimuth.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Horizon Mask file, where the data is stored.</p><p>Please Double Click to Select.</p></body></html>"))
        self.satAzimuth.setText(_translate("MainWindow", "-"))
        self.satAltitude.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Horizon Mask file, where the data is stored.</p><p>Please Double Click to Select.</p></body></html>"))
        self.satAltitude.setText(_translate("MainWindow", "-"))
        self.satRa.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Horizon Mask file, where the data is stored.</p><p>Please Double Click to Select.</p></body></html>"))
        self.satRa.setText(_translate("MainWindow", "-"))
        self.satDec.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Horizon Mask file, where the data is stored.</p><p>Please Double Click to Select.</p></body></html>"))
        self.satDec.setText(_translate("MainWindow", "-"))
        self.label_202.setText(_translate("MainWindow", "°"))
        self.label_207.setText(_translate("MainWindow", "°"))
        self.label_221.setText(_translate("MainWindow", "hours"))
        self.label_226.setText(_translate("MainWindow", "°"))
        self.label_227.setText(_translate("MainWindow", "Number"))
        self.satelliteNumber.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Horizon Mask file, where the data is stored.</p><p>Please Double Click to Select.</p></body></html>"))
        self.satelliteNumber.setText(_translate("MainWindow", "-"))
        self.label_239.setText(_translate("MainWindow", "Latitude"))
        self.label_240.setText(_translate("MainWindow", "Longitude"))
        self.label_252.setText(_translate("MainWindow", "°"))
        self.satLongitude.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Horizon Mask file, where the data is stored.</p><p>Please Double Click to Select.</p></body></html>"))
        self.satLongitude.setText(_translate("MainWindow", "-"))
        self.satLatitude.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Horizon Mask file, where the data is stored.</p><p>Please Double Click to Select.</p></body></html>"))
        self.satLatitude.setText(_translate("MainWindow", "-"))
        self.label_260.setText(_translate("MainWindow", "Start Time"))
        self.label_270.setText(_translate("MainWindow", "End Time"))
        self.label_271.setText(_translate("MainWindow", "Need flip"))
        self.satNeedFlip.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Horizon Mask file, where the data is stored.</p><p>Please Double Click to Select.</p></body></html>"))
        self.satNeedFlip.setText(_translate("MainWindow", "-"))
        self.satTransitStartUTC.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Horizon Mask file, where the data is stored.</p><p>Please Double Click to Select.</p></body></html>"))
        self.satTransitStartUTC.setText(_translate("MainWindow", "-"))
        self.satTransitEndUTC.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Horizon Mask file, where the data is stored.</p><p>Please Double Click to Select.</p></body></html>"))
        self.satTransitEndUTC.setText(_translate("MainWindow", "-"))
        self.label_272.setText(_translate("MainWindow", "UTC"))
        self.label_273.setText(_translate("MainWindow", "UTC"))
        self.stopSatelliteTracking.setToolTip(_translate("MainWindow", "<html><head/><body><p>Cancels model run.</p></body></html>"))
        self.stopSatelliteTracking.setText(_translate("MainWindow", "Stop satellite tracking"))
        self.label_253.setText(_translate("MainWindow", "°"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Satellite), _translate("MainWindow", "Satellite"))
        self.cancelAnalyse.setToolTip(_translate("MainWindow", "<html><head/><body><p>Cancels the analyse run.</p></body></html>"))
        self.cancelAnalyse.setText(_translate("MainWindow", "Cancel Analyse Run"))
        self.groupBox_14.setTitle(_translate("MainWindow", "Load analyse file"))
        self.le_analyseFileName.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Analysis file, where the data ist stored.</p><p>Please Double Click to select.</p></body></html>"))
        self.loadAnalyseData.setToolTip(_translate("MainWindow", "<html><head/><body><p>Loads the modeling data from file.</p></body></html>"))
        self.loadAnalyseData.setText(_translate("MainWindow", "Load"))
        self.groupBox_15.setTitle(_translate("MainWindow", "Analyse flexure"))
        self.label_177.setText(_translate("MainWindow", "Delay"))
        self.label_111.setText(_translate("MainWindow", "°"))
        self.label_174.setText(_translate("MainWindow", "Altitude"))
        self.label_112.setText(_translate("MainWindow", "°"))
        self.label_176.setText(_translate("MainWindow", "Number Cycles"))
        self.label_115.setText(_translate("MainWindow", "s"))
        self.label_175.setText(_translate("MainWindow", "Azimuth"))
        self.runFlexure.setToolTip(_translate("MainWindow", "<html><head/><body><p>Runs a analyse cycle by constant pointing to given alt/az and measuring the error each delay seconds.</p></body></html>"))
        self.runFlexure.setText(_translate("MainWindow", "Run Flexure"))
        self.groupBox_16.setTitle(_translate("MainWindow", "Analyse hysterese"))
        self.label_182.setText(_translate("MainWindow", "Azimuth"))
        self.label_183.setText(_translate("MainWindow", "Number Cycles"))
        self.label_119.setText(_translate("MainWindow", "°"))
        self.label_120.setText(_translate("MainWindow", "°"))
        self.label_185.setText(_translate("MainWindow", "Point 1    Altitude"))
        self.label_187.setText(_translate("MainWindow", "Point 2    Altitude"))
        self.label_184.setText(_translate("MainWindow", "Delay"))
        self.label_114.setText(_translate("MainWindow", "°"))
        self.label_117.setText(_translate("MainWindow", "s"))
        self.label_186.setText(_translate("MainWindow", "Azimuth"))
        self.label_121.setText(_translate("MainWindow", "°"))
        self.runHysteresis.setToolTip(_translate("MainWindow", "<html><head/><body><p>Runs a analyse cycle by swapping two alt/az positions and measuring the error each delay seconds</p></body></html>"))
        self.runHysteresis.setText(_translate("MainWindow", "Run Hysteresis"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Analyse), _translate("MainWindow", "Analyse"))
        self.label_96.setText(_translate("MainWindow", "Relays"))
        self.relayButton0.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starts for Relay the defined function (Switch or Pulse)</p></body></html>"))
        self.relayButton0.setText(_translate("MainWindow", "Relay1"))
        self.relayButton1.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starts for Relay the defined function (Switch or Pulse)</p></body></html>"))
        self.relayButton1.setText(_translate("MainWindow", "Relay2"))
        self.relayButton2.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starts for Relay the defined function (Switch or Pulse)</p></body></html>"))
        self.relayButton2.setText(_translate("MainWindow", "Relay3"))
        self.relayButton3.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starts for Relay the defined function (Switch or Pulse)</p></body></html>"))
        self.relayButton3.setText(_translate("MainWindow", "Relay4"))
        self.relayButton4.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starts for Relay the defined function (Switch or Pulse)</p></body></html>"))
        self.relayButton4.setText(_translate("MainWindow", "Relay5"))
        self.relayButton5.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starts for Relay the defined function (Switch or Pulse)</p></body></html>"))
        self.relayButton5.setText(_translate("MainWindow", "Relay6"))
        self.relayButton6.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starts for Relay the defined function (Switch or Pulse)</p></body></html>"))
        self.relayButton6.setText(_translate("MainWindow", "Relay7"))
        self.relayButton7.setToolTip(_translate("MainWindow", "<html><head/><body><p>Starts for Relay the defined function (Switch or Pulse)</p></body></html>"))
        self.relayButton7.setText(_translate("MainWindow", "Relay8"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Relay), _translate("MainWindow", "Relay"))
        self.groupBox_13.setTitle(_translate("MainWindow", "Power Port 4"))
        self.label_283.setText(_translate("MainWindow", "A"))
        self.label_292.setText(_translate("MainWindow", "Current"))
        self.powerPort4.setText(_translate("MainWindow", "Power On / Off"))
        self.powerBootPort4.setText(_translate("MainWindow", "Power on boot"))
        self.powerLabel4.setText(_translate("MainWindow", "---"))
        self.groupBox_17.setTitle(_translate("MainWindow", "Power Port 3"))
        self.label_317.setText(_translate("MainWindow", "A"))
        self.label_318.setText(_translate("MainWindow", "Current"))
        self.powerLabel3.setText(_translate("MainWindow", "---"))
        self.powerBootPort3.setText(_translate("MainWindow", "Power on boot"))
        self.powerPort3.setText(_translate("MainWindow", "Power On / Off"))
        self.groupBox_18.setTitle(_translate("MainWindow", "Power Port 2"))
        self.label_319.setText(_translate("MainWindow", "A"))
        self.label_320.setText(_translate("MainWindow", "Current"))
        self.powerLabel2.setText(_translate("MainWindow", "---"))
        self.powerPort2.setText(_translate("MainWindow", "Power On / Off"))
        self.powerBootPort2.setText(_translate("MainWindow", "Power on boot"))
        self.groupBox_19.setTitle(_translate("MainWindow", "Power Port 1"))
        self.label_321.setText(_translate("MainWindow", "A"))
        self.label_322.setText(_translate("MainWindow", "Current"))
        self.powerLabel1.setText(_translate("MainWindow", "---"))
        self.powerBootPort1.setText(_translate("MainWindow", "Power on boot"))
        self.powerPort1.setText(_translate("MainWindow", "Power On / Off"))
        self.label_229.setText(_translate("MainWindow", "Ports"))
        self.groupBox_20.setTitle(_translate("MainWindow", "USB Hub"))
        self.hubUSB.setText(_translate("MainWindow", "USB Hub On / Off"))
        self.label_238.setText(_translate("MainWindow", "Overall"))
        self.groupBox_21.setTitle(_translate("MainWindow", "Consumption"))
        self.label_192.setText(_translate("MainWindow", "A"))
        self.label_284.setText(_translate("MainWindow", "Wh"))
        self.label_193.setText(_translate("MainWindow", "Ah"))
        self.label_191.setText(_translate("MainWindow", "W"))
        self.label_286.setText(_translate("MainWindow", "AmpHours"))
        self.label_287.setText(_translate("MainWindow", "Avg. Curr."))
        self.label_165.setText(_translate("MainWindow", "Voltage"))
        self.label_285.setText(_translate("MainWindow", "WattHours"))
        self.label_289.setText(_translate("MainWindow", "Current"))
        self.label_288.setText(_translate("MainWindow", "Power"))
        self.label_173.setText(_translate("MainWindow", "V"))
        self.label_172.setText(_translate("MainWindow", "A"))
        self.groupBox_22.setTitle(_translate("MainWindow", "Dew heater A"))
        self.label_276.setText(_translate("MainWindow", "%"))
        self.label_268.setText(_translate("MainWindow", "Current"))
        self.label_279.setText(_translate("MainWindow", "A"))
        self.label_266.setText(_translate("MainWindow", "DutyCycle"))
        self.groupBox_23.setTitle(_translate("MainWindow", "Dew heater B"))
        self.label_280.setText(_translate("MainWindow", "%"))
        self.label_274.setText(_translate("MainWindow", "Current"))
        self.label_281.setText(_translate("MainWindow", "A"))
        self.label_275.setText(_translate("MainWindow", "DutyCycle"))
        self.groupBox_24.setTitle(_translate("MainWindow", "Automatic dew"))
        self.autoDew.setText(_translate("MainWindow", "Enable Auto Dew"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Power), _translate("MainWindow", "Power"))
        self.label_305.setText(_translate("MainWindow", "Directory"))
        self.label_304.setText(_translate("MainWindow", "Batch renaming fits files"))
        self.checkIncludeSubdirs.setText(_translate("MainWindow", "Include subdirectories"))
        self.renameInputSelect.setText(_translate("MainWindow", "Select"))
        self.renameStart.setText(_translate("MainWindow", "Start"))
        self.label_60.setText(_translate("MainWindow", "Choice of FITS header entries"))
        self.label_8.setText(_translate("MainWindow", "Position 1"))
        self.label_9.setText(_translate("MainWindow", "Position 2"))
        self.label_10.setText(_translate("MainWindow", "Position 3"))
        self.label_11.setText(_translate("MainWindow", "Position 4"))
        self.label_39.setText(_translate("MainWindow", "Position 5"))
        self.label_57.setText(_translate("MainWindow", "Position 6"))
        self.posButton6.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton6.setText(_translate("MainWindow", "Park Pos6"))
        self.label_83.setText(_translate("MainWindow", "Defined park positions (slew&park)"))
        self.posButton4.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton4.setText(_translate("MainWindow", "Park Pos4"))
        self.posButton5.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton5.setText(_translate("MainWindow", "Park Pos5"))
        self.posButton3.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton3.setText(_translate("MainWindow", "Park Pos3"))
        self.posButton1.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton1.setText(_translate("MainWindow", "Park Pos1"))
        self.posButton0.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton0.setText(_translate("MainWindow", "Park Pos0"))
        self.posButton2.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton2.setText(_translate("MainWindow", "Park Pos2"))
        self.posButton7.setToolTip(_translate("MainWindow", "<html><head/><body><p>Slew the mount to the defined position.</p></body></html>"))
        self.posButton7.setText(_translate("MainWindow", "Park Pos7"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Tools), _translate("MainWindow", "Tools"))
        self.settingsTabWidget.setToolTip(_translate("MainWindow", "<html><head/><body><p>Configuration of settings</p></body></html>"))
        self.label_302.setText(_translate("MainWindow", "Oberservation Conditions"))
        self.label_49.setText(_translate("MainWindow", "Remote Device"))
        self.label_263.setText(_translate("MainWindow", "Core devices"))
        self.label_264.setText(_translate("MainWindow", "Accessories"))
        self.label_46.setText(_translate("MainWindow", "Relay Device"))
        self.sky.setText(_translate("MainWindow", "Sky Meter Device"))
        self.env.setText(_translate("MainWindow", "Environment Device"))
        self.label_48.setText(_translate("MainWindow", "Measure Device"))
        self.ccd.setText(_translate("MainWindow", "Imaging Device"))
        self.ast.setText(_translate("MainWindow", "Astrometry Device"))
        self.dome.setText(_translate("MainWindow", "Dome Device"))
        self.po.setText(_translate("MainWindow", "PowerBox Device"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.Devices), _translate("MainWindow", "Devices"))
        self.domePort.setText(_translate("MainWindow", "7624"))
        self.label_13.setText(_translate("MainWindow", "Device Name"))
        self.label_53.setText(_translate("MainWindow", "Device Name"))
        self.label_51.setText(_translate("MainWindow", "Device Name"))
        self.indiMessage.setToolTip(_translate("MainWindow", "<html><head/><body><p>Checked if you would like to enable blind solver function in SGPro</p></body></html>"))
        self.indiMessage.setText(_translate("MainWindow", "Show INDI messages in message window"))
        self.label_mountIP_6.setText(_translate("MainWindow", "Host IP/Port"))
        self.label_17.setText(_translate("MainWindow", "Imaging"))
        self.domeHost.setToolTip(_translate("MainWindow", "<html><head/><body><p>IP address of INDI server, ip address and host names!</p></body></html>"))
        self.domeHost.setText(_translate("MainWindow", "localhost"))
        self.imagingPort.setText(_translate("MainWindow", "7624"))
        self.label_23.setText(_translate("MainWindow", "Skymeter"))
        self.label_22.setText(_translate("MainWindow", "Dome"))
        self.label_36.setText(_translate("MainWindow", "Host IP/Port"))
        self.environPort.setText(_translate("MainWindow", "7624"))
        self.label_16.setText(_translate("MainWindow", "Environment"))
        self.label_110.setText(_translate("MainWindow", "Power / Dewheater"))
        self.skymeterPort.setText(_translate("MainWindow", "7624"))
        self.powerPort.setText(_translate("MainWindow", "7624"))
        self.label_66.setText(_translate("MainWindow", "Host IP/Port"))
        self.powerHost.setText(_translate("MainWindow", "localhost"))
        self.skymeterHost.setText(_translate("MainWindow", "localhost"))
        self.label_19.setText(_translate("MainWindow", "Device Name"))
        self.label_mountIP_3.setText(_translate("MainWindow", "Host IP/Port"))
        self.imagingHost.setToolTip(_translate("MainWindow", "<html><head/><body><p>IP address of INDI server, ip address and host names!</p></body></html>"))
        self.imagingHost.setText(_translate("MainWindow", "localhost"))
        self.environHost.setToolTip(_translate("MainWindow", "<html><head/><body><p>IP address of INDI server, ip address and host names!</p></body></html>"))
        self.environHost.setText(_translate("MainWindow", "localhost"))
        self.label_mountIP_7.setText(_translate("MainWindow", "Host IP/Port"))
        self.label_21.setText(_translate("MainWindow", "Device Name"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.INDI), _translate("MainWindow", "INDI"))
        self.label_294.setText(_translate("MainWindow", "s"))
        self.label_295.setText(_translate("MainWindow", "s"))
        self.label_296.setText(_translate("MainWindow", "%"))
        self.label_297.setText(_translate("MainWindow", "ISO Setting"))
        self.label_40.setText(_translate("MainWindow", "Mount Settle Time"))
        self.label_298.setText(_translate("MainWindow", "Imaging configuration"))
        self.label_41.setText(_translate("MainWindow", "Camera Binning (X/Y)"))
        self.isoSetting.setToolTip(_translate("MainWindow", "Sets the iso setting"))
        self.subFrame.setToolTip(_translate("MainWindow", "<html><head/><body><p>sets the percentage of frame size for subframe </p></body></html>"))
        self.expTime.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the exposure time</p></body></html>"))
        self.settleTimeMount.setToolTip(_translate("MainWindow", "Sets the time to wait after mount stop before imaging"))
        self.checkFastDownload.setToolTip(_translate("MainWindow", "Use Highspeed download of CCD camera if availabe"))
        self.checkFastDownload.setText(_translate("MainWindow", "Use fast download mode"))
        self.checkKeepImages.setToolTip(_translate("MainWindow", "Checked if you would like to keep the images of modeling session"))
        self.checkKeepImages.setText(_translate("MainWindow", "Keep model images after modeling"))
        self.label_299.setText(_translate("MainWindow", "Exposure Time of Camera"))
        self.binning.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the binning for the camera</p></body></html>"))
        self.label_307.setText(_translate("MainWindow", "Subframe of image"))
        self.label_166.setText(_translate("MainWindow", "CCD Pixel size (from driver)"))
        self.label_313.setText(_translate("MainWindow", "um"))
        self.pixelSize.setToolTip(_translate("MainWindow", "<html><head/><body><p>sets the percentage of frame size for subframe </p></body></html>"))
        self.label_100.setText(_translate("MainWindow", "Dome Settle Time"))
        self.settleTimeDome.setToolTip(_translate("MainWindow", "Sets the time to wait after mount stop before imaging"))
        self.label_314.setText(_translate("MainWindow", "s"))
        self.label_306.setText(_translate("MainWindow", "Astrometry configuration"))
        self.searchRadius.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the exposure time</p></body></html>"))
        self.label_315.setText(_translate("MainWindow", "Search radius"))
        self.label_316.setText(_translate("MainWindow", "Solving timeout"))
        self.solveTimeout.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the exposure time</p></body></html>"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.Imaging), _translate("MainWindow", "Imaging"))
        self.label_399.setText(_translate("MainWindow", "OpenWeatherMap API Key"))
        self.mountTypeConnection.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">MAC address of the mount</span></p></body></html>"))
        self.mountTypeConnection.setText(_translate("MainWindow", "-"))
        self.label_200.setText(_translate("MainWindow", "Mount connection"))
        self.mountHost.setToolTip(_translate("MainWindow", "<html><head/><body><p>IP address of mount, ip address and host names!</p></body></html>"))
        self.mountHost.setText(_translate("MainWindow", "127.0.0.1"))
        self.label_mountMAC.setText(_translate("MainWindow", "MAC Address"))
        self.label_mountIP.setText(_translate("MainWindow", "IP Address"))
        self.label_259.setText(_translate("MainWindow", "MAC Address"))
        self.bootRackComp.setText(_translate("MainWindow", "Boot / WOL"))
        self.mountMAC.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">MAC address of the mount</span></p></body></html>"))
        self.mountMAC.setText(_translate("MainWindow", "00:00:00:00:00:00"))
        self.label_195.setText(_translate("MainWindow", "Computer on Rig"))
        self.label_mountMAC_2.setText(_translate("MainWindow", "Connection is:"))
        self.rackCompMAC.setText(_translate("MainWindow", "00:00:00:00:00:00"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.MountRemote), _translate("MainWindow", "Mount / IP"))
        self.checkAutoDeletePoints.setToolTip(_translate("MainWindow", "<html><head/><body><p>Removes modeling points, which are below the horizon mask.</p></body></html>"))
        self.checkAutoDeletePoints.setText(_translate("MainWindow", "Auto delete points below horizon"))
        self.groupBoxCoordinateSystems_2.setTitle(_translate("MainWindow", "Auto sort points"))
        self.checkSortHL.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sorts the modeling points EAST / WEST side of pier.</p></body></html>"))
        self.checkSortHL.setText(_translate("MainWindow", "High / Low"))
        self.checkSortEW.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sorts the modeling points EAST / WEST side of pier.</p></body></html>"))
        self.checkSortEW.setText(_translate("MainWindow", "East / West"))
        self.checkSortNothing.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sorts the modeling points EAST / WEST side of pier.</p></body></html>"))
        self.checkSortNothing.setText(_translate("MainWindow", "Not Sorting"))
        self.saveHorizonMaskAs.setToolTip(_translate("MainWindow", "<html><head/><body><p>Saves the horizon mask in new file</p></body></html>"))
        self.saveHorizonMaskAs.setText(_translate("MainWindow", "Save as"))
        self.horizonFileName.setToolTip(_translate("MainWindow", "<html><head/><body><p>Name of Horizon Mask file, where the data is stored.</p><p>Please Double Click to Select.</p></body></html>"))
        self.saveHorizonMask.setToolTip(_translate("MainWindow", "<html><head/><body><p>Saves the horizon mask in file</p></body></html>"))
        self.saveHorizonMask.setText(_translate("MainWindow", "Save"))
        self.label_194.setText(_translate("MainWindow", "Horizon mask file"))
        self.loadHorizonMask.setToolTip(_translate("MainWindow", "<html><head/><body><p>Loads the horizon mask from given file</p></body></html>"))
        self.loadHorizonMask.setText(_translate("MainWindow", "Load"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.Horizon), _translate("MainWindow", "Horizon"))
        self.posAlt2.setInputMask(_translate("MainWindow", "99"))
        self.posAlt2.setText(_translate("MainWindow", "1"))
        self.posText2.setToolTip(_translate("MainWindow", "Button text park menu 3"))
        self.posText2.setText(_translate("MainWindow", "Dew"))
        self.posText0.setToolTip(_translate("MainWindow", "Button text park menu 1"))
        self.posText0.setText(_translate("MainWindow", "Flat West"))
        self.label_54.setText(_translate("MainWindow", "Configuration of fixed park positions "))
        self.posAz2.setInputMask(_translate("MainWindow", "999"))
        self.posAz2.setText(_translate("MainWindow", "270"))
        self.posText1.setToolTip(_translate("MainWindow", "Button text park menu 2"))
        self.posText1.setText(_translate("MainWindow", "Flat East"))
        self.posAz0.setInputMask(_translate("MainWindow", "999"))
        self.posAz0.setText(_translate("MainWindow", "1"))
        self.posAlt3.setInputMask(_translate("MainWindow", "99"))
        self.posAlt3.setText(_translate("MainWindow", "1"))
        self.posAlt0.setInputMask(_translate("MainWindow", "99"))
        self.posAlt0.setText(_translate("MainWindow", "89"))
        self.label_2.setText(_translate("MainWindow", "Button 1"))
        self.posAlt1.setInputMask(_translate("MainWindow", "99"))
        self.posAlt1.setText(_translate("MainWindow", "89"))
        self.label_50.setText(_translate("MainWindow", "Button 4"))
        self.label_44.setText(_translate("MainWindow", "Button 2"))
        self.posAz1.setInputMask(_translate("MainWindow", "999"))
        self.posAz1.setText(_translate("MainWindow", "359"))
        self.posAz3.setInputMask(_translate("MainWindow", "999"))
        self.posAz3.setText(_translate("MainWindow", "90"))
        self.posText3.setToolTip(_translate("MainWindow", "Button text park menu 4"))
        self.posText3.setText(_translate("MainWindow", "CCD"))
        self.label_47.setText(_translate("MainWindow", "Button 3"))
        self.posAz4.setInputMask(_translate("MainWindow", "999"))
        self.posAz4.setText(_translate("MainWindow", "270"))
        self.label_92.setText(_translate("MainWindow", "Button 0"))
        self.posAlt5.setInputMask(_translate("MainWindow", "99"))
        self.posAlt5.setText(_translate("MainWindow", "1"))
        self.label_94.setText(_translate("MainWindow", "Altitude"))
        self.label_95.setText(_translate("MainWindow", "Button 5"))
        self.posAlt4.setInputMask(_translate("MainWindow", "99"))
        self.posAlt4.setText(_translate("MainWindow", "1"))
        self.posText5.setToolTip(_translate("MainWindow", "Button text park menu 4"))
        self.posText5.setText(_translate("MainWindow", "CCD"))
        self.posText4.setToolTip(_translate("MainWindow", "Button text park menu 3"))
        self.posText4.setText(_translate("MainWindow", "Dew"))
        self.label_132.setText(_translate("MainWindow", "Azimuth"))
        self.posAz5.setInputMask(_translate("MainWindow", "999"))
        self.posAz5.setText(_translate("MainWindow", "90"))
        self.label_97.setText(_translate("MainWindow", "Text"))
        self.label_93.setText(_translate("MainWindow", "°"))
        self.label_131.setText(_translate("MainWindow", "°"))
        self.label_209.setText(_translate("MainWindow", "°"))
        self.label_243.setText(_translate("MainWindow", "°"))
        self.label_244.setText(_translate("MainWindow", "°"))
        self.label_245.setText(_translate("MainWindow", "°"))
        self.label_246.setText(_translate("MainWindow", "°"))
        self.label_247.setText(_translate("MainWindow", "°"))
        self.label_248.setText(_translate("MainWindow", "°"))
        self.label_249.setText(_translate("MainWindow", "°"))
        self.label_250.setText(_translate("MainWindow", "°"))
        self.label_251.setText(_translate("MainWindow", "°"))
        self.posAlt6.setInputMask(_translate("MainWindow", "99"))
        self.posAlt6.setText(_translate("MainWindow", "1"))
        self.label_257.setText(_translate("MainWindow", "°"))
        self.posAz6.setInputMask(_translate("MainWindow", "999"))
        self.posAz6.setText(_translate("MainWindow", "90"))
        self.posText6.setToolTip(_translate("MainWindow", "Button text park menu 4"))
        self.posText6.setText(_translate("MainWindow", "CCD"))
        self.label_101.setText(_translate("MainWindow", "Button 6"))
        self.label_258.setText(_translate("MainWindow", "°"))
        self.posAlt7.setInputMask(_translate("MainWindow", "99"))
        self.posAlt7.setText(_translate("MainWindow", "1"))
        self.label_261.setText(_translate("MainWindow", "°"))
        self.posAz7.setInputMask(_translate("MainWindow", "999"))
        self.posAz7.setText(_translate("MainWindow", "90"))
        self.posText7.setToolTip(_translate("MainWindow", "Button text park menu 4"))
        self.posText7.setText(_translate("MainWindow", "CCD"))
        self.label_103.setText(_translate("MainWindow", "Button 7"))
        self.label_262.setText(_translate("MainWindow", "°"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.ParkPos), _translate("MainWindow", "ParkPos"))
        self.label_129.setText(_translate("MainWindow", "3"))
        self.label_56.setText(_translate("MainWindow", "Relay"))
        self.label_198.setText(_translate("MainWindow", "Function"))
        self.label_199.setText(_translate("MainWindow", "7"))
        self.relayButtonText0.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relayButtonText0.setText(_translate("MainWindow", "free"))
        self.relayButtonText7.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relayButtonText7.setText(_translate("MainWindow", "free"))
        self.label_211.setText(_translate("MainWindow", "2"))
        self.relayButtonText5.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relayButtonText5.setText(_translate("MainWindow", "free"))
        self.label_220.setText(_translate("MainWindow", "4"))
        self.label_146.setText(_translate("MainWindow", "Relay button text setup"))
        self.label_255.setText(_translate("MainWindow", "8"))
        self.relayButtonText1.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relayButtonText1.setText(_translate("MainWindow", "free"))
        self.label_256.setText(_translate("MainWindow", "6"))
        self.relayButtonText6.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relayButtonText6.setText(_translate("MainWindow", "free"))
        self.label_149.setText(_translate("MainWindow", "5"))
        self.relayButtonText3.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relayButtonText3.setText(_translate("MainWindow", "free"))
        self.label_43.setText(_translate("MainWindow", "1"))
        self.relayButtonText2.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relayButtonText2.setText(_translate("MainWindow", "free"))
        self.label_45.setText(_translate("MainWindow", "Text"))
        self.relayButtonText4.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Button text for relay</span></p></body></html>"))
        self.relayButtonText4.setText(_translate("MainWindow", "free"))
        self.relayPassword.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Password of relay switch. Is not saved encrypted !</span></p></body></html>"))
        self.relayPassword.setText(_translate("MainWindow", "empty"))
        self.label_196.setText(_translate("MainWindow", "Username"))
        self.label_mountIP_2.setText(_translate("MainWindow", "IP Address"))
        self.label_197.setText(_translate("MainWindow", "Password"))
        self.label_141.setText(_translate("MainWindow", "Configuration of relay box (KMTronics Switch)"))
        self.relayUser.setToolTip(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:400;\">Username of relay switch. Is not saved encrypted !</span></p></body></html>"))
        self.relayUser.setText(_translate("MainWindow", "empty"))
        self.relayHost.setToolTip(_translate("MainWindow", "<html><head/><body><p>IP address of relay switch, ip address and host names!</p></body></html>"))
        self.relayHost.setText(_translate("MainWindow", "127.0.0.1"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.KMTronic), _translate("MainWindow", "KMTronic"))
        self.soundMountSlewFinished.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the audio signal to be played when mount finished slewing</p></body></html>"))
        self.label_210.setText(_translate("MainWindow", "Play audio signals"))
        self.soundMountAlert.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the audio signal to be played when mount stopped in emergency case</p></body></html>"))
        self.soundDomeSlewFinished.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the audio signal to be played when dome finished slewing</p></body></html>"))
        self.label_212.setText(_translate("MainWindow", "Mount slew finished"))
        self.label_214.setText(_translate("MainWindow", "Dome slew finished"))
        self.label_215.setText(_translate("MainWindow", "Mount alert for STOP"))
        self.label_217.setText(_translate("MainWindow", "Modeling finished"))
        self.soundModelingFinished.setToolTip(_translate("MainWindow", "<html><head/><body><p>Sets the audio signal to be played when mount stopped in emergency case</p></body></html>"))
        self.logLevelMW.setTitle(_translate("MainWindow", "MountWizzard"))
        self.loglevelDebug.setToolTip(_translate("MainWindow", "<html><head/><body><p>Choose local solver</p></body></html>"))
        self.loglevelDebug.setText(_translate("MainWindow", "Debug"))
        self.loglevelInfo.setToolTip(_translate("MainWindow", "<html><head/><body><p>Choose online solver nova.astrometry.net as solver</p></body></html>"))
        self.loglevelInfo.setText(_translate("MainWindow", "Info"))
        self.loglevelWarning.setToolTip(_translate("MainWindow", "<html><head/><body><p>Choose online solver nova.astrometry.net as solver</p></body></html>"))
        self.loglevelWarning.setText(_translate("MainWindow", "Warning"))
        self.loglevelError.setToolTip(_translate("MainWindow", "<html><head/><body><p>Choose online solver nova.astrometry.net as solver</p></body></html>"))
        self.loglevelError.setText(_translate("MainWindow", "Error"))
        self.label_59.setText(_translate("MainWindow", "Log Levels"))
        self.btn_resetWindowSizes.setToolTip(_translate("MainWindow", "<html><head/><body><p>Opens the image window</p></body></html>"))
        self.btn_resetWindowSizes.setText(_translate("MainWindow", "Collect Windows"))
        self.label_62.setText(_translate("MainWindow", "Internet Online Connection"))
        self.label_401.setText(_translate("MainWindow", "Firmware data"))
        self.firmwareProductName_3.setText(_translate("MainWindow", "FW:"))
        self.numberString.setText(_translate("MainWindow", "-"))
        self.firmwareProductName_4.setText(_translate("MainWindow", "Date:"))
        self.fwtime.setText(_translate("MainWindow", "-"))
        self.firmwareProductName_5.setText(_translate("MainWindow", "Time:"))
        self.productName.setText(_translate("MainWindow", "-"))
        self.firmwareProductName_2.setText(_translate("MainWindow", "Product:"))
        self.hwVersion.setText(_translate("MainWindow", "-"))
        self.fwdate.setText(_translate("MainWindow", "-"))
        self.Hardware.setText(_translate("MainWindow", "HW:"))
        self.isOnline.setToolTip(_translate("MainWindow", "<html><head/><body><p>Choose online solver nova.astrometry.net as solver</p></body></html>"))
        self.isOnline.setText(_translate("MainWindow", "online and update data files if necessary"))
        self.enableNoise.setToolTip(_translate("MainWindow", "<html><head/><body><p>Choose online solver nova.astrometry.net as solver</p></body></html>"))
        self.enableNoise.setText(_translate("MainWindow", "add random error for astrometry solving"))
        self.groupBox.setTitle(_translate("MainWindow", "MountWizzard4 (C) 2019 MWORION"))
        self.le_firmwareProductName_6.setText(_translate("MainWindow", "qimage2ndarray"))
        self.le_firmwareProductName_3.setText(_translate("MainWindow", "open source software embedded: "))
        self.le_firmwareProductName_4.setText(_translate("MainWindow", "pyqt5, matplotlib, wakeonlan,"))
        self.le_firmwareProductName_5.setText(_translate("MainWindow", "numpy, requests, astropy, skyfield,"))
        self.settingsTabWidget.setTabText(self.settingsTabWidget.indexOf(self.AudioMisc), _translate("MainWindow", "Audio / Misc"))
        self.mainTabWidget.setTabText(self.mainTabWidget.indexOf(self.Setting), _translate("MainWindow", "Sett. / Conf."))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QWidget()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())
